Package: verifyr2
Type: Package
Title: Compare and Verify File Contents
Description: Extendable 'R6' file comparison classes, including a 'shiny' app for combining the comparison functionality into a file comparison application. The package idea originates from pharma companies' drug development processes, where statisticians and statistical programmers need to review and compare different versions of the same outputs and datasets. The package implementation itself is not tied to any specific industry and can be used in any context for easy file comparisons between different file version sets.
Version: 1.2.0
License: MIT + file LICENSE
Authors@R: c(
    person("Juha", "Javanainen", email = "ejuhjav@gmail.com", role = c("aut", "cre")),
    person("Tiina", "Kirsilä", email = "tiina.kirsila@gmail.com", role = "ctb"),
    person("Anna", "Wiksten", email = "a.s.wiksten@gmail.com", role = "ctb"))
Depends: R (>= 4.1.0)
Imports: base64enc, diffobj, dplyr, jsonlite, magrittr, mime, R6,
        rappdirs, shiny, stringr, striprtf, tibble
Suggests: DT, fs, magick, pkgload, pdftools, purrr, rmarkdown, shinyjs,
        shinyFiles, shinytest2, testthat (>= 3.0.0)
Config/testthat/edition: 3
Encoding: UTF-8
RoxygenNote: 7.3.3
Collate: 'FileComparator.R' 'BinaryFileComparator.R' 'Config.R'
        'Debugger.R' 'TxtFileComparator.R' 'FileComparatorFactory.R'
        'ImgFileComparator.R' 'PdfFileComparator.R'
        'TxtWithImagesComparator.R' 'RtfFileComparator.R'
        'list_files.R' 'list_folder_files.R' 'run_example.R'
        'utils-pipe.R'
NeedsCompilation: no
Packaged: 2026-05-23 10:48:38 UTC; juha
Author: Juha Javanainen [aut, cre],
  Tiina Kirsilä [ctb],
  Anna Wiksten [ctb]
Maintainer: Juha Javanainen <ejuhjav@gmail.com>
Repository: CRAN
Date/Publication: 2026-05-23 11:30:02 UTC
Built: R 4.4.3; ; 2026-05-24 23:52:57 UTC; unix

Package: verifyr2
Type: Package
Title: Compare and Verify File Contents
Description: Extendable 'R6' file comparison classes, including a 'shiny' app for combining the comparison functionality into a file comparison application. The package idea originates from pharma companies' drug development processes, where statisticians and statistical programmers need to review and compare different versions of the same outputs and datasets. The package implementation itself is not tied to any specific industry and can be used in any context for easy file comparisons between different file version sets.
Version: 1.2.0
License: MIT + file LICENSE
Authors@R: c(
    person("Juha", "Javanainen", email = "ejuhjav@gmail.com", role = c("aut", "cre")),
    person("Tiina", "Kirsilä", email = "tiina.kirsila@gmail.com", role = "ctb"),
    person("Anna", "Wiksten", email = "a.s.wiksten@gmail.com", role = "ctb"))
Depends: R (>= 4.1.0)
Imports: base64enc, diffobj, dplyr, jsonlite, magrittr, mime, R6,
        rappdirs, shiny, stringr, striprtf, tibble
Suggests: DT, fs, magick, pkgload, pdftools, purrr, rmarkdown, shinyjs,
        shinyFiles, shinytest2, testthat (>= 3.0.0)
Config/testthat/edition: 3
Encoding: UTF-8
RoxygenNote: 7.3.3
Collate: 'FileComparator.R' 'BinaryFileComparator.R' 'Config.R'
        'Debugger.R' 'TxtFileComparator.R' 'FileComparatorFactory.R'
        'ImgFileComparator.R' 'PdfFileComparator.R'
        'TxtWithImagesComparator.R' 'RtfFileComparator.R'
        'list_files.R' 'list_folder_files.R' 'run_example.R'
        'utils-pipe.R'
NeedsCompilation: no
Packaged: 2026-05-23 10:48:38 UTC; juha
Author: Juha Javanainen [aut, cre],
  Tiina Kirsilä [ctb],
  Anna Wiksten [ctb]
Maintainer: Juha Javanainen <ejuhjav@gmail.com>
Repository: CRAN
Date/Publication: 2026-05-23 11:30:02 UTC
Built: R 4.6.0; ; 2026-05-23 15:11:15 UTC; unix

# Generated by roxygen2: do not edit by hand

export("%>%")
export(BinaryFileComparator)
export(Config)
export(Debugger)
export(FileComparator)
export(ImgFileComparator)
export(PdfFileComparator)
export(RtfFileComparator)
export(TxtFileComparator)
export(XlsxFileComparator)
export(create_comparator)
export(list_files)
export(list_folder_files)
export(run_example)
import(base64enc)
import(htmltools)
import(jsonlite)
import(rappdirs)
import(stringr)
import(striprtf)
importFrom(R6,R6Class)
importFrom(magrittr,"%>%")

# verifyr2<img src="./man/figures/verifyr2.png" align="right" height="223" />

<!-- badges: start -->
[![CRAN status](https://www.r-pkg.org/badges/version/verifyr2)](https://cran.r-project.org/package=verifyr2)
![Coverage](https://github.com/ejuhjav/verifyr2/actions/workflows/check-package.yml/badge.svg)
[![Coverage Status](https://ejuhjav.github.io/verifyr2/coverage.svg)](https://ejuhjav.github.io/verifyr2/coverage/)
<!-- badges: end -->

`verifyr2` is an R package for extendable file comparison classes and methods including an
example `shiny` app for combining the functions into a convenient application.

## Package installation

The project is available in cran and can be installed with the following command:

``` bash
> install.packages("verifyr2")
```

### Development version

The latest development version can be installed from github using different install packages
that work with github projects. One such option would be to use the `devtools` package:

``` bash
> install.packages("devtools")
> library("devtools")
> install_github("ejuhjav/verifyr2")
```

### Extended functionality

Following additional libraries can be installed to extend the provided comparison functionality:

``` bash
> # image comparison functionality
> install.packages("magick")
>
> # pdf comparison functionality
> install.packages("pdftools")
```

## Shiny app usage

Launching the `shiny` app requires that you have the following additional libraries installed as
documented in the suggestion list; `shinyjs`, `shinyFiles`, and `DT`. After all the necessary
libraries are available, the included `shiny` app can be launched with the following command:

``` bash
> verifyr2::run_example()
```

## Creating and calling the separators manually

``` bash
> config <- Config$new()
> comparator <- verifyr2::create_comparator(file1, file2)
> comparator$vrf_summary(config = config)
> comparator$vrf_details(config = config)
```

instantiating an explicit comparator manually

``` bash
> config <- Config$new()
> comparator <- RtfFileComparator$new(file1, file2)
> comparator$vrf_summary(config = config)
> comparator$vrf_details(config = config)
```

## Optional features

Image comparison features - comparing single images as well as supported embedded images in file
contents - requires additional `magick` library to be installed. This is defined as an optional
requirement for the `verifyr2` package due to the additional OS level requirements of the package.
For more details on the package installation, see [magick package](https://github.com/ropensci/magick).

PDF comparison features - reading PDF file contents as text for details comparison - requires
additional `pdftools` library to be installed. This is defined as an optional requirement for the
`verifyr2` package due to the additional OS level requirements of the package. For more details on
the package installation, see [pdftools package](https://github.com/ropensci/pdftools).

## Adding support to additional file types

The file comparison logic is implemented with a generic structure that supports easy addition of
new comparison logic for different file types. In most cases, this will simply require an implementation
of a new file comparator with the file extension name and implementing the file contents getter. The included
simplified implementation for pdf file comparison - without testing pdftools availability and excluding the
logging parts - is an example of how to do this (see source file to see the full implementation with
documentation).

When using the provided generic `create_comparator` function along with your custom comparator class implementation, 
note that the comparator class name prefix must match with the compared file type. 

``` bash
#' PdfFileComparator.R
#'
#' @include TxtFileComparator.R
#'
#' @export
#'
PdfFileComparator <- R6::R6Class(
  "PdfFileComparator",
  inherit = TxtFileComparator,
  public = list(
    vrf_contents = function(file, config, omit) {
      content <- pdftools::pdf_text(file)
      content <- paste(content, collapse = "")
      content <- strsplit(content, "\n")[[1]]

      self$vrf_contents_inner(content, omit, options)
    }
  )
)

```

## Contributing

Contributions to `verifyr2` are dynamic and welcome! Whether you are fixing a typo, reporting a bug, or building a brand-new file comparator, we appreciate your help.

Please see our [Contributing Guide](https://github.com/ejuhjav/verifyr2/blob/main/.github/CONTRIBUTING.md) for details on our code style, test requirements, and how to set up your local R development environment.

## Credits

`verifyr2` package implementation is based on the concept of the [verifyr package](https://github.com/novartis/verifyr).

# verifyr2<img src="./man/figures/verifyr2.png" align="right" height="223" />

<!-- badges: start -->
[![CRAN status](https://www.r-pkg.org/badges/version/verifyr2)](https://cran.r-project.org/package=verifyr2)
![Coverage](https://github.com/ejuhjav/verifyr2/actions/workflows/check-package.yml/badge.svg)
[![Coverage Status](https://ejuhjav.github.io/verifyr2/coverage.svg)](https://ejuhjav.github.io/verifyr2/coverage/)
<!-- badges: end -->

Verifyr2 is an R package for extendable file comparison classes and methods including an
example 'shiny' app for combining the functions into a convenient application.

## Package installation

The project is available in cran and can be installed with the following command:

``` bash
> install.packages("verifyr2")
```

### Development version

The latest development version can be installed from github using different install packages
that work with github projects. One such option would be to use the 'devtools' package:

``` bash
> install.packages("devtools")
> library("devtools")
> install_github("ejuhjav/verifyr2")
```

### Extended functionality

Following additional libraries can be installed to extend the provided comparison functionality:

``` bash
> # image comparison functionality
> install.packages("magick")
>
> # pdf comparison functionality
> install.packages("pdftools")
```

## Shiny app usage

Launching the 'shiny' app requires that you have the following additional libraries installed as
documented in the suggestion list; 'shinyjs', 'shinyFiles', and 'DT'. After all the necessary
libraries are available, the included 'shiny' app can be launched with the following command:

``` bash
> verifyr2::run_example()
```

## Creating and calling the separators manually

``` bash
> config <- Config$new()
> comparator <- verifyr2::create_comparator(file1, file2)
> comparator$vrf_summary(config = config)
> comparator$vrf_details(config = config)
```

instantiating an explicit comparator manually

``` bash
> config <- Config$new()
> comparator <- RtfFileComparator$new(file1, file2)
> comparator$vrf_summary(config = config)
> comparator$vrf_details(config = config)
```

## Optional features

Image comparison features - comparing single images as well as supported embedded images in file
contents - requires additional 'magick' library to be installed. This is defined as an optional
requirement for the 'verifyr2' package due to the additional OS level requirements of the package.
For more details on the package installation, see [magick package](https://github.com/ropensci/magick).

PDF comparison features - reading PDF file contents as text for details comparison - requires
additional 'pdftools' library to be installed. This is defined as an optional requirement for the
'verifyr2' package due to the additional OS level requirements of the package. For more details on
the package installation, see [pdftools package](https://github.com/ropensci/pdftools).

## Adding support to additional file types

The file comparison logic is implemented with a generic structure that supports easy addition of
new comparison logic for different file types. In most cases, this will simply require an implementation
of a new file comparator with the file extension name and implementing the file contents getter. The included
simplified implementation for pdf file comparison - without testing pdftools availability and excluding the
logging parts - is an example of how to do this (see source file to see the full implementation with
documentation).

When using the provided generic create_comparator function along with your custom comparator class implementation, 
note that the comparator class name prefix must match with the compared file type. 

``` bash
#' PdfFileComparator.R
#'
#' @include TxtFileComparator.R
#'
#' @export
#'
PdfFileComparator <- R6::R6Class(
  "PdfFileComparator",
  inherit = TxtFileComparator,
  public = list(
    vrf_contents = function(file, config, omit) {
      content <- pdftools::pdf_text(file)
      content <- paste(content, collapse = "")
      content <- strsplit(content, "\n")[[1]]

      self$vrf_contents_inner(content, omit, options)
    }
  )
)

```

## Credits

Verifyr2 implementation is based on the concept of the [verifyr package](https://github.com/novartis/verifyr).

#' Call for shiny example where the user can test verifyr2 package functions
#'
#' @include Config.R
#'
#' \code{verifyr2::run_example} returns simple Shiny App where user can see how
#' the verifyr2 functions work

if (!"verifyr2" %in% loadedNamespaces()) {
  library(verifyr2)
}

# initialize the configuration and possible debug override option
config <- verifyr2::Config$new()
debug  <- getOption("verifyr2.debug", default = FALSE)

if (isTRUE(debug)) {
  config$set("generic.debug", "yes")
}

# ==============================================================================
# Custom input functions
# ==============================================================================

vrf_folder_input <- function(input_id, label, value = "", ...) {
  label_class  <- "control-label"
  input_class  <- "shiny-input-text form-control"
  folder_value <- system.file(value, package = "verifyr2")

  shiny::div(class = "form-group form-group-custom shiny-input-container",
    shinyFiles::shinyDirButton(
      paste0(input_id, "_select"),
      NULL,
      "Select folder",
      icon = shiny::icon("folder-open")
    ),

    shiny::tags$label(
      label,
      `for` = input_id,
      class = label_class
    ),

    shiny::tags$input(
      id    = input_id,
      type  = "text",
      class = input_class,
      value = folder_value,
      ...
    ),
  )
}

vrf_file_input <- function(input_id, label, value = "", ...) {
  label_class <- "control-label"
  input_class <- "shiny-input-text form-control"
  file_value  <- system.file(value, package = "verifyr2")

  shiny::div(class = "form-group form-group-custom shiny-input-container",
    shinyFiles::shinyFilesButton(
      paste0(input_id, "_select"),
      NULL,
      "Select file",
      FALSE,
      icon = shiny::icon("folder-open")
    ),

    shiny::tags$label(
      label,
      `for` = input_id,
      class = label_class
    ),

    shiny::tags$input(
      id    = input_id,
      type  = "text",
      class = input_class,
      value = file_value,
      ...
    ),
  )
}

# ==============================================================================
# Interface definition
# ==============================================================================

javascript_additions <- function() {
  shiny::tags$head(
    shiny::tags$script(htmltools::HTML("
      Shiny.addCustomMessageHandler('highlightRow', function(message) {
        $('.dataTable tr').removeClass('row_highlighted');
        $('.dataTable #process_' + message.row_id)
          .closest('tr')
          .addClass('row_highlighted');
      });
    "))
  )
}

page_icon <- function() {
  shiny::tags$head(
    shiny::tags$link(rel = "icon", type = "image/x-icon", href = "verifyr2.ico")
  )
}

search_container <- function() {
  shiny::div(
    shiny::headerPanel("File content comparison"),
    shiny::wellPanel(
      shiny::tabsetPanel(id = "compare_tabs",
        shiny::tabPanel("Compare folder contents", value = "tabs_folder",
          shiny::fluidRow(
            shiny::column(6,
              vrf_folder_input(
                "folder1",
                "Folder 1",
                "/extdata/base_files"
              ),

              vrf_folder_input(
                "folder2",
                "Folder 2",
                "/extdata/compare_files"
              ),
            ),
            shiny::column(6,
              shiny::textInput(
                "omit_rows",
                "Omit rows with text",
                "versicolor"
              ),

              shiny::textInput(
                "file_name_pattern",
                "File Name Pattern"
              ),
            ),
          ),
        ),
        shiny::tabPanel("Compare specific files", value = "tabs_file",
          shiny::fluidRow(
            shiny::column(6,
              vrf_file_input(
                "file1",
                "File 1",
                "/extdata/base_files/file2_additional_rows.rtf"
              ),

              vrf_file_input(
                "file2",
                "File 2",
                "/extdata/compare_files/file3_changed_rows.rtf"
              ),
            ),
            shiny::column(6,
              shiny::textInput(
                "omit_file_rows",
                "Omit rows with text",
                "versicolor"
              ),
            ),
          ),
        ),
      ),
      shiny::fluidRow(
        shiny::column(12,
          shiny::actionButton("go", "Go"),
          shiny::actionButton("configure", "Configure"),
        ),
      ),
    ),
  )
}

summary_container <- function() {
  shiny::column(12,
    shiny::h2("Summary comparison (vrf_summary):"),
    DT::dataTableOutput("summary_out"),
    shiny::textOutput("summary_text_output"),
    shiny::downloadButton(
      "download_csv",
      "Download Comparison Results as CSV"
    ),
    shiny::downloadButton(
      "download_html",
      "Download Comparison Report as HTML"
    ),
  )
}

details_container <- function() {
  shiny::column(12,
    shiny::h2("Details comparison (vrf_details):"),
    shiny::fluidRow(
      shiny::column(6,
        shiny::downloadLink(
          "file1_link",
          shiny::textOutput("file1_link_output")
        ),
      ),
      shiny::column(6,
        shiny::downloadLink(
          "file2_link",
          shiny::textOutput("file2_link_output")
        ),
      ),
    ),
    shiny::tabsetPanel(id = "details_tabs",
      shiny::tabPanel("Summary contents", value = "tabs_details_summary",
        shiny::htmlOutput("details_out_summary"),
      ),
      shiny::tabPanel("Full file contents", value = "tabs_details_full",
        shiny::htmlOutput("details_out_full"),
      ),
    ),
    shiny::htmlOutput("details_out_generic"),
    shiny::textOutput("details_text_output"),
    shiny::fluidRow(
      id = "comparison_comments_container",
      shiny::column(12,
        shiny::textAreaInput(
          "details_out_comments",
          "Comments",
          width = "100%"
        ),

        shiny::actionButton("save_comments", "Save comments"),
        shiny::actionButton("clear_comments", "Clear comments"),
      ),
    ),
  )
}

ui <- shiny::fluidPage(
  shinyjs::useShinyjs(),
  shiny::includeCSS("styles.css"),
  javascript_additions(),
  page_icon(),
  search_container(),
  shiny::fluidRow(
    summary_container(),
    details_container(),
  ),
)

# ==============================================================================
# Server helper functions
# ==============================================================================

update_download_links <- function(output, row, file1_link, file2_link) {
  file1_link("")
  file2_link("")

  if (!is.na(row$file1)) {
    filename <- basename(row$file1)
    file1_link(paste0("Open ", row$file1))
    output$file1_link <- shiny::downloadHandler(
      filename = function() {
        paste0("old_", filename)
      },
      content = function(file) {
        file.copy(paste0(row$file1), file)
      }
    )
  }

  if (!is.na(row$file2)) {
    filename <- basename(row$file2)
    file2_link(paste0("Open ", row$file2))
    output$file2_link <- shiny::downloadHandler(
      filename = function() {
        paste0("new_", filename)
      },
      content = function(file) {
        file.copy(paste0(row$file2), file)
      }
    )
  }
}

# Helper method for getting the comparator related to specific row index
# or creating a new comparator instance if one is not found from local
# storage.
get_comparator <- function(row_index, file1, file2, comparators_rv) {
  row_index_str <- as.character(row_index)
  cache <- comparators_rv()

  if (row_index_str %in% names(cache)) {
    return(cache[[row_index_str]])
  }

  comparator <- verifyr2::create_comparator(file1, file2)
  cache[[row_index_str]] <- comparator
  comparators_rv(cache)

  comparator
}

update_details_comparison <- function(
  input,
  output,
  session,
  config_param,
  row,
  row_index,
  mode,
  omit,
  comparators_rv
) {
  file1 <- paste0(row[1])
  file2 <- paste0(row[2])

  options <- config_param$clone(deep = TRUE)
  options$set("details.mode", mode)

  # empty and hide the display elements by default before redrawing the contents
  set_visibility("details_tabs", FALSE)

  output$details_out_summary <- shiny::renderUI({
    shiny::HTML("")
  })

  output$details_out_full <- shiny::renderUI({
    shiny::HTML("")
  })

  output$details_out_generic <- shiny::renderUI({
    shiny::HTML("")
  })

  shiny::withProgress(
    message = "Processing comparison details...",
    value = 0,
    {
      comparator <- get_comparator(row_index, file1, file2, comparators_rv)
      details <- comparator$vrf_details(
        omit   = omit,
        config = options
      )

      shiny::incProgress(1)
    }
  )

  lapply(seq_along(details), function(index) {
    instance_data <- details[[index]]

    if ("text" == instance_data$type) {
      set_visibility("details_tabs", TRUE)

      if ("full" == mode) {
        output$details_out_full <- shiny::renderUI({
          shiny::HTML(
            as.character(
              instance_data$content
            )
          )
        })
      } else {
        output$details_out_summary <- shiny::renderUI({
          shiny::HTML(
            as.character(
              instance_data$content
            )
          )
        })
      }
    }
  })

  output$details_out_generic <- shiny::renderUI({
    lapply(seq_along(details), function(index) {
      instance_data <- details[[index]]

      if ("image" == instance_data$type) {
        shiny::tags$div(
          style = "padding: 9.5px;",
          class = "custom-img-diffobj-wrapper",
          shiny::tags$div(
            style = "display: flex;",
            class = "custom-img-diffobj-container top",
            shiny::tags$div(
              style = "display: inline-block; flex: 0 0 33.3333%;",
              shiny::tags$div(
                class = "custom-img-diffobj-image custom-img-diffobj-image-1",
                shiny::tags$span("Image version 1")
              )
            ),
            shiny::tags$div(
              style = "display: inline-block; flex: 0 0 33.3333%;",
              shiny::tags$div(
                class = "custom-img-diffobj-image custom-img-diffobj-image-2",
                shiny::tags$span("Image version 2")
              )
            ),
            shiny::tags$div(
              style = "display: inline-block; flex: 0 0 33.3333%;",
              shiny::tags$div(
                class = "custom-img-diffobj-image custom-img-diffobj-image-3",
                shiny::tags$span("Image difference")
              )
            )
          ),
          shiny::tags$div(
            style = "display: flex;",
            class = "custom-img-diffobj-container bottom",
            shiny::tags$div(
              style = "display: inline-block; flex: 0 0 33.3333%;",
              shiny::tags$div(
                class = "custom-img-diffobj-image-display",
                shiny::tags$img(
                  src   = instance_data$content$image1,
                  alt   = "Image1",
                  style = "width: 100%;"
                )
              )
            ),
            shiny::tags$div(
              style = "display: inline-block; flex: 0 0 33.3333%;",
              shiny::tags$div(
                class = "custom-img-diffobj-image-display",
                shiny::tags$img(
                  src   = instance_data$content$image2,
                  alt   = "Image2",
                  style = "width: 100%;"
                )
              )
            ),
            shiny::tags$div(
              style = "display: inline-block; flex: 0 0 33.3333%;",
              shiny::tags$div(
                class = "custom-img-diffobj-image-display",
                if (!is.null(instance_data$content$image3)) {
                  shiny::tags$img(
                    src   = instance_data$content$image3,
                    alt   = "Difference Image",
                    style = "width: 100%;"
                  )
                } else {
                  shiny::tags$div(
                    style = paste0(
                      "background-color: #fff;",
                      "justify-content: center;",
                      "align-items: center;",
                      "display: flex;",
                      "height: 100%;"
                    ),
                    shiny::tags$div(
                      style = "text-align: center",
                      "No differences"
                    )
                  )
                }
              )
            )
          )
        )
      }
    })
  })
}

update_folder_selections <- function(input, session, roots) {
  if (!is.integer(input$folder1_select)) {
    shiny::updateTextInput(session, "folder1",
      NULL,
      shinyFiles::parseDirPath(roots, input$folder1_select)
    )
  }

  if (!is.integer(input$folder2_select)) {
    shiny::updateTextInput(session, "folder2",
      NULL,
      shinyFiles::parseDirPath(roots, input$folder2_select)
    )
  }
}

update_file_selections <- function(input, session, roots) {
  if (!is.integer(input$file1_select)) {
    shiny::updateTextInput(session, "file1",
      NULL,
      paste0(shinyFiles::parseFilePaths(roots, input$file1_select)$datapath)
    )
  }

  if (!is.integer(input$file2_select)) {
    shiny::updateTextInput(session, "file2",
      NULL,
      paste0(shinyFiles::parseFilePaths(roots, input$file2_select)$datapath)
    )
  }
}

set_visibility <- function(id, visible) {
  if (visible) {
    del <- "'custom_hiddon'"
    add <- "'custom_visible'"
  } else {
    del <- "'custom_visible'"
    add <- "'custom_hidden'"
  }

  element <- paste0("$('#", id, "')")
  shinyjs::runjs(paste0(element, ".removeClass(", del, ").addClass(", add, ")"))
}

set_reactive_text <- function(reactive_id, text, class = "") {
  do.call(reactive_id, list(text))
}

# ==============================================================================
# Server definition
# ==============================================================================

server <- function(input, output, session) {

  # ============================================================================
  # Element initializations
  # ============================================================================

  roots  <- c(
    Home = fs::path_home(),
    Examples = fs::path_package("verifyr2", "extdata")
  )

  params <- list(
    roots = roots,
    session = session,
    restrictions = system.file(package = "base"),
    allowDirCreate = FALSE
  )

  do.call(shinyFiles::shinyDirChoose, c(list(input, "folder1_select"), params))
  do.call(shinyFiles::shinyDirChoose, c(list(input, "folder2_select"), params))

  params <- list(
    roots = roots,
    session = session,
    restrictions = system.file(package = "base")
  )

  do.call(shinyFiles::shinyFileChoose, c(list(input, "file1_select"), params))
  do.call(shinyFiles::shinyFileChoose, c(list(input, "file2_select"), params))

  default1 <- paste0(
    "Select the compared file folders and execute the summary",
    "comparison by clicking on the 'Go' button."
  )

  default2 <- paste0(
    "Click on a row in the summary comparison result to view",
    "the side-by-side details comparison."
  )

  list_of_files   <- shiny::reactiveVal(NULL)
  summary_text    <- shiny::reactiveVal(default1)
  details_text    <- shiny::reactiveVal(default2)
  file1_link      <- shiny::reactiveVal("")
  file2_link      <- shiny::reactiveVal("")
  prev_comments   <- shiny::reactiveVal(c())
  reprocess       <- shiny::reactiveVal(0)
  comparators_rv  <- shiny::reactiveVal(list())
  row_index_rv    <- shiny::reactiveVal(NULL)
  current_mode_rv <- shiny::reactiveVal(NULL)
  current_omit_rv <- shiny::reactiveVal(NULL)
  dt_data_rv      <- shiny::reactiveVal(NULL)

  dt_proxy <- DT::dataTableProxy("summary_out")

  output$summary_text_output <- shiny::renderText(summary_text())
  output$details_text_output <- shiny::renderText(details_text())

  output$file1_link_output <- shiny::renderText(file1_link())
  output$file2_link_output <- shiny::renderText(file2_link())

  output$download_csv <- shiny::downloadHandler(
    filename = function() {
      paste0(
        "verifyr2_comparison_",
        format(Sys.time(), "%Y%m%d_%H%M%S"),
        ".csv"
      )
    },
    content = function(file) {
      col_exclude <- c("comments", "process_button")
      dt_current  <- dt_data_rv()
      dt_subset   <- dt_current[, !(names(dt_current) %in% col_exclude)]
      prev_comments(dt_current[["comments_details"]])
      write.csv(dt_subset, file, row.names = FALSE)
    }
  )

  # Enable/disable HTML report button based on required package availability
  shiny::observe({
    has_deps <- requireNamespace("rmarkdown", quietly = TRUE)
    shinyjs::toggleState("download_html", condition = has_deps)
    if (!has_deps) {
      shinyjs::runjs(paste0(
        "$('#download_html').attr('title', 'Missing packages: rmarkdown');"
      ))
    }
  })

  output$download_html <- shiny::downloadHandler(
    filename = function() {
      paste0(
        "Verifyr2_Comparison_Report_",
        format(Sys.time(), "%Y%m%d_%H%M%S"),
        ".html"
      )
    },
    content = function(file) {
      dt <- dt_data_rv()
      if (is.null(dt)) {
        writeLines("<html><body><p>No data available.</p></body></html>", file)
        return()
      }

      version  <- as.character(utils::packageVersion("verifyr2"))
      tmp_dir  <- tempdir()
      tmp_rmd  <- file.path(tmp_dir, "report_template.Rmd")
      tmp_file <- file.path(tmp_dir, "report_output.html")

      # Files that need to be copied to tmp dir for HTML export generation
      files_to_copy <- c(
        file.path(getwd(), "report_template.Rmd"),
        file.path(getwd(), "report_styles.css"),
        file.path(getwd(), "www", "verifyr2.ico")
      )

      file.copy(
        from = files_to_copy,
        to   = file.path(tmp_dir, basename(files_to_copy)),
        overwrite = TRUE
      )

      shiny::withProgress(message = "Generating HTML report...", value = 0, {
        tryCatch({
          rmarkdown::render(
            input  = tmp_rmd,
            output_file = tmp_file,
            params = list(
              dt       = dt,
              omit_val = current_omit_rv(),
              version  = version
            ),
            envir = new.env(parent = globalenv())
          )
          prev_comments(dt[["comments_details"]])
          file.copy(tmp_file, file, overwrite = TRUE)
        }, error = function(e) {
          msg <- conditionMessage(e)
          writeLines(paste0(
            "<html><body>",
            "<h2 style='color:red;'>Report generation failed</h2>",
            "<pre>", htmltools::htmlEscape(msg), "</pre>",
            "</body></html>"
          ), file)
        })
        shiny::incProgress(1)
      })
    }
  )

  output$summary_out <- DT::renderDataTable({
    shiny::req(summary_verify())
    dt <- summary_verify()

    options <- list(
      columnDefs = list(
        list(visible = FALSE, targets = c("comments_details")),
        list(orderable = FALSE, targets = c("process_button"))
      )
    )

    colnames <- c(
      names(dt)[-ncol(dt)],
      "Actions"
    )

    DT::datatable(
      dt,
      selection = "none",
      escape    = FALSE,
      options   = options,
      colnames  = colnames
    )
  })

  # ============================================================================
  # Reactive elements and observe triggers
  # ============================================================================

  shiny::observeEvent(input$go, {
    check_comment_changes(
      input,
      prev_comments,
      dt_data_rv,
      on_confirm = function() {
        comparators_rv(list())
        result <- list_files(input, summary_text, current_omit_rv)
        list_of_files(result)
      }
    )
  })

  shiny::observeEvent(input$details_tabs, {
    if (!is.null(current_mode_rv())) {
      mode <- sub("^tabs_details_", "", input$details_tabs)

      if (mode != current_mode_rv()) {
        current_mode_rv(mode)

        new_row_index <- input$process_row
        row <- summary_verify()[new_row_index, ]

        update_details_comparison(
          input,
          output,
          session,
          config,
          row,
          new_row_index,
          current_mode_rv(),
          current_omit_rv(),
          comparators_rv
        )
      }
    } else {
      current_mode_rv(config$get("details.mode"))
    }
  })

  shiny::observeEvent(input$save_comments, {
    idx     <- row_index_rv()
    comment <- input$details_out_comments

    dt <- dt_data_rv()
    dt[idx, "comments_details"] <- comment
    dt[idx, "comments"] <- ifelse(comment != "", "yes", "no")
    dt_data_rv(dt)

    DT::replaceData(dt_proxy, dt_data_rv(), resetPaging = FALSE)
    shinyjs::delay(100, {
      session$sendCustomMessage(
        "highlightRow",
        list(row_id = idx)
      )
    })
  })

  shiny::observeEvent(input$clear_comments, {
    idx <- row_index_rv()
    dt  <- dt_data_rv()

    if (!is.null(dt)) {
      if ("" != dt[idx, "comments_details"]) {
        dt[idx, "comments_details"] <- ""
        dt[idx, "comments"] <- "no"
        dt_data_rv(dt)

        DT::replaceData(dt_proxy, dt_data_rv(), resetPaging = FALSE)
        shinyjs::delay(100, {
          session$sendCustomMessage(
            "highlightRow",
            list(row_id = idx)
          )
        })
      }
    }

    shiny::updateTextAreaInput(session, "details_out_comments", value = "")
  })

  shiny::observeEvent(input$configure, {
    ui_elems <- generate_config_ui_grouped(config$schema, config)

    shiny::showModal(shiny::modalDialog(
      shiny::tags$h2("Comparison configuration"),
      shiny::tagList(ui_elems),

      footer = shiny::tagList(
        shiny::actionButton("reset_config_modal", "Reset"),
        shiny::actionButton("submit_apply", "Apply"),
        shiny::actionButton("submit_apply_save", "Apply and Save"),
        shiny::actionButton("cancel_config_modal", "Cancel"),
      )
    ))
  })

  shiny::observeEvent(input$submit_apply, {
    apply_config(
      session,
      input,
      prev_comments,
      reprocess,
      config,
      dt_data_rv,
      save = FALSE
    )
  })

  shiny::observeEvent(input$submit_apply_save, {
    apply_config(
      session,
      input,
      prev_comments,
      reprocess,
      config,
      dt_data_rv,
      save = TRUE
    )
  })

  shiny::observeEvent(input$reset_config_modal, {
    reset_config(session, config)
  })

  shiny::observeEvent(input$cancel_config_modal, {
    cancel_config(session, config)
    shiny::removeModal()
  })

  summary_verify <- shiny::reactive({
    # processing requires changes in list_of_files or force reprocess() update
    shiny::req(list_of_files())
    reprocess()

    # Natural sort so file2 comes before file10 regardless of naming convention
    natural_key <- function(x) {
      parts <- strsplit(x, "(?<=\\D)(?=\\d)|(?<=\\d)(?=\\D)", perl = TRUE)[[1]]
      nums  <- suppressWarnings(as.numeric(parts))
      paste(
        ifelse(is.na(nums), parts, sprintf("%020.0f", nums)), collapse = "\t"
      )
    }
    files_sorted <- list_of_files()
    files_sorted <- files_sorted[
      order(vapply(basename(files_sorted$file1), natural_key, character(1))),
    ]

    dt_file_list <- tibble::tibble(files_sorted) |>
      dplyr::mutate(
        omitted = current_omit_rv(),
        comparison = NA_character_,
        comments = "no",
        comments_details = "",
        process_button = purrr::pmap_chr(
          .l = list(row_id = dplyr::row_number()),
          .f = function(row_id) {
            as.character(
              shiny::actionButton(
                inputId = paste0("process_", row_id),
                label   = "Compare",
                class   = "process_button",
                onclick = sprintf(
                  "Shiny.setInputValue('process_row', %d)", row_id
                )
              )
            )
          }
        )
      )

    shiny::withProgress(
      message = "Processing comparison summaries...",
      value = 0,
      {
        dt_file_list <- dt_file_list |>
          dplyr::mutate(
            comparison = purrr::pmap_chr(
              .l = list(
                file1     = dt_file_list$file1,
                file2     = dt_file_list$file2,
                omitted   = dt_file_list$omitted,
                row_index = seq_along(dt_file_list$file1)
              ),
              .f = function(file1, file2, omitted, row_index) {
                shiny::setProgress(detail = file1)

                # Process a single row
                comparator <- get_comparator(
                  row_index,
                  file1,
                  file2,
                  comparators_rv
                )

                result <- comparator$vrf_summary(
                  omit   = omitted,
                  config = config
                )

                # Update progress
                shiny::incProgress(1 / nrow(dt_file_list))
                result
              }
            )
          )
      }
    )

    dt_file_list
  })

  shiny::observeEvent(summary_verify(), {
    dt_data_rv(summary_verify())
    row_index_rv(NULL)
    shiny::updateTextAreaInput(session, "details_out_comments", value = "")
  })

  shiny::observe({
    # handle changes in folder selections
    update_folder_selections(input, session, roots)
    update_file_selections(input, session, roots)

    shiny::req(input$process_row)
    new_row_index <- input$process_row

    # clear/initialize comparison specific comment value when selecting a row
    if (!is.null(row_index_rv())) {
      if (row_index_rv() != new_row_index) {
        dt <- dt_data_rv()

        if (!is.null(dt)) {
          row_comment <- paste0(dt[new_row_index, "comments_details"])
        } else {
          row_comment <- ""
        }

        shiny::updateTextAreaInput(
          session,
          "details_out_comments",
          value = row_comment
        )
      }
    }

    set_visibility("comparison_comments_container", TRUE)
    set_visibility("details_tabs", TRUE)

    row_index_rv(new_row_index)
    row <- summary_verify()[new_row_index, ]

    # list side-by-side comparison
    details_text("")
    update_details_comparison(
      input,
      output,
      session,
      config,
      row,
      new_row_index,
      current_mode_rv(),
      current_omit_rv(),
      comparators_rv
    )

    # set up the file download links for the compared files
    update_download_links(output, row, file1_link, file2_link)

    # set the compared row as highlighted in the table
    session$sendCustomMessage(
      "highlightRow",
      list(row_id = new_row_index)
    )
  })

}

list_files <- function(input, summary_text, current_omit_rv) {
  if (input$compare_tabs == "tabs_folder") {
    current_omit_rv(input$omit_rows)

    if (file.exists(input$folder1) && file.exists(input$folder2)) {
      set_visibility("comparison_comments_container", FALSE)
      shinyjs::runjs("$('#download_csv').css('display', 'inline-block');")
      shinyjs::runjs("$('#download_html').css('display', 'inline-block');")
      set_reactive_text(summary_text, "")

      verifyr2::list_folder_files(
        input$folder1,
        input$folder2,
        input$file_name_pattern
      )
    } else {
      set_reactive_text(
        summary_text,
        "No folder selected or folders do not exist"
      )
      NULL
    }
  } else {
    current_omit_rv(input$omit_file_rows)

    if (file.exists(input$file1) && file.exists(input$file2)) {
      set_visibility("comparison_comments_container", FALSE)
      shinyjs::runjs("$('#download_csv').css('display', 'inline-block');")
      shinyjs::runjs("$('#download_html').css('display', 'inline-block');")
      set_reactive_text(summary_text, "")

      verifyr2::list_files(
        input$file1,
        input$file2
      )
    } else {
      set_reactive_text(
        summary_text,
        "No files selected or files do not exist"
      )
      NULL
    }
  }
}

generate_config_ui_inputs <- function(schema, config, prefix = "") {
  inputs <- list()

  for (key in names(schema)) {
    full_key <- if (prefix == "") key else paste(prefix, key, sep = ".")
    entry    <- schema[[key]]

    if (is.list(entry)) {
      if (!is.null(entry$options) && !is.null(entry$title)) {
        inputs[[full_key]] <- shiny::selectInput(
          inputId  = full_key,
          label    = entry$title,
          choices  = entry$options,
          selected = config$get(full_key)
        )
      } else {
        sub_inputs <- generate_config_ui_inputs(entry, config, full_key)
        inputs     <- c(inputs, sub_inputs)
      }
    }
  }
  inputs
}

generate_config_ui_grouped <- function(schema, config, prefix = "") {
  groups <- list()

  for (group in names(schema)) {
    entries    <- schema[[group]]
    group_desc <- group

    if (!is.null(entries$title)) {
      group_desc <- entries$title
    }

    # Exclude "title" key itself from leaf scanning
    keys <- setdiff(names(entries), "title")

    inputs <- list()
    for (key in keys) {
      full_key <- paste(group, key, sep = ".")
      entry    <- entries[[key]]

      # Recurse if it's nested
      if (is.list(entry)) {
        if (!is.null(entry$options) && !is.null(entry$title)) {
          inputs[[full_key]] <- shiny::selectInput(
            inputId  = full_key,
            label    = shiny::tagList(
              entry$title,
              " ",
              shiny::tags$span(
                shiny::icon("question-circle"),
                title = entry$desc,
                style = "color:#6c757d; margin-left:5px;"
              )
            ),
            choices  = entry$options,
            selected = config$get(full_key)
          )
        }
      }
    }

    groups[[group]] <- shiny::tagList(
      shiny::tags$h4(group_desc),
      inputs
    )
  }
  groups
}

apply_config <- function(
  session,
  input,
  prev_comments,
  reprocess,
  config,
  dt_data_rv,
  save
) {
  schema <- config$schema
  keys   <- names(generate_config_ui_inputs(schema, config))
  reload <- FALSE

  for (key in keys) {
    val <- input[[key]]
    if (!is.null(val) && val != config$get(key)) {
      reload <- reload || config$get_schema_item(key)$reload
    }
  }

  # possible reload is wrapped in the confirmation dialog check
  if (reload) {
    check_comment_changes(
      input,
      prev_comments,
      dt_data_rv,
      on_confirm = function() {
        store_config(input, keys, config, save)

        # force current file reprocessing
        reprocess(reprocess() + 1)
      }, on_cancel = function() {
        cancel_config(session, config)
      }
    )
  } else {
    store_config(input, keys, config, save)
  }
}

store_config <- function(input, keys, config, save) {
  for (key in keys) {
    val <- input[[key]]
    if (!is.null(val) && val != config$get(key)) {
      config$set(key, val)
    }
  }

  if (save) {
    config$save()
    shiny::showNotification("Configuration saved and applied", type = "message")
  } else {
    shiny::showNotification("Configuration applied", type = "message")
  }

  shiny::removeModal()
}

# set values from existing config to options popup
cancel_config <- function(session, config) {
  keys <- names(generate_config_ui_inputs(config$schema, config))
  for (key in keys) {
    shiny::updateSelectInput(session, key, selected = config$get(key))
  }
}

# set default values from config schema to options popup
reset_config <- function(session, config) {
  keys <- names(generate_config_ui_inputs(config$schema, config))
  for (key in keys) {
    default_value <- config$get_schema_item(key)$default
    shiny::updateSelectInput(session, key, selected = default_value)
  }
}

check_comment_changes <- function(
  input,
  prev_contents,
  dt_data_rv,
  on_confirm,
  on_cancel = function(...) {}
) {
  comments <- dt_data_rv()[["comments_details"]]

  # if the current comment data is empty don't show the modal
  skip <- length(comments) == 0 || all(is.na(comments) | trimws(comments) == "")

  if (!skip && !identical(prev_contents(), comments)) {
    shiny::showModal(
      shiny::modalDialog(
        title = "Confirm file reloading",
        paste0(
          "You have unsaved comparison comments added after the last export. ",
          "Reloading the files will reprocess the comparisons and remove all ",
          "current comments."
        ),
        footer = shiny::tagList(
          shiny::actionButton("cancel_proceed", "Cancel"),
          shiny::actionButton("confirm_proceed", "Confirm")
        )
      )
    )

    shiny::observeEvent(input$confirm_proceed, {
      shiny::removeModal()
      on_confirm()
    }, once = TRUE, ignoreInit = TRUE)

    shiny::observeEvent(input$cancel_proceed, {
      shiny::removeModal()
      on_cancel()
    }, once = TRUE, ignoreInit = TRUE)
  } else {
    on_confirm()
  }
}

shiny::shinyApp(ui, server)

base <- "test_outputs/bin"

################################################################################
# Generic file existence checks
################################################################################

test_that("returns null if both of the files do not exist", {
  file1 <- testthat::test_path(base, "nonexisting1.bin")
  file2 <- testthat::test_path(base, "nonexisting2.bin")

  result <- list_files(file1, file2)
  expect_null(result)
})

test_that("returns null if one of the files does not exist", {
  file1 <- testthat::test_path(base, "base.bin")
  file2 <- testthat::test_path(base, "nonexisting.bin")

  result <- list_files(file1, file2)
  expect_null(result)
})

################################################################################
# Successfull cases
################################################################################

test_that("Returns tibble when both files exist", {
  file1 <- testthat::test_path(base, "base.bin")
  file2 <- testthat::test_path(base, "modified.bin")

  result <- list_files(file1, file2)

  expect_s3_class(result, "tbl_df")
  expect_named(result, c("file1", "file2"))
  expect_equal(result$file1, file1)
  expect_equal(result$file2, file2)
})

proc datasets library=work kill;run;
/**********************************************************************
Creates case study for NMA combining IPD and AD
Author:Anna Wiksten
Date:19May2015
**********************************************************************/
/*set path for  folder to save figures*/
%let fp=H:\data\PhD\project4\manuscript;
/*Set prefix to be added to result figures*/
%let pgm=case1;
/*set seed for simulation*/
*%let seed=474685678;
/*set seed for simulation*/
*%let seed=474685678;
%let seed=45668;
/*Between and within study standard deviation*/
%let sigmau=0.1;*between;
%let sigmae=1.6;*within;
/*Give treatment effects, intercept(theta) and slope(beta)*/
data effects;
input trtn int slope;
cards;
1	0	0.66
2	0.66	0
3	0.5151	-0.00241
4	0.2	0.2
5	-0.2356	0.4
6	0.3192	0.211
7	0		0
run;

/*Studies in the network*/
data studies;
input study ipd xmean xstd A B C D E F xP;
label 	study="Study ID(must be unique)"
		ipd="0=AD,1=IPD"
		xmean="Mean of  covariate distribution"
		xmean="Standard deviation of  covariate distribution";
cards;
1	0	0.79	0.42	0	219	0	0	0	0	104
2	0	1.02	0.44	0	122	0	0	56	0	0
3	0	0.76	0.37	0	370	0	0	0	0	183
4	0	1.26	0.44	0	854	427	0	0	0	425
5	0	1.07	0.50	0	1028	45	0	0	542	419
6	0	1.07	0.48	0	330	0	0	327	0	334
7	0	1.39	0.56	0	211	0	0	0	0	205
8	0	0.87	0.30	0	1707	0	0	0	1714	0
9	0	1.09	0.43	0	760	0	0	0	776	0
10	0	1.02	0.48	0	161	0	0	0	0	155
11	0	1.10	0.50	0	150	0	0	0	0	155
12	0	1.04	0.48	0	473	0	471	0	478	229
13	0	1.05	0.46	0	260	0	258	0	0	260
14	0	0.95	0.48	0	251	0	249	0	0	247
15	1	1.11	0.50	523	0	0	0	0	266	267
16	1	1.06	0.46	550	0	0	0	0	0	265
17	1	0.77	0.37	301	0	0	0	0	0	154
18	1	1.04	0.49	325	0	0	0	0	326	0
19	1	1.06	0.50	222	0	0	0	0	0	218
20	1	1.34	0.47	216	0	0	0	0	0	215
21	1	0.85	0.30	702	0	0	707	0	689	0
run;
proc transpose data=studies out=s1;
by study ipd xmean xstd notsorted;
var a b c d e f xP;
run;

data s2;
set s1;
where col1 ne 0;
trt2=_name_;
if trt2="A" then trtn=1	;
if trt2="B" then trtn=2;
if trt2="C" then trtn=3;	
if trt2="D" then trtn=4;	
if trt2="E" then trtn=5;	
if trt2="F" then trtn=6	;
if trt2="xP" then trtn=7;
proc sort;by trtn;
run;

data s3;
merge s2 effects;
by trtn;
proc sort; by study trtn;
run;
/*Simulate ipd data*/
data s4;
call streaminit(&seed.);
set s3;
by study trtn;
if first.study or first.trtn then do;
u=rand("NORMAL",0,&sigmau.);
do i=1 to col1;
e=rand("NORMAL",0,&sigmae.);
x=rand("NORMAL",xmean,xstd);
y=int+slope*x+u+e;output;
end; 
end;
run; 

data ipd0;
set s4;
keep ipd study trtn  y x;
proc sort;by ipd study;
run;

/*Create aggregate data for all studies*/
ods output lsmeans=ad0;
proc mixed data=ipd0;
by ipd study;
class trtn;
model y=trtn ;
lsmeans trtn;
run;
/*Mean covariate values*/
proc means data=ipd0;
by ipd study;
var x;
output out=x(keep=ipd study x) mean=x;
run;

data ad1;
merge ad0 x;
by ipd study;
y=estimate;
keep ipd study trtn y x stderr;
run;
proc sort;by ipd study trtn;
run;
/*Create case data, different case number for each study*treatment combination*/
data case;
set ad0;
case=_n_;
keep ipd study trtn case;
run;
/*Final AD*/
data ad2;
merge case ad1 ;
by ipd study trtn;
case2=case; if ipd=1 then case2=1000;
x1=x-1;
run;




/*Add case and mean x values to IPD*/
data ipd1;
merge case ipd0 ;
by ipd study trtn;
keep case ipd study trtn y x;
run;
data ipd2;
merge ipd1 x(rename=(x=meanx));
by ipd study;
case2=case; if ipd=1 then case2=1000;
centx=x-meanx;
x1=x-1;
run;

/*Create combined AD and IPD*/
data ipd_ad0;
set ad2(where=(ipd=0)) ipd2(where=(ipd=1)) ;
run;

data ipd_ad1;
merge ipd_ad0 x(rename=(x=meanx));
by ipd study;
centx=x-meanx;
x1=x-1;
run;

/*IPD analysis*/
ods output estimates=est_i;
proc mixed  data=ipd2;
	title 'IPD and AD';
	class study trtn ;
	model y=  study trtn x1*study x1*trtn/ ddf=(10000,10000,10000,10000) noint s cl;
	random study*trtn;
	*repeated/group=case;/*heterogeneus residual variances*/
	estimate "ap" trtn 1 0 0 0 0 0 -1/alpha=0.05;
	estimate "bp" trtn 0 1 0 0 0 0 -1;
	estimate "cp" trtn 0 0 1 0 0 0 -1;
	estimate "dp" trtn 0 0 0 1 0 0 -1;
	estimate "ep" trtn 0 0 0 0 1 0 -1;
	estimate "fp" trtn 0 0 0 0 0 1 -1;
	estimate "ba" trtn 1 -1;
	estimate "bap" x1*trtn 1 0 0 0 0 0 -1/alpha=0.05;
	estimate "bbp" x1*trtn 0 1 0 0 0 0 -1;
	estimate "bcp" x1*trtn 0 0 1 0 0 0 -1;
	estimate "bdp" x1*trtn 0 0 0 1 0 0 -1;
	estimate "bep" x1*trtn 0 0 0 0 1 0 -1;
	estimate "bfp" x1*trtn 0 0 0 0 0 1 -1;
	estimate "bba" x1*trtn 1 -1;
run;
/*IPD and AD analysis*/
/*Create covariance parameter data*/
data u;
study=0;stderr=sqrt(0.000165);case=0;
run;
data parms_ip_ad;
set u ad2;
est=stderr**2;
if ipd=1 then est=0.05;
case2=case; if ipd=1 then case2=1000;
keep study trtn est case2;
proc sort nodupkey;by case2;
run;

data firstipdcase;
set ad2(where=(ipd=1));
if _n_=1;
call symput('firstipdcase',case);
run;

ods output estimates=est_ia;
proc mixed  data=ipd_ad0;
	title 'IPD and AD';
	class study trtn ;
	model y=  study trtn x1*study x1*trtn/ddf=(10000,10000,10000,10000)  noint s cl;
	random trtn/subject=study;
	repeated/group=case2;/*heterogeneus residual variances*/
	parms/eqcons=2 to &firstipdcase. parmsdata=parms_ip_ad;
	estimate "ap" trtn 1 0 0 0 0 0 -1/alpha=0.05;
	estimate "bp" trtn 0 1 0 0 0 0 -1;
	estimate "cp" trtn 0 0 1 0 0 0 -1;
	estimate "dp" trtn 0 0 0 1 0 0 -1;
	estimate "ep" trtn 0 0 0 0 1 0 -1;
	estimate "fp" trtn 0 0 0 0 0 1 -1;
	estimate "ba" trtn 1 -1;
	estimate "bap" x1*trtn 1 0 0 0 0 0 -1/alpha=0.05;
	estimate "bbp" x1*trtn 0 1 0 0 0 0 -1;
	estimate "bcp" x1*trtn 0 0 1 0 0 0 -1;
	estimate "bdp" x1*trtn 0 0 0 1 0 0 -1;
	estimate "bep" x1*trtn 0 0 0 0 1 0 -1;
	estimate "bfp" x1*trtn 0 0 0 0 0 1 -1;
	estimate "bba" x1*trtn 1 -1;
run;
/*AD analysis*/
/*Create covariance parameter data*/
data parms_ad;
set u ad0;
est=stderr**2;
keep study trtn est;
run;

ods output estimates=est_a;
proc mixed  data=ad2;
	title 'IPD and AD';
	class study trtn ;
	model y=  study trtn  x1*trtn/  ddf=(10000,10000,10000) noint s cl;
	random study*trtn;
	repeated/group=case;/*heterogeneus residual variances*/
	parms/eqcons=2 to 10000 parmsdata=parms_ad;
	estimate "ap" trtn 1 0 0 0 0 0 -1/alpha=0.05;
	estimate "bp" trtn 0 1 0 0 0 0 -1;
	estimate "cp" trtn 0 0 1 0 0 0 -1;
	estimate "dp" trtn 0 0 0 1 0 0 -1;
	estimate "ep" trtn 0 0 0 0 1 0 -1;
	estimate "fp" trtn 0 0 0 0 0 1 -1;
	estimate "ba" trtn 1 -1;
	estimate "bap" x1*trtn 1 0 0 0 0 0 -1/alpha=0.05;
	estimate "bbp" x1*trtn 0 1 0 0 0 0 -1;
	estimate "bcp" x1*trtn 0 0 1 0 0 0 -1;
	estimate "bdp" x1*trtn 0 0 0 1 0 0 -1;
	estimate "bep" x1*trtn 0 0 0 0 1 0 -1;
	estimate "bfp" x1*trtn 0 0 0 0 0 1 -1;
	estimate "bba" x1*trtn 1 -1;
run;

/*Combine estimates for figure*/
libname library "&fp";
proc format library=library;
value dt 1="IPD"
2="AD and IPD"
3="AD";
run;

data est;
set est_i(in=a) est_ia(in=b) est_a(in=c);
if a then dat=1;
if b then dat=2;
if c then dat=3;
if a then x=label*1-0.05;
if b then x=label*1;
if c then x=label*1+0.05;
format dat dt.;
run;


options ls=256 ps=100 nocenter;

data aaa;
set est_i(in=a) est_ia(in=b) est_a(in=c);
if a then d=1;
if b then d=2;
if c then d=3;
e=round(estimate,0.001); 
l=round(lower,0.001); 
u=round(upper,0.001); 
ci=compress(e||"("||l||","||u||")");
x=(d-1)*14;
obsn=_n_;
parid=obsn-x;
run;
proc sort data=aaa;by parid;
run;

proc transpose data=aaa out=a1 prefix=d;
by parid;
id d ;
var ci;
run;
data a2;
retain d1 a1 d2 a2 d3 a3;
set a1;
a1="&";
a2="&";
a3="&";
run;
proc print; run;




\begin{table}[h]
\begin{tabular}{ L{3.0cm}  L{1.6cm}  C{3.6cm}  C{3.6cm} C{3.6cm}  }
	&	&	\multicolumn{3}{c}{Estimate and  95\% CI}\\ \cline{3-5}
	Model&Parameter&IPD& IPD and AD & AD\\ \hline
Models \ref{eq:m3ipd} and \ref{eq:m2ipd}&$\theta_A$&	 0.658(0.501,0.815)      &     0.656(0.492,0.82)        &     0.662(0.476,0.849) 	 	 	\\
 REML			&$\theta_B$&							 0.542(0.423,0.661)      &     0.541(0.412,0.671)       &     0.543(0.406,0.68) 	 	 	\\
				&$\theta_C$&							 0.62(0.319,0.922)       &     0.81(0.093,1.526)        &     0.807(0.077,1.538) 	 	 	\\
				&$\theta_D$&							 0.539(0.356,0.722)      &     0.539(0.347,0.731)       &     0.531(0.322,0.74) 	 	 	\\
				&$\theta_E$&							 0.243(-0.067,0.554)     &     0.062(-1.028,1.152)      &     0.063(-1.056,1.182) 	 	 	\\
				&$\theta_F$&							 0.431(0.279,0.583)      &     0.432(0.272,0.592)       &     0.441(0.265,0.618) 	 	 	\\
				&$\theta_A-\theta_B$&					 0.116(-0.063,0.295)     &     0.115(-0.074,0.304)      &     0.12(-0.087,0.327) 	 	 	\\
				&$\beta_A$&								 0.608(0.397,0.818)      &     0.66(0.412,0.908)        &     0.821(-0.262,1.905) 	 	 	\\
				&$\beta_B$&								 -0.023(-0.176,0.129)    &     -0.124(-0.832,0.584)     &     -0.158(-0.925,0.609) 	 	 	\\
				&$\beta_C$&								 -0.168(-0.554,0.217)    &     -1.064(-4.064,1.936)     &     -1.069(-4.155,2.017) 	 	 	\\
				&$\beta_D$&								 0.253(0.01,0.496)       &     0.341(-0.193,0.874)      &     -0.012(-2.376,2.352) 	 	 	\\
				&$\beta_E$&								 -0.121(-0.542,0.299)    &     3.096(-16.113,22.305)    &     3.073(-16.731,22.877) 	 	 	\\
				&$\beta_F$&								 0.077(-0.109,0.263)     &     0.206(-0.144,0.556)      &     -0.003(-1.578,1.572) 	 	 	\\
				&$\beta_A-\beta_B$&						 0.631(0.398,0.864)      &     0.784(0.045,1.524)       &     0.98(-0.274,2.233)	 	 	\\
				&$\sigma_u^2$&&&\\

	&&&&
\end{tabular}
\caption{Proportion of simulations where the ratio of the lengths of  fixed effect and \hk\  confidence intervals is greater than given value} \label{tab:simres1}
\end{table}


  0.658(0.501,0.815)      &     0.656(0.492,0.82)        &     0.662(0.476,0.849)
  0.542(0.423,0.661)      &     0.541(0.412,0.671)       &     0.543(0.406,0.68)
  0.62(0.319,0.922)       &     0.81(0.093,1.526)        &     0.807(0.077,1.538)
  0.539(0.356,0.722)      &     0.539(0.347,0.731)       &     0.531(0.322,0.74)
  0.243(-0.067,0.554)     &     0.062(-1.028,1.152)      &     0.063(-1.056,1.182)
  0.431(0.279,0.583)      &     0.432(0.272,0.592)       &     0.441(0.265,0.618)
  0.116(-0.063,0.295)     &     0.115(-0.074,0.304)      &     0.12(-0.087,0.327)
  0.608(0.397,0.818)      &     0.66(0.412,0.908)        &     0.821(-0.262,1.905)
  -0.023(-0.176,0.129)    &     -0.124(-0.832,0.584)     &     -0.158(-0.925,0.609)
  -0.168(-0.554,0.217)    &     -1.064(-4.064,1.936)     &     -1.069(-4.155,2.017)
  0.253(0.01,0.496)       &     0.341(-0.193,0.874)      &     -0.012(-2.376,2.352)
  -0.121(-0.542,0.299)    &     3.096(-16.113,22.305)    &     3.073(-16.731,22.877)
  0.077(-0.109,0.263)     &     0.206(-0.144,0.556)      &     -0.003(-1.578,1.572)
  0.631(0.398,0.864)      &     0.784(0.045,1.524)       &     0.98(-0.274,2.233)

proc datasets library=work kill;run;
/**********************************************************************
Creates case study for NMA combining IPD and AD
Author:Anna Wiksten
Date:19May2015
**********************************************************************/
/*set path for  folder to save figures*/
%let fp=H:\data\PhD\project4\manuscript;
/*Set prefix to be added to result figures*/
%let pgm=case1;
/*set seed for simulation*/
*%let seed=474685678;
/*set seed for simulation*/
*%let seed=474685678;
%let seed=45668;
/*Between and within study standard deviation*/
%let sigmau=0.1;*between;
%let sigmae=1.6;*within;
/*Give treatment effects, intercept(theta) and slope(beta)*/
data effects;
input trtn int slope;
cards;
1	0	0.66
2	0.66	0
3	0.5151	-0.00241
4	0.2	0.2
5	-0.2356	0.4
6	0.3192	0.211
7	0		0
run;
/*Studies in the network*/
data studies;
input study ipd xmean xstd A B C D E F xP;
label 	study="Study ID(must be unique)"
		ipd="0=AD,1=IPD"
		xmean="Mean of  covariate distribution"
		xmean="Standard deviation of  covariate distribution";
cards;
1	0	0.79	0.42	0	219	0	0	0	0	104
2	0	1.02	0.44	0	122	0	0	56	0	0
3	0	0.76	0.37	0	370	0	0	0	0	183
4	0	1.26	0.44	0	854	427	0	0	0	425
5	0	1.07	0.50	0	1028	45	0	0	542	419
6	0	1.07	0.48	0	330	0	0	327	0	334
7	0	1.39	0.56	0	211	0	0	0	0	205
8	0	0.87	0.30	0	1707	0	0	0	1714	0
9	0	1.09	0.43	0	760	0	0	0	776	0
10	0	1.02	0.48	0	161	0	0	0	0	155
11	0	1.10	0.50	0	150	0	0	0	0	155
12	0	1.04	0.48	0	473	0	471	0	478	229
13	0	1.05	0.46	0	260	0	258	0	0	260
14	0	0.95	0.48	0	251	0	249	0	0	247
15	1	1.11	0.50	523	0	0	0	0	266	267
16	1	1.06	0.46	550	0	0	0	0	0	265
17	1	0.77	0.37	301	0	0	0	0	0	154
18	1	1.04	0.49	325	0	0	0	0	326	0
19	1	1.06	0.50	222	0	0	0	0	0	218
20	1	1.34	0.47	216	0	0	0	0	0	215
21	1	0.85	0.30	702	0	0	707	0	689	0
run;
proc transpose data=studies out=s1;
by study ipd xmean xstd notsorted;
var a b c d e f xP;
run;

data s2;
set s1;
where col1 ne 0;
trt2=_name_;
if trt2="A" then trtn=1	;
if trt2="B" then trtn=2;
if trt2="C" then trtn=3;	
if trt2="D" then trtn=4;	
if trt2="E" then trtn=5;	
if trt2="F" then trtn=6	;
if trt2="xP" then trtn=7;
proc sort;by trtn;
run;

data s3;
merge s2 effects;
by trtn;
proc sort; by study trtn;
run;
/*Simulate ipd data*/
data s4;
call streaminit(&seed.);
set s3;
by study trtn;
if first.study or first.trtn then do;
u=rand("NORMAL",0,&sigmau.);
do i=1 to col1;
e=rand("NORMAL",0,&sigmae.);
x=rand("NORMAL",xmean,xstd);
y=int+slope*x+u+e;output;
end; 
end;
run; 

data ipd0;
set s4;
keep ipd study trtn  y x;
proc sort;by ipd study;
run;

/*Create aggregate data for all studies*/
ods output lsmeans=ad0;
proc mixed data=ipd0;
by ipd study;
class trtn;
model y=trtn ;
lsmeans trtn;
run;
/*Mean covariate values*/
proc means data=ipd0;
by ipd study;
var x;
output out=x(keep=ipd study x) mean=x;
run;

data ad1;
merge ad0 x;
by ipd study;
y=estimate;
keep ipd study trtn y x stderr;
run;
proc sort;by ipd study trtn;
run;
/*Create case data, different case number for each study*treatment combination*/
data case;
set ad0;
case=_n_;
keep ipd study trtn case;
run;
/*Final AD*/
data ad2;
merge case ad1 ;
by ipd study trtn;
case2=case; if ipd=1 then case2=1000;
x1=x-1;
run;




/*Add case and mean x values to IPD*/
data ipd1;
merge case ipd0 ;
by ipd study trtn;
keep case ipd study trtn y x;
run;
data ipd2;
merge ipd1 x(rename=(x=meanx));
by ipd study;
case2=case; if ipd=1 then case2=1000;
centx=x-meanx;
x1=x-1;
run;

/*Create combined AD and IPD*/
data ipd_ad0;
set ad2(where=(ipd=0)) ipd2(where=(ipd=1)) ;
run;

data ipd_ad1;
merge ipd_ad0 x(rename=(x=meanx));
by ipd study;
centx=x-meanx;
x1=x-1;
run;

/*IPD analysis*/
ods output estimates=est_i;
proc mixed  data=ipd2;
	title 'IPD and AD';
	class study trtn ;
	model y=  study trtn x1*study x1*trtn/ ddf=(10000,10000,10000,10000) noint s cl;
	random study*trtn;
	*repeated/group=case;/*heterogeneus residual variances*/
estimate "0.5" trtn 1 -1 x1*trtn -0.5 0.5/alpha=0.05;
estimate "1" trtn 1 -1 x1*trtn 0 0/alpha=0.05;
estimate "1.5" trtn 1 -1 x1*trtn 0.5 -0.5/alpha=0.05;
estimate "2" trtn 1 -1 x1*trtn 1 -1/alpha=0.05;
estimate "2.5" trtn 1 -1 x1*trtn 1.5 -1.5/alpha=0.05;
estimate "3" trtn 1 -1 x1*trtn 2 -2/alpha=0.05;
estimate "3.5" trtn 1 -1 x1*trtn 2.5 -2.5/alpha=0.05;
run;
/*IPD and AD analysis*/
/*Create covariance parameter data*/
data u;
study=0;stderr=sqrt(0.000165);case=0;
run;
data parms_ip_ad;
set u ad2;
est=stderr**2;
if ipd=1 then est=0.05;
case2=case; if ipd=1 then case2=1000;
keep study trtn est case2;
proc sort nodupkey;by case2;
run;

data firstipdcase;
set ad2(where=(ipd=1));
if _n_=1;
call symput('firstipdcase',case);
run;

ods output estimates=est_ia;
proc mixed  data=ipd_ad0;
	title 'IPD and AD';
	class study trtn ;
	model y=  study trtn x1*study x1*trtn/ddf=(10000,10000,10000,10000)  noint s cl;
	random trtn/subject=study;
	repeated/group=case2;/*heterogeneus residual variances*/
	parms/eqcons=2 to &firstipdcase. parmsdata=parms_ip_ad;
estimate "0.5" trtn 1 -1 x1*trtn -0.5 0.5/alpha=0.05;
estimate "1" trtn 1 -1 x1*trtn 0 0/alpha=0.05;
estimate "1.5" trtn 1 -1 x1*trtn 0.5 -0.5/alpha=0.05;
estimate "2" trtn 1 -1 x1*trtn 1 -1/alpha=0.05;
estimate "2.5" trtn 1 -1 x1*trtn 1.5 -1.5/alpha=0.05;
estimate "3" trtn 1 -1 x1*trtn 2 -2/alpha=0.05;
estimate "3.5" trtn 1 -1 x1*trtn 2.5 -2.5/alpha=0.05;
run;
/*AD analysis*/
/*Create covariance parameter data*/
data parms_ad;
set u ad0;
est=stderr**2;
keep study trtn est;
run;

ods output estimates=est_a;
proc mixed  data=ad2;
	title 'IPD and AD';
	class study trtn ;
	model y=  study trtn  x1*trtn/  ddf=(10000,10000,10000) noint s cl;
	random study*trtn;
	repeated/group=case;/*heterogeneus residual variances*/
	parms/eqcons=2 to 10000 parmsdata=parms_ad;
estimate "0.5" trtn 1 -1 x1*trtn -0.5 0.5/alpha=0.05;
estimate "1" trtn 1 -1 x1*trtn 0 0/alpha=0.05;
estimate "1.5" trtn 1 -1 x1*trtn 0.5 -0.5/alpha=0.05;
estimate "2" trtn 1 -1 x1*trtn 1 -1/alpha=0.05;
estimate "2.5" trtn 1 -1 x1*trtn 1.5 -1.5/alpha=0.05;
estimate "3" trtn 1 -1 x1*trtn 2 -2/alpha=0.05;
estimate "3.5" trtn 1 -1 x1*trtn 2.5 -2.5/alpha=0.05;
run;

/*Combine estimates for figure*/
libname library "&fp";
proc format library=library;
value dt 1="IPD"
2="AD and IPD"
3="AD";
run;

data est;
set est_i(in=a) est_ia(in=b) est_a(in=c);
if a then dat=1;
if b then dat=2;
if c then dat=3;
if a then x=label*1-0.05;
if b then x=label*1;
if c then x=label*1+0.05;
format dat dt.;
run;

   %modstyle(name=markstyle, parent=statistical, type=CLM,
             markers= circlefilled squarefilled trianglefilled,
			   linestyles=1 1 1,
colors=red green blue);


ods graphics on /
      width=10cm
      height=8cm
      imagefmt=pdf
      /*Nimi vaihtoon*/
      imagename="&pgm._resultsx0"
      reset=index
      border=off;
/*polku vaihtoon, image_dpi maarittaa kuvan tarkkuuden*/
ods listing   style=markstyle  gpath ="&fp" image_dpi=300;

title;
proc sgplot data=est;
scatter x=x y=estimate/  transparency=1;
*vector x=x y=lower/XORIGIN=x yORIGIN= upper   group=dat NOARROWHEADS;
refline 0/axis=y;
XAXIS label="Covariate value" values=(0 to 4 by 0.5);
YAXIS label="Comparison treatment A vs B";;
keylegend "d"  /ACROSS=3 position=bottom noborder title="" ;
lineparm x=1 y=0 slope=1;
yaxis min=-1 max=7;
run;

ods graphics off;


ods graphics on /
      width=10cm
      height=8cm
      imagefmt=pdf
      /*Nimi vaihtoon*/
      imagename="&pgm._resultsx1"
      reset=index
      border=off;
/*polku vaihtoon, image_dpi maarittaa kuvan tarkkuuden*/
ods listing   style=markstyle  gpath ="&fp" image_dpi=300;

title;
proc sgplot data=est;
where dat in (1);
scatter x=x y=estimate/  group=dat name="d";
vector x=x y=lower/XORIGIN=x yORIGIN= upper   group=dat NOARROWHEADS;
refline 0/axis=y;
XAXIS label="Covariate value" values=(0 to 4 by 0.5);
YAXIS label="Comparison treatment A vs B";;
keylegend "d"  /ACROSS=3 position=bottom noborder title="" ;
lineparm x=1 y=0 slope=1;
yaxis min=-1 max=7;
run;

ods graphics off;

ods graphics on /
      width=10cm
      height=8cm
      imagefmt=pdf
      /*Nimi vaihtoon*/
      imagename="&pgm._resultsx2"
      reset=index
      border=off;
/*polku vaihtoon, image_dpi maarittaa kuvan tarkkuuden*/
ods listing   style=markstyle  gpath ="&fp" image_dpi=300;

title;
proc sgplot data=est;
where dat in (1 2);
scatter x=x y=estimate/  group=dat name="d";
vector x=x y=lower/XORIGIN=x yORIGIN= upper   group=dat NOARROWHEADS;
refline 0/axis=y;
XAXIS label="Covariate value" values=(0 to 4 by 0.5);
YAXIS label="Comparison treatment A vs B";;
keylegend "d"  /ACROSS=3 position=bottom noborder title="" ;
lineparm x=1 y=0 slope=1;
yaxis min=-1 max=7;
run;

ods graphics off;

ods graphics on /
      width=10cm
      height=8cm
      imagefmt=pdf
      /*Nimi vaihtoon*/
      imagename="&pgm._resultsx3"
      reset=index
      border=off;
/*polku vaihtoon, image_dpi maarittaa kuvan tarkkuuden*/
ods listing   style=markstyle  gpath ="&fp" image_dpi=300;

title "Model 2: Estimated treatment difference A vs B ";
proc sgplot data=est;
scatter x=x y=estimate/  group=dat name="d";
vector x=x y=lower/XORIGIN=x yORIGIN= upper   group=dat NOARROWHEADS;
refline 0/axis=y;
XAXIS label="Covariate value" values=(0 to 4 by 0.5);
YAXIS label="Comparison treatment A vs B";;
keylegend "d"  /ACROSS=3 position=bottom noborder title="" ;
lineparm x=1 y=0 slope=1;
yaxis min=-1 max=7;
run;

ods graphics off;


   %modstyle(name=markstyle, parent=statistical, type=CLM,
             markers= circlefilled squarefilled trianglefilled,
			   linestyles=1 1 1,
colors=black black black);

ods graphics on /
      width=10cm
      height=8cm
      imagefmt=pdf
      /*Nimi vaihtoon*/
      imagename="&resultsPaper"
      reset=index
      border=off;
/*polku vaihtoon, image_dpi maarittaa kuvan tarkkuuden*/
ods listing   style=markstyle  gpath ="&fp" image_dpi=300;

title;
proc sgplot data=est;
scatter x=x y=estimate/  group=dat name="d";
vector x=x y=lower/XORIGIN=x yORIGIN= upper   group=dat NOARROWHEADS;
refline 0/axis=y;
XAXIS label="Covariate value" values=(0 to 4 by 0.5);
YAXIS label="Comparison treatment A vs B";;
keylegend "d"  /ACROSS=3 position=bottom noborder title="" ;
lineparm x=1 y=0 slope=0.66;
yaxis min=-1 max=7;
run;
ods graphics off;


proc format library=library;
value dt 3="IPD"
2="IPD-AD"
1="AD";
run;

data est;
set est_a(in=c) est_ia(in=b)  est_i(in=a);
if a then dat=3;
if b then dat=2;
if c then dat=1;
if a then x=label*1;
if b then x=label*1;
if c then x=label*1;
format dat dt.;
run;

   %modstyle(name=markstyle, parent=statistical, type=CLM,
             markers= circlefilled squarefilled trianglefilled,
			   linestyles=34 4 1,
colors=black black black,FILLCOLORS=lightgrey darkgrey grey  );

ods graphics on /
      width=10cm
      height=8cm
      imagefmt=pdf
      /*Nimi vaihtoon*/
      imagename="&resultsPaperBands"
      reset=index
      border=off;
/*polku vaihtoon, image_dpi maarittaa kuvan tarkkuuden*/
ods listing   style=markstyle  gpath ="&fp" image_dpi=300;

title;
proc sgplot data=est;
  band x=x lower=lower upper=upper /group=dat
       legendlabel="95% CLI" name="band1" TRANSPARENCY=0;
refline 0/axis=y;
series x=x y=estimate/  group=dat name="d" LINEATTRS= (THICKNESS=2);
XAXIS label="Covariate value" values=(0.5 to 3.5 by 0.5);

keylegend "d" "band1"  /ACROSS=3 position=bottom noborder title="" ;
yaxis min=-1 max=7;
YAXIS label="Comparison treatment A vs B";;
run;
ods graphics off;

#' TextFileComparator.R
#'
#' Fallback comparator for text files without any specific definied comparator.
#' This comparator contains the methods for doing basic comparisons on raw text
#' contents.
#'
#' @import stringr
#'
#' @include BinaryFileComparator.R
#'
#' @examples
#'
#' # The normal way for creating a comparator would be to call the generic
#' # factory method verifyr2::create_comparator that will automatically create
#' # the correct comparator instance based on the file types.
#'
#' file1 <- 'my_file1.txt'
#' file2 <- 'my_file2.txt'
#' comparator <- verifyr2::create_comparator(file1, file2)
#'
#' # If needed, an explicit comparator can be created as well.
#'
#' file1 <- 'my_file1.lst'
#' file2 <- 'my_file2.lst'
#' comparator <- TxtFileComparator$new(file1, file2)
#'
#' @export
#'

# Disable cyclomatic complexity lint for the R6 class definition as lintr
# considers the whole class definition as a single function.
#
# nolint start: cyclocomp_linter
TxtFileComparator <- R6::R6Class(
  "TxtFileComparator",
  inherit = BinaryFileComparator,
  public = list(

    #' @description
    #' Method for comparing the inner part for the details query. This method
    #' can be overwritten by more specialized comparator classes. This method is
    #' intended to be called only by the comparator classes in the processing
    #' and shouldn't be called directly by the user.
    #'
    #' @param config configuration values
    #' @param omit   string pattern to omit from the comparison
    #'
    vrf_summary_inner = function(config, omit) {
      self$vrf_open_debug("Txt::vrf_summary_inner" , config)

      file1_contents_list <- self$file1_contents_list
      file2_contents_list <- self$file2_contents_list

      if (is.null(file1_contents_list)) {
        file1_contents_list <- self$vrf_contents(self$file1, config, omit)
        self$file1_contents_list <- file1_contents_list
      }

      if (is.null(file2_contents_list)) {
        file2_contents_list <- self$vrf_contents(self$file2, config, omit)
        self$file2_contents_list <- file2_contents_list
      }

      file1_contents_processed <- file1_contents_list[[2]]
      file2_contents_processed <- file2_contents_list[[2]]

      difference    <- all.equal(file1_contents_processed, file2_contents_processed)
      result        <- "File content comparison failed!"
      result_images <- ""
      pattern       <- "Lengths \\((\\d+), (\\d+)\\) differ \\(string compare on first"

      if (typeof(difference) == "logical") {
        # all.equal returns logical vector if there are no differences
        result <- "No differences."
      } else if (length(difference) >= 1 && grepl(pattern, difference[1])) {
        # all.equal returns length 1/2 vector with first element comtaining text
        #  matching the pattern
        result <- "Different number of lines in compared content."
      } else if (length(difference) == 1) {
        # all.equal returns length 1 vector if the number of rows are the same but
        # there are differences
        count  <- as.numeric(gsub("[^[:digit:].]", "", difference))
        result <- paste0("File content has changes in ", count, " place(s).")
      }

      self$vrf_close_debug()
      return(result)
    },

    #' @description
    #' Method for comparing the inner part for the details query. This method
    #' can be overwritten by more specialized comparator classes. This method is
    #' intended to be called only by the comparator classes in the processing
    #' and shouldn't be called directly by the user.
    #'
    #' @param config configuration values
    #' @param omit   string pattern to omit from the comparison
    #'
    vrf_details_inner = function(config, omit) {
      self$vrf_open_debug("Txt::vrf_details_inner" , config)

      file1_contents_list <- self$file1_contents_list
      file2_contents_list <- self$file2_contents_list

      if (is.null(file1_contents_list)) {
        file1_contents_list <- self$vrf_contents(self$file1, config, omit)
        self$file1_contents_list <- file1_contents_list
      }

      if (is.null(file2_contents_list)) {
        file2_contents_list <- self$vrf_contents(self$file2, config, omit)
        self$file2_contents_list <- file2_contents_list
      }

      file1_contents_whole <- file1_contents_list[[1]]
      file2_contents_whole <- file2_contents_list[[1]]

      context <- 2
      if ("full" == super$vrf_option_value(config, "details.mode")) {
        context <- -1
      }

      my_equalizer_with_omit <- function(x, x.chr) {
        my_finalizer(x, x.chr, omit)
      }

      style <- diffobj::StyleHtmlLightRgb(
        html.output = "diff.w.style",
        finalizer = my_equalizer_with_omit
      )

      self$vrf_open_debug("Txt::vrf_details_inner::diffPrint" , config)
      diff_print <- diffobj::diffChr(
        file1_contents_whole,
        file2_contents_whole,
        context = context,
        style = style,
        ignore.white.space = ("yes" == super$vrf_option_value(config, "generic.spaces")),
        mode = "sidebyside",
        word.diff = ("word" == super$vrf_option_value(config, "generic.level"))
      )
      self$vrf_close_debug()

      result <- list(
        list(
          type = "text",
          contents = diff_print
        )
      )

      self$vrf_close_debug()
      return(result)
    },

    #' @description
    #' Method for getting the inner part for the file contents query. The method
    #' returns the file contents in two separate vectors inside a list. The
    #' first vector is the file contents and the second one is the file contents
    #' processed for empty spaces and omit terms if applicable. This method can
    #' be overwritten by more specialized comparator classes. This method is
    #' intended to be called only by the comparator classes in the processing
    #' and shouldn't be called directly by the user.
    #'
    #' @param contents file contents
    #' @param config   configuration values
    #' @param omit     string pattern to omit from the comparison
    #'
    vrf_contents_inner = function(contents, config, omit) {
      self$vrf_open_debug("Txt::vrf_contents_inner" , config)

      contents_processed <- contents

      if (!is.null(omit) && "" != paste0(omit)) {
        contents_processed <- stringr::str_subset(
          string = contents,
          pattern = paste0(omit),
          negate = TRUE
        )
      }

      if ("yes" == super$vrf_option_value(config, "generic.spaces")) {
          contents_processed <- stringr::str_replace_all(contents_processed, "[ \t]+", " ")
          contents_processed <- stringr::str_trim(contents_processed, side = "both")
      }

      self$vrf_close_debug()
      return(list(contents, contents_processed))
    },

    #' @description
    #' Inherited method for indicating whether detailed comparison is available
    #' with the current comparator. Returns an empty string if the comparator is
    #' is supported, otherwise a string that will be concatenated with the
    #' summary string.
    #'
    #' @param config configuration values
    #'
    vrf_details_supported = function(config) {
      return("")
    }
  )
)
# nolint end: cyclocomp_linter

#' Custom finalizer method for diffobj html content finalizing. This method is
#' used to modify the diff html output so that omitted rows have their own
#' special styling and gutters.
#'
#' @param x     comparator instance used for the comparison that is meant to
#'              be created with the factory method vrf_comparator.
#' @param x.chr character text representation of x, typically generated with
#'              the as character
#' @param omit  all lines containing the omit string will be excluded from the
#'              comparison (detaulf = NULL)
#'
#' @keywords internal

my_finalizer <- function(x, x.chr, omit) {

  split <- strsplit(x.chr, "<div class='diffobj-row'>")[[1]]
  width <- nchar(length(split))
  index <- 0

  # add row numbers to compare results manually for diffChr. diffobj-header
  # values need to be processed to identify the actual line numbers in summary
  # results.
  for (i in seq_along(split)) {
    row <- split[[i]]

    # remove the special note regarding whitespace ignoring as it is not
    # relevant with the custom whitespace ignoring supported.
    if (1 == i) {
      old <- paste0(
        "<div class='diffobj-container light rgb'>",
        "<pre class='diffobj-content'>",
        "No visible differences between objects, but there are some ",
        "differencessuppressed by `ignore.white.space`, ",
        "`convert.hz.white.space`, `strip.sgr`,and/or `trim`. Set all ",
        "those arguments to FALSE to highlight the differences."
      )

      if (old == row) {
        split[[i]] <- paste0(
          "<div class='diffobj-container light rgb'>",
          "<pre class='diffobj-content'>",
          "No visible differences between objects."
        )
      }
    }

    if (grepl("<div class='diffobj-header'>@@ .*@@</div>", row)) {
      index <- as.integer(sub(".*@@\\s*([0-9]+),.*@@.*", "\\1", row)) - 1
    }

    if (i > 3) {
      if (grepl("<div class='diffobj-header'>@@ .*@@</div>", row)) {
        index <- as.integer(sub(".*@@\\s*([0-9]+),.*@@.*", "\\1", row)) - 1
      } else {
        index <- index + 1
      }
      row_str <- sprintf("%*s", width + 3, sprintf("[%d] ", index))

      row <- gsub(
        "<div class='diffobj-text'><div class='diffobj-match'>",
        paste0(
          "<div class='diffobj-text'><div class='diffobj-match'>",
          "<span class='diffobj-trim'>",
          row_str,
          "</span>"
        ),
        row
      )

      row <- gsub(
        "<div class='diffobj-text'><div class='delete'>",
        paste0(
          "<div class='diffobj-text'><div class='delete'>",
          "<span class='diffobj-trim'>",
          row_str,
          "</span>"
        ),
        row
      )

      row <- gsub(
        "<div class='diffobj-text'><div class='insert'>",
        paste0(
          "<div class='diffobj-text'><div class='insert'>",
          "<span class='diffobj-trim'>",
          row_str,
          "</span>"
        ),
        row
      )

      split[[i]] <- row
    }
  }

  # custom processing for omit row styles
  if (!is.null(omit) && "" != paste0(omit)) {
    for (i in seq_along(split)) {
      if (grepl(omit, split[[i]])) {
        row <- split[[i]]

        # modifying maching row markup
        row <- gsub(
          "class='diffobj-match'",
          "class='ignore'",
          row
        )

        row <- gsub(
          "<div class='diffobj-gutter'><div class='ignore'>&nbsp;",
          "<div class='diffobj-gutter'><div class='ignore'>x",
          row
        )

        # modifying inserted row markup
        row <- gsub(
          "class='insert'",
          "class='ignore'",
          row
        )

        row <- gsub(
          "class='diffobj-word insert'",
          "class='diffobj-word ignore'",
          row
        )

        row <- gsub(
          "<div class='diffobj-gutter'><div class='ignore'>&gt;",
          "<div class='diffobj-gutter'><div class='ignore'>X",
          row
        )

        # modifying deleted row markup
        row <- gsub(
          "class='delete'",
          "class='ignore'",
          row
        )

        row <- gsub(
          "class='diffobj-word delete'",
          "class='diffobj-word ignore'",
          row
        )

        row <- gsub(
          "<div class='diffobj-gutter'><div class='ignore'>&lt;",
          "<div class='diffobj-gutter'><div class='ignore'>X",
          row
        )

        # highlight the ommitted part
        row <- gsub(
          omit,
          paste0(
            "<span class='diffobj-word-highlight ignore'>",
            omit,
            "</span>"
          ),
          row
        )

        split[[i]] <- row
      }
    }
  }

  html_string <- paste(split, collapse = "<div class='diffobj-row'>")

  diffobj::finalizeHtml(x, html_string)
}

#' One row list for two distinct files
#'
#' \code{list_files} List single file row based on the explicit parameter
#' files. This is a conveniency function for building same structure list for
#' direct two file comparison case.
#'
#' @param file1 character, giving the the full file path of the first file
#' @param file2 character, giving the the full file path of the second file
#'
#' @return Returns a tibble, \code{selected_files} with 2 columns \code{file1},
#'                 \code{file2}
#'
#' @examples
#'
#' path1 <- "/extdata/base_files/file2_additional_rows.rtf"
#' file1 <- paste0(fs::path_package(path1, package = "verifyr2"))
#'
#' path2 <- "/extdata/compare_files/file3_changed_rows.rtf"
#' file2 <- paste0(fs::path_package(path2, package = "verifyr2"))
#'
#' verifyr2::list_files(file1, file2)
#'
#' @export

list_files <- function(file1, file2) {

  ## do the comparison only if both of the files exist
  if (file.exists(file1) && file.exists(file2)) {

    data <- tibble::tibble(
      "file1" = file1,
      "file2" = file2
    )

    return(data)
  }

  NULL
}

#' BinaryFileComparator.R
#'
#' Fallback comparator for binary files without any specific definied
#' comparator.
#'
#' @include FileComparator.R
#'
#' @examples
#'
#' # The normal way for creating a comparator would be to call the generic
#' # factory method verifyr2::create_comparator that will automatically create
#' # the correct comparator instance based on the file types.
#'
#' file1 <- 'my_file1.bin'
#' file2 <- 'my_file2.bin'
#' comparator <- verifyr2::create_comparator(file1, file2)
#'
#' # If needed, an explicit comparator can be created as well.
#'
#' file1 <- 'my_file1.bin'
#' file2 <- 'my_file2.bin'
#' comparator <- BinaryFileComparator$new(file1, file2)
#'
#' @export
#'
BinaryFileComparator <- R6::R6Class(
  "BinaryFileComparator",
  inherit = FileComparator,
  public = list(

    #' @description
    #' Method for getting the single file contents for the comparison. This
    #' method returns the file contents in two separate vectors inside a list.
    #' The first vector is the file contents and the second one is the file
    #' contents with the rows matching the omit string excluded. This method
    #' can be overwritten by more specialized comparator classes. This method
    #' is intended to be called only by the comparator classes in the processing
    #' and shouldn not be called directly by the user.
    #'
    #' @param file   file for which to get the contents
    #' @param config configuration values
    #' @param omit   string pattern to omit from the comparison
    #'
    vrf_contents = function(file, config, omit) {
      self$vrf_open_debug("Binary::vrf_contents", config)

      contents <- readLines(file, warn = FALSE)
      result   <- self$vrf_contents_inner(contents, config, omit)

      self$vrf_close_debug()
      result
    },

    #' @description
    #' Method for getting the inner part for the file contents query. The method
    #' returns the file contents in two separate vectors inside a list. The
    #' first vector is the file contents and the second one is the file contents
    #' with the rows matching the omit string excluded. This method can be
    #' overwritten by more specialized comparator classes. This method is
    #' intended to be called only by the comparator classes in the processing
    #' and shouldn not be called directly by the user.
    #'
    #' @param contents file contents
    #' @param config   configuration values
    #' @param omit     string pattern to omit from the comparison
    #'
    vrf_contents_inner = function(contents, config, omit) {
      self$vrf_add_debug("Binary::vrf_contents_inner")
      list(contents, contents)
    },

    #' @description
    #' Method for comparing the inner part for the details query. The method
    #' returns the file contents in two separate vectors inside a list. The
    #' first vector is the file contents and the second one is the file contents
    #' processed for empty spaces and omit terms if applicable. This method can
    #' be overwritten by more specialized comparator classes. This method is
    #' intended to be called only by the comparator classes in the processing
    #' and shouldn not be called directly by the user.
    #'
    #' @param config configuration values
    #' @param omit   string pattern to omit from the comparison
    #'
    vrf_summary_inner = function(config, omit) {
      self$vrf_open_debug("Binary::vrf_summary_inner", config)

      file_info1 <- file.info(self$file1)
      file_info2 <- file.info(self$file2)

      if (file_info1$size != file_info2$size) {
        self$vrf_close_debug()
        return("Different file sizes for compared files.")
      }

      file1_contents_list <- self$file1_contents_list
      file2_contents_list <- self$file2_contents_list

      if (is.null(file1_contents_list)) {
        file1_contents_list <- self$vrf_contents(self$file1, config, omit)
        self$file1_contents_list <- file1_contents_list
      }

      if (is.null(file2_contents_list)) {
        file2_contents_list <- self$vrf_contents(self$file2, config, omit)
        self$file2_contents_list <- file2_contents_list
      }

      file1_contents_processed <- file1_contents_list[[2]]
      file2_contents_processed <- file2_contents_list[[2]]

      if (!identical(file1_contents_processed, file2_contents_processed)) {
        self$vrf_close_debug()
        return("Different content in compared files.")
      }

      self$vrf_close_debug()
      "No differences."
    },

    #' @description
    #' Method for comparing the inner part for the details query. This method
    #' can be overwritten by more specialized comparator classes. This method is
    #' intended to be called only by the comparator classes in the processing
    #' and shouldn't be called directly by the user.
    #'
    #' @param config configuration values
    #' @param omit   string pattern to omit from the comparison
    #'
    vrf_details_inner = function(config, omit) {
      self$vrf_add_debug("Binary::vrf_details_inner")
      result <- list(
        type = "text",
        contents = "Binary file without applicable comparator."
      )
      list(result)
    }
  )
)

#' Debugger.R
#'
#' Class for managing the library debugging. Tracks the debugged method
#' execution times and prints out the full debugging information only when
#' the main debugging instance is stopped.
#'
#' @examples
#'
#' # Creates a debugger instance.
#'
#' debugger <- Debugger$new()
#'
#' # Opening and closing debugs in multileveled function calls.
#'
#' function1 <- function() {
#'   debugger$open_debug("function1")
#'   function2()
#'   debuger$close_debug()
#' }
#'
#' function2 <- function() {
#'   debugger$open_debug("function2")
#'   Sys.sleep(1)
#'   debugger$close_debug()
#' }
#'
#' # This will produce the following printout to the console after the
#' # function1 finishes
#' #
#' # - 'function1' (execution time 7 ms)
#' #   - 'function2' (execution time 5 ms)
#'
#' @importFrom R6 R6Class
#'
#' @field stack local property for storing debugging labels and start times
#'
#' @export
#'
Debugger <- R6::R6Class("Debugger",
  public = list(
    stack = list(),

    #' @description
    #' Constructor for initializing the Debugger instance.
    #'
    initialize = function() {
      self$stack <- list()
    },

    #' @description
    #' Method for opening new debug instance to the current debugging stack.
    #' Stores also the start time for execution time calculation.
    #'
    #' @param label debugging message
    #'
    open_debug = function(label) {
      entry <- list(
        label      = label,
        start_time = Sys.time(),
        children   = list(),
        duration   = NULL,
        is_message = FALSE
      )
      self$stack[[length(self$stack) + 1]] <- entry
    },

    #' @description
    #' Method for adding a new debug string under the currently open
    #' debug instance.
    #'
    #' @param label debugging message
    #'
    add_debug = function(label) {
      msg <- list(
        label      = label,
        children   = list(),
        duration   = NULL,
        is_message = TRUE
      )

      parent <- self$stack[[length(self$stack)]]
      parent$children[[length(parent$children) + 1]] <- msg
      self$stack[[length(self$stack)]] <- parent
    },

    #' @description
    #' Method for closing a debug instance from the current debugging
    #' stack. If the stopped debug instance is the main level one, the
    #' whole debug data is printed out to console. If the stopped debug
    #' instance is not the main level one, calculates the execution time
    #' of current debug instance and updates the stack data.
    #'
    close_debug = function() {
      # Pop the last entry
      entry <- self$stack[[length(self$stack)]]
      self$stack <- self$stack[-length(self$stack)]

      time_diff      <- difftime(Sys.time(), entry$start_time, units = "secs")
      entry$duration <- round(as.numeric(time_diff) * 1000)

      if (length(self$stack) > 0) {
        parent <- self$stack[[length(self$stack)]]
        parent$children[[length(parent$children) + 1]] <- entry
        self$stack[[length(self$stack)]] <- parent
      } else {
        self$print_debug_tree(entry)
        cat("[DEBUG]'\n")
      }
    },

    #' @description
    #' Recursive method for printing out the current debug stack items and
    #' recursively all the item children. This method is called for the whole
    #' stack once the topmost debug instance is stopped.
    #'
    #' @param entry current debug level being processed for printing
    #' @param depth current processing depth for printing indentation
    #'
    print_debug_tree = function(entry, depth = 0) {
      indent <- paste("[DEBUG]", strrep("  ", depth))
      if (isTRUE(entry$is_message)) {
        cat(indent, "'", entry$label, "'\n", sep = "")
      } else {
        execution_time <- paste0("(execution time ", entry$duration, " ms)")
        cat(indent, "'", entry$label, "' ", execution_time, "\n", sep = "")

        for (child in entry$children) {
          self$print_debug_tree(child, depth + 1)
        }
      }
    }
  )
)

################################################################################
# Test debugger simple open/close methods
################################################################################

test_that("Debugger printout", {
  debugger <- Debugger$new()

  output <- capture.output({
    debugger$open_debug("opened1")
    debugger$close_debug()
  })

  expected <- "[DEBUG] 'opened1' (execution time"
  expect_content(expected, output)
})

test_that("Debugger printout with add method", {
  debugger <- Debugger$new()

  output <- capture.output({
    debugger$open_debug("opened2")
    debugger$add_debug("additional2")
    debugger$close_debug()
  })

  expected <- "[DEBUG] 'opened2' (execution time"
  expect_content(expected, output)

  expected <- "[DEBUG]   'additional2'"
  expect_content(expected, output)
})

test_that("Debugger printout with multiple levels", {
  debugger <- Debugger$new()

  output <- capture.output({
    debugger$open_debug("opened3")
    debugger$open_debug("internal-opened4")
    debugger$open_debug("internal-opened5")
    debugger$close_debug()
    debugger$add_debug("additional3")
    debugger$close_debug()
    debugger$close_debug()
  })

  expected <- "[DEBUG] 'opened3' (execution time"
  expect_content(expected, output)

  expected <- "[DEBUG]   'internal-opened4' (execution time"
  expect_content(expected, output)

  expected <- "[DEBUG]     'internal-opened5' (execution time"
  expect_content(expected, output)

  expected <- "[DEBUG]     'additional3'"
  expect_content(expected, output)
})

#' ImgFileComparator.R
#'
#' Specialiced comparator for image file (jpg, jpef, png)  comparison.
#' This comparator contains the custom handling for handling only img content
#' part for the comparison.
#'
#' @import base64enc
#'
#' @include BinaryFileComparator.R
#'
#' @field image1_raw extracted raw data for image1.
#' @field image2_raw extracted raw data for image2.
#'
#' @examples
#'
#' # The normal way for creating a comparator would be to call the generic
#' # factory method verifyr2::create_comparator that will automatically create
#' # the correct comparator instance based on the file types.
#'
#' file1 <- 'my_file1.jpg'
#' file2 <- 'my_file2.jpg'
#' comparator <- verifyr2::create_comparator(file1, file2)
#'
#' # If needed, an explicit comparator can be created as well.
#'
#' file1 <- 'my_file1.png'
#' file2 <- 'my_file2.png'
#' comparator <- ImgFileComparator$new(file1, file2)
#'
#' # This comparator has also second explicit creation method that is used
#' # by the library for processing embedded image contents.
#'
#' image1_raw <- 'raw hex vector data'
#' image2_raw <- 'raw hex vector data'
#' comparator <- ImgFileComparator$new(NULL, NULL, image1_raw, image2_raw)
#'
#' @export
#'
ImgFileComparator <- R6::R6Class(
  "ImgFileComparator",
  inherit = BinaryFileComparator,
  public = list(
    image1_raw = NULL,
    image2_raw = NULL,

    #' @description
    #' Initialize a ImgFileComparator instance
    #'
    #' @param file1 First file to compare.
    #' @param file2 Second file to compare.
    #' @param raw1  First image in raw format to compare.
    #' @param raw2  Second image in raw format to compare.
    #'
    initialize = function(
      file1 = NULL,
      file2 = NULL,
      raw1  = NULL,
      raw2  = NULL
    ) {
      self$image1_raw <- raw1
      self$image2_raw <- raw2
      super$initialize(file1, file2)
    },

    #' @description
    #' Method for comparing the inner part for the details query. This method
    #' can be overwritten by more specialized comparator classes. This method is
    #' intended to be called only by the comparator classes in the processing
    #' and shouldn't be called directly by the user.
    #'
    #' @param config configuration values
    #' @param omit   string pattern to omit from the comparison
    #'
    vrf_details_inner = function(config, omit) {
      self$vrf_open_debug("Img::vrf_details_inner", config)

      if ("no" == super$vrf_option_value(config, "generic.images")) {
        result <- list(
          list(
            type = "text",
            contents = "Image comparison disabled; no comparison done."
          )
        )

        self$vrf_close_debug()
        return(result)
      }

      if (is.null(self$image1_raw) || is.null(self$image2_raw)) {
        result <- self$vrf_details_inner_from_files(config)

        self$vrf_close_debug()
        return(result)
      }

      result <- self$vrf_details_inner_from_raw(config)

      self$vrf_close_debug()
      result
    },

    #' @description
    #' Internal method for comparing the earlier populated raw image contents in
    #' details and generating the difference highlight image in case differences
    #' are found.
    #'
    #' @param config configuration values
    #*
    vrf_details_inner_from_raw = function(config) {
      self$vrf_open_debug("Img::vrf_details_inner_from_raw", config)

      png_prefix <- "data:image/png;base64,"
      image1_raw <- self$image1_raw
      image2_raw <- self$image2_raw
      image3_base64 <- NULL

      if (!identical(image1_raw, image2_raw)) {
        image1 <- magick::image_read(image1_raw)
        image2 <- magick::image_read(image2_raw)

        diff      <- magick::image_compare(image1, image2, metric = "AE")
        highlight <- magick::image_composite(image1, diff, operator = "atop")

        image3_raw    <- magick::image_write(highlight, format = "png")
        image3_base64 <- paste0(png_prefix, base64enc::base64encode(image3_raw))
      }

      image1_base64 <- paste0(png_prefix, base64enc::base64encode(image1_raw))
      image2_base64 <- paste0(png_prefix, base64enc::base64encode(image2_raw))

      result <- list(
        type = "image",
        contents = list(
          image1 = image1_base64,
          image2 = image2_base64,
          image3 = image3_base64
        )
      )

      self$vrf_close_debug()
      list(result)
    },

    #' @description
    #' Method for comparing the inner part for the details query with the file
    #' names as the base arguments. This is a part of a group of image
    #' processing functions that work with different image abstractions (file,
    #' image, raw image). These methods are intended to improve the performance
    #' so that best suiting method version can be used depending on what data is
    #' available from the earlier method calls to the same comparator instance.
    #'
    #' @param config configuration values
    #'
    vrf_details_inner_from_files = function(config) {
      self$vrf_open_debug("Img::vrf_details_inner_from_files", config)

      file1_size <- file.info(self$file1)$size
      self$image1_raw <- readBin(self$file1, what = "raw", n = file1_size)

      file2_size <- file.info(self$file2)$size
      self$image2_raw <- readBin(self$file2, what = "raw", n = file2_size)

      result <- self$vrf_details_inner_from_raw(config)

      self$vrf_close_debug()
      result
    },

    #' @description
    #' Inherited method for indicating whether detailed comparison is available
    #' with the current comparator. Returns an empty string if the comparator is
    #' is supported, otherwise a string that will be concatenated with the
    #' summary string.
    #'
    #' @param config configuration values
    #'
    vrf_details_supported = function(config) {
      if ("no" == super$vrf_option_value(config, "generic.images")) {
        return("Image details comparison disabled.")
      }
      ""
    }
  )
)

generic_level_desc <- paste0(
  "Option to define whether the comparison is processed and displayed on ",
  "word or row level. Word-level processing requires additional time for ",
  "comparison calculation, so it may be advisable to disable this option ",
  "when working with large files if row-level comparison is sufficient."
)

generic_images_desc <- paste0(
  "Option to define whether embedded images should be processed for all ",
  "supported file types. Processing embedded images requires additional ",
  "time to detect and extract images from documents, so it may be ",
  "advisable to disable this option when working with large files or a ",
  "high volume of files that do not contain images."
)

generic_spaces_desc <- paste0(
  "Option to define whether differences in whitespace should be ignored ",
  "during comparison. When enabled, differences caused solely by additional ",
  "spaces or tab characters are not reported."
)

generic_debug_desc <- paste0(
  "Option to enable debugging so that the application prints diagnostic ",
  "information to the output stream. This option is available to all users ",
  "and can be enabled to include debugging details when reporting issues."
)

rtf_images_desc <- paste0(
  "Option to define whether embedded images should be processed for RTF file ",
  "types. Processing embedded images requires additional time to detect and ",
  "extract images from documents, so it may be advisable to disable this ",
  "option when working with large files or a high volume of files that do ",
  "not contain images."
)

pdf_details_desc <- paste0(
  "Option to define whether detailed PDF comparison should be performed. ",
  "Disabling this option is generally not recommended for users. However, it ",
  "may be automatically disabled by the application if the required ",
  "'pdftools' package is not available."
)

details_mode_desc <- paste0(
  "Option to define the default display mode for the details summary view. ",
  "The available options are 'full,' which displays the complete compared ",
  "file, and 'summary,' which displays only the detected differences."
)

merge_values <- function(defaults, overrides) {
  result <- list()

  for (name in names(defaults)) {
    def <- defaults[[name]]
    over <- overrides[[name]]

    has_defaults <- "default" %in% names(def)
    has_options  <- "options" %in% names(def)

    if (is.list(def) && has_defaults && has_options) {
      if (!is.null(over) && over %in% def$options) {
        result[[name]] <- over
      } else {
        result[[name]] <- def$default
      }
    } else if (is.list(def)) {
      if (!is.list(over)) over <- list()
      result[[name]] <- merge_values(def, over)
    } else {
      result[[name]] <- if (!is.null(over)) over else def
    }
  }

  result
}

get_nested_value <- function(config, key) {
  parts <- strsplit(key, ".", fixed = TRUE)[[1]]

  for (p in parts) {
    config <- config[[p]]
  }
  config
}

set_nested_value <- function(config, key, value) {
  parts <- strsplit(key, ".", fixed = TRUE)[[1]]

  if (length(parts) == 1) {
    config[[parts[1]]] <- value
  } else {
    sub_key <- paste(parts[-1], collapse = ".")
    config[[parts[1]]] <- set_nested_value(config[[parts[1]]], sub_key, value)
  }
  config
}

check_magick_available <- function() {
  requireNamespace("magick", quietly = TRUE)
}

check_pdftools_available <- function() {
  requireNamespace("pdftools", quietly = TRUE)
}

#' Config.R
#'
#' Class for manging the library configuration options. Creates the default
#' configuration without any source file, populates partial or missing config
#' elements, stores the config file to local machine, and provides easy access
#' methods for setting and getting config values.
#'
#' @examples
#'
#' # Creates the configuration instance. Checks automatically if there is
#' # a previously stored configuration json file available for usage. Note
#' # that you don't need to explicitly define the config file path for the
#' # Config instance - by default the config file will be searched from and
#' # written in user-specific configuration directory for the package.
#'
#' path   <- tempfile(fileext = ".json")
#' config <- Config$new(config_path = path)
#'
#' # Getting and setting configuration values
#'
#' value <- config$get("defailts.mode")
#' config$set("details.mode", "full")
#'
#' # Saving the current configuration to local machine (to tmp folder with
#' # the given explicit file path in initialization).
#'
#' config$save()
#'
#' @import jsonlite
#' @import rappdirs
#' @importFrom R6 R6Class
#'
#' @field schema configuration schema
#' @field config current configuration data
#' @field path   configuration json file path
#'
#' @export
#'
Config <- R6::R6Class(
  "Config",
  public = list(
    schema = NULL,
    config = NULL,
    path   = NULL,

    #' @description
    #' Constructor for initializing the configuration. Checks the local machine
    #' for existing configuration file is load_config = TRUE. Ensures that all
    #' the project configuration values are included.
    #'
    #' @param load_config load configuration from local machine if available
    #' @param config_path location of the used/stored configuration json file
    #'
    initialize = function(load_config = TRUE, config_path = NULL) {
      self$schema <- self$get_default_schema()
      self$config <- self$get_default_config()

      if (is.null(config_path)) {
        config_dir <- rappdirs::user_config_dir("verifyr2")
        self$path  <- file.path(config_dir, "config.json")
      } else {
        self$path <- config_path
      }

      if (load_config && file.exists(self$path)) {
        file_config <- jsonlite::read_json(self$path, simplifyVector = TRUE)
        self$config <- merge_values(self$get_default_schema(), file_config)
        self$config <- self$get_default_config() |> merge_values(self$config)
      }
    },

    #' @description
    #' Method for getting schema item based on configuration key.
    #' Configuration item children are separated with a dot in the key notation.
    #'
    #' @param key configuration property key for which to get the schema item
    #'
    get_schema_item = function(key) {
      get_nested_value(self$schema, key)
    },

    #' @description
    #' Method for getting configuration value based on configuration key.
    #' Configuration item children are separated with a dot in the key notation.
    #'
    #' @param key configuration property key for which to get the value
    #'
    get = function(key) {
      get_nested_value(self$config, key)
    },

    #' @description
    #' Method for setting configuration value based on configuration key.
    #' Configuration item children are separated with a dot in the key notation.
    #'
    #' @param key   configuration property key for which to get the value
    #' @param value value to set for the specified configuration key
    #'
    set = function(key, value) {
      self$config <- set_nested_value(self$config, key, value)
    },

    #' @description
    #' Method for saving the current configuration data into local machine.
    #'
    save = function() {
      dir.create(dirname(self$path), showWarnings = FALSE, recursive = TRUE)
      jsonlite::write_json(self$config, self$path, pretty = TRUE)
    },

    #' @description
    #' Helper method for getting configuration default values. These default
    #' values will be used in the configuration in case the configuration
    #' properties are not present previously.
    #'
    get_default_config = function() {
      defaults <- list()

      for (group in names(self$schema)) {
        defaults[[group]] <- list()

        for (key in names(self$schema[[group]])) {
          if (key != "title") {
            defaults[[group]][[key]] <- self$schema[[group]][[key]]$default
          }
        }
      }
      defaults
    },

    #' @description
    #' Method for getting the full configuration schema. Apart from the
    #' configuration data, the schema contains property descriptions as well as
    #' all possible values for the configuration properties.
    #'
    get_default_schema = function() {
      schema <- list(
        generic = list(
          title = "Generic options",
          level = list(
            title   = "Comparison level",
            options = c("word", "row"),
            default = "word",
            reload  = TRUE,
            desc    = generic_level_desc
          ),
          spaces = list(
            titlei  = "Ignore empty space differences",
            options = c("yes", "no"),
            default = "no",
            reload  = TRUE,
            desc    = generic_spaces_desc
          ),
          images = list(
            title   = "Process embedded images",
            options = c("yes", "no"),
            default = "yes",
            reload  = TRUE,
            desc    = generic_images_desc
          ),
          debug = list(
            title   = "Debugging enabled",
            options = c("yes", "no"),
            default = "no",
            reload  = FALSE,
            desc    = generic_debug_desc
          )
        ),
        rtf = list(
          title = "RTF comparison (summary and details)",
          images = list(
            title   = "Process embedded images",
            options = c("yes", "no"),
            default = "yes",
            reload  = TRUE,
            desc    = rtf_images_desc
          )
        ),
        pdf = list(
          title = "PDF comparison (summary)",
          details = list(
            title = "Process PDF detailed comparison",
            options = c("yes", "no"),
            default = "yes",
            reload  = TRUE,
            desc    = pdf_details_desc
          )
        ),
        details = list(
          title = "Details comparison",
          mode = list(
            title = "Mode",
            options = c("full", "summary"),
            default = "summary",
            reload  = FALSE,
            desc    = details_mode_desc
          )
        )
      )

      if (!check_magick_available()) {
        schema[["generic"]][["images"]] <- list(
          title   = "Process embedded images (missing magick library)",
          options = c("no"),
          default = "no",
          reload  = TRUE,
          desc    = generic_images_desc
        )

        schema[["rtf"]][["images"]] <- list(
          title   = "Process embedded images (missing magick library)",
          options = c("no"),
          default = "no",
          reload  = TRUE,
          desc    = rtf_images_desc
        )
      }

      if (!check_pdftools_available()) {
        schema[["pdf"]][["details"]] <- list(
          title   = "Process PDF details (missing pdftools library)",
          options = c("no"),
          default = "no",
          reload  = TRUE,
          desc    = pdf_details_desc
        )
      }

      schema
    }
  )
)

base <- "test_configs"

################################################################################
# Test configuration json loading
################################################################################

test_that("Configuration json file loaded successfully", {
  path   <- testthat::test_path(base, "config1.json")
  config <- Config$new(config_path = path)

  expect_equal(config$get("generic.debug"), "yes")
  expect_equal(config$get("generic.spaces"), "yes")
  expect_equal(config$get("generic.level"), "row")
  expect_equal(config$get("rtf.images"), "no")
  expect_equal(config$get("details.mode"), "full")
  expect_equal(config$get("pdf.details"), "yes")
})

test_that("Partial configuration successfully updated with defaults", {
  path   <- testthat::test_path(base, "config2.json")
  config <- Config$new(config_path = path)

  expect_equal(config$get("generic.debug"), "no")
  expect_equal(config$get("generic.spaces"), "no")
  expect_equal(config$get("generic.level"), "word")
  expect_equal(config$get("rtf.images"), "no")
  expect_equal(config$get("details.mode"), "summary")
  expect_equal(config$get("pdf.details"), "yes")
})

################################################################################
# Test configuration invalid value in saved file
################################################################################

test_that("Configuration json file loaded successfully", {
  path   <- testthat::test_path(base, "config1.json")

  # mock the magick available method to return false to replicate situation
  # that magick library is not installed.
  local_mocked_bindings(
    check_magick_available = function() FALSE
  )

  config <- Config$new(config_path = path)

  expect_equal(config$get("generic.debug"), "yes")
  expect_equal(config$get("generic.images"), "no")
  expect_equal(config$get("rtf.images"), "no")
  expect_equal(config$get("details.mode"), "full")
  expect_equal(config$get("pdf.details"), "yes")
})

################################################################################
# Test configuration saving
################################################################################

test_that("Configuration json file loaded successfully", {
  file_path <- tempfile(fileext = ".json")
  config    <- Config$new(load_config = FALSE, config_path = file_path)

  config$save()

  expect_true(file.exists(file_path))
  unlink(file_path)
})

base   <- "test_outputs/txt"
config <- Config$new(FALSE)

################################################################################
# Generic file existence checks
################################################################################

test_that(paste(
  "Returns 'File(s) not available; unable to compare.'",
  "if both files do not exist"
), {
  file1 <- testthat::test_path(base, "nonexisting1.txt")
  file2 <- testthat::test_path(base, "nonexisting2.txt")

  comparator <- create_comparator(file1, file2)
  result     <- comparator$vrf_summary(config = config)

  expect_equal(result, "File(s) not available; unable to compare.")
})

test_that(paste(
  "Returns 'File(s) not available; unable to compare.'",
  "if one file does not exist"
), {
  file1 <- testthat::test_path(base, "base.txt")
  file2 <- testthat::test_path(base, "nonexisting.txt")

  comparator <- create_comparator(file1, file2)
  result     <- comparator$vrf_summary(config = config)

  expect_equal(result, "File(s) not available; unable to compare.")
})

################################################################################
# Text file comparison - WITHOUT omit
################################################################################

test_that(paste(
  "Returns 'No differences' for identical files."
), {
  file1 <- testthat::test_path(base, "base.txt")
  file2 <- testthat::test_path(base, "copy.txt")

  comparator <- create_comparator(file1, file2)
  result     <- comparator$vrf_summary(config = config)

  expect_equal(result, "No differences.")
})

test_that(paste(
  "Returns 'Different number of lines in compared content.'",
  "for files with additional line in the content"
), {
  file1 <- testthat::test_path(base, "base.txt")
  file2 <- testthat::test_path(base, "addition_one_row.txt")

  comparator <- create_comparator(file1, file2)
  result     <- comparator$vrf_summary(config = config)

  expect_equal(result, "Different number of lines in compared content.")
})

test_that(paste(
  "Returns 'File content has changes in 1 place(s).'",
  "for files with a single different row in content"
), {
  file1 <- testthat::test_path(base, "changes_one_row.txt")
  file2 <- testthat::test_path(base, "base.txt")

  comparator <- create_comparator(file1, file2)
  result     <- comparator$vrf_summary(config = config)

  expect_equal(result, "File content has changes in 1 place(s).")
})

test_that(paste(
  "Returns 'File content has changes in 2 place(s).'",
  "for files with two different rows in content"
), {
  file1 <- testthat::test_path(base, "changes_two_rows.txt")
  file2 <- testthat::test_path(base, "base.txt")

  comparator <- create_comparator(file1, file2)
  result     <- comparator$vrf_summary(config = config)

  expect_equal(result, "File content has changes in 2 place(s).")
})

test_that(paste(
  "Returns 'No differences' for two files with differences in spaces",
  "with ignore spaces option = 'yes'"
), {
  file1 <- testthat::test_path(base, "additional_spaces_and_tabs.txt")
  file2 <- testthat::test_path(base, "base.txt")

  config_local <- Config$new(FALSE)
  config_local$set("generic.spaces", "yes")

  comparator <- create_comparator(file1, file2)
  result     <- comparator$vrf_summary(config = config_local)

  expect_equal(result, "No differences.")
})

test_that(paste(
  "Returns 'File content has changes in 3 place(s).'",
  "for files with differences in spaces with ignore",
  "spaces option = 'no'"
), {
  file1 <- testthat::test_path(base, "additional_spaces_and_tabs.txt")
  file2 <- testthat::test_path(base, "base.txt")

  config_local <- Config$new(FALSE)
  config_local$set("generic.spaces", "no")

  comparator <- create_comparator(file1, file2)
  result     <- comparator$vrf_summary(config = config_local)

  expect_equal(result, "File content has changes in 3 place(s).")
})

################################################################################
# Text file comparison - WITH omit
################################################################################

test_that(paste(
  "Returns 'No differences.'",
  "for comparison with 'omit' parameter that is not present in either file"
), {
  file1 <- testthat::test_path(base, "base.txt")
  file2 <- testthat::test_path(base, "copy.txt")

  comparator <- create_comparator(file1, file2)
  result     <- comparator$vrf_summary(config = config, omit = "Nothing")

  expect_equal(result, "No differences.")
})

test_that(paste(
  "No differences'",
  "when called with 'omit' catching the one row changed row in content"
), {
  file1 <- testthat::test_path(base, "base.txt")
  file2 <- testthat::test_path(base, "changes_one_row.txt")

  comparator <- create_comparator(file1, file2)
  result     <- comparator$vrf_summary(config = config, omit = "Line 2")

  expect_equal(result, "No differences.")
})

test_that(paste(
  "Returns 'No differences.'",
  "when called with 'omit' catching the one additional row in content"
), {
  file1 <- testthat::test_path(base, "base.txt")
  file2 <- testthat::test_path(base, "addition_one_row.txt")

  comparator <- create_comparator(file1, file2)
  result     <- comparator$vrf_summary(config = config, omit = "Line 4")

  expect_equal(result, "No differences.")
})

test_that(paste(
  "Returns 'No differences.'",
  "when called with 'omit' catching the two additional rows in content"
), {
  file1 <- testthat::test_path(base, "base.txt")
  file2 <- testthat::test_path(base, "addition_two_rows.txt")

  comparator <- create_comparator(file1, file2)
  result     <- comparator$vrf_summary(config = config, omit = "Line 4")

  expect_equal(result, "No differences.")
})

test_that(paste(
  "Returns 'Different number of lines in compared content.'",
  "when called with 'omit' catching one of the two additional rows"
), {
  file1 <- testthat::test_path(base, "base.txt")
  file2 <- testthat::test_path(base, "addition_two_rows.txt")

  comparator <- create_comparator(file1, file2)
  result     <- comparator$vrf_summary(config = config, omit = "Line 41")

  expect_equal(result, "Different number of lines in compared content.")
})

test_that(paste(
  "Returns 'File content has changes in 1 place(s).'",
  "when called with 'omit' catching one of the two changed rows in content"
), {
  file1 <- testthat::test_path(base, "base.txt")
  file2 <- testthat::test_path(base, "changes_two_rows.txt")

  comparator <- create_comparator(file1, file2)
  result     <- comparator$vrf_summary(config = config, omit = "Line 2")

  expect_equal(result, "File content has changes in 1 place(s).")
})

base   <- "test_outputs/json"
config <- Config$new(FALSE)

################################################################################
# Generic file existence checks
################################################################################

test_that(paste(
  "Returns 'File(s) not available; unable to compare.' ",
  "if both of the files do not exist"
), {
  file1 <- testthat::test_path(base, "nonexisting1.json")
  file2 <- testthat::test_path(base, "nonexisting2.json")

  comparator <- create_comparator(file1, file2)
  result     <- comparator$vrf_details(config = config)[[1]]

  expect_equal(result$type, "text")
  expect_equal(result$contents, "File(s) not available; unable to compare.")
})

test_that(paste(
  "Returns 'File(s) not available; unable to compare.' ",
  "if one file does not exist"
), {
  file1 <- testthat::test_path(base, "base.json")
  file2 <- testthat::test_path(base, "nonexisting.json")

  comparator <- create_comparator(file1, file2)
  result     <- comparator$vrf_details(config = config)[[1]]

  expect_equal(result$type, "text")
  expect_equal(result$contents, "File(s) not available; unable to compare.")
})

################################################################################
# Simple tests that a S4 object is received for details comparison
################################################################################

test_that(paste(
  "Returns S4 comparison object for two files with same content"
), {
  file1 <- testthat::test_path(base, "base.json")
  file2 <- testthat::test_path(base, "copy.json")

  config <- Config$new(FALSE)
  config$set("details.mode", "summary")

  comparator <- create_comparator(file1, file2)
  result     <- comparator$vrf_details(config = config)[[1]]

  expect_equal(result$type, "text")
  expect_equal(typeof(result$contents), "S4")
})

test_that(paste(
  "Returns S4 comparison object for two files with differences in content"
), {
  file1 <- testthat::test_path(base, "base.json")
  file2 <- testthat::test_path(base, "modified.json")

  config <- Config$new(FALSE)
  config$set("details.mode", "full")

  comparator <- create_comparator(file1, file2)
  result     <- comparator$vrf_details(config = config)[[1]]

  expect_equal(result$type, "text")
  expect_equal(typeof(result$contents), "S4")

  html_output <- as.character(result$contents)
})

base   <- "test_outputs/rtf"
config <- Config$new(FALSE)

################################################################################
# Generic file existence checks
################################################################################

test_that(paste(
  "Returns 'File(s) not available; unable to compare.'",
  "if one file does not exist"
), {
  file1 <- testthat::test_path(base, "base.rtf")
  file2 <- testthat::test_path(base, "nonexisting.rtf")

  comparator <- create_comparator(file1, file2)
  result     <- comparator$vrf_summary(config = config)

  expect_equal(result, "File(s) not available; unable to compare.")
})

################################################################################
# WITHOUT omits
################################################################################

test_that(paste(
  "Returns 'No differences.' for identical files"
), {
  file1 <- testthat::test_path(base, "base.rtf")
  file2 <- testthat::test_path(base, "base.rtf")

  comparator <- create_comparator(file1, file2)
  result     <- comparator$vrf_summary(config = config)

  expect_equal(result, "No differences.")
})

test_that(paste(
  "Returns 'Different number of lines in compared content.'",
  "for files with additional line in the content"
), {
  file1 <- testthat::test_path(base, "base.rtf")
  file2 <- testthat::test_path(base, "addition_one_row_content.rtf")

  comparator <- create_comparator(file1, file2)
  result     <- comparator$vrf_summary(config = config)

  expect_equal(result, "Different number of lines in compared content.")
})

test_that(paste(
  "Returns 'Different number of lines in compared content.'",
  "for files with additional line in the content"
), {
  file1 <- testthat::test_path(base, "addition_one_row_content.rtf")
  file2 <- testthat::test_path(base, "base.rtf")

  comparator <- create_comparator(file1, file2)
  result     <- comparator$vrf_summary(config = config)

  expect_equal(result, "Different number of lines in compared content.")
})

test_that(paste(
  "Returns 'No differences.'",
  "for files with additional line in the footer"
), {
  file1 <- testthat::test_path(base, "base.rtf")
  file2 <- testthat::test_path(base, "addition_one_row_footer.rtf")

  comparator <- create_comparator(file1, file2)
  result     <- comparator$vrf_summary(config = config)

  expect_equal(result, "No differences.")
})

test_that(paste(
  "Returns 'No differences.'",
  "for files with additional line in the footer"
), {
  file1 <- testthat::test_path(base, "base.rtf")
  file2 <- testthat::test_path(base, "addition_one_row_footer.rtf")

  comparator <- create_comparator(file1, file2)
  result     <- comparator$vrf_summary(config = config)

  expect_equal(result, "No differences.")
})

test_that(paste(
  "Returns 'File content has changes in 1 place(s).'",
  "for files with a single different row in content"
), {
  file1 <- testthat::test_path(base, "changes_one_row_content.rtf")
  file2 <- testthat::test_path(base, "base.rtf")

  comparator <- create_comparator(file1, file2)
  result     <- comparator$vrf_summary(config = config)

  expect_equal(result, "File content has changes in 1 place(s).")
})

test_that(paste(
  "Returns 'No differences.'",
  "for files with a single different row in footer"
), {
  file1 <- testthat::test_path(base, "base.rtf")
  file2 <- testthat::test_path(base, "changes_one_row_footer.rtf")

  comparator <- create_comparator(file1, file2)
  result     <- comparator$vrf_summary(config = config)

  expect_equal(result, "No differences.")
})

test_that(paste(
  "Returns 'File content has changes in 2 place(s).'",
  "for files with two different rows in content"
), {
  file1 <- testthat::test_path(base, "changes_two_rows_content.rtf")
  file2 <- testthat::test_path(base, "base.rtf")

  comparator <- create_comparator(file1, file2)
  result     <- comparator$vrf_summary(config = config)

  expect_equal(result, "File content has changes in 2 place(s).")
})

test_that(paste(
  "Return 'No differences.'",
  "for files with two different rows in footer"
), {
  file1 <- testthat::test_path(base, "base.rtf")
  file2 <- testthat::test_path(base, "changes_two_rows_footer.rtf")

  comparator <- create_comparator(file1, file2)
  result     <- comparator$vrf_summary(config = config)

  expect_equal(result, "No differences.")
})

################################################################################
# WITH omits
################################################################################

test_that(paste(
  "Returns 'No differences.'",
  "with 'omit' parameter that is not present in either file"
), {
  file1 <- testthat::test_path(base, "base.rtf")
  file2 <- testthat::test_path(base, "copy.rtf")

  comparator <- create_comparator(file1, file2)
  result     <- comparator$vrf_summary(config = config, omit = "Nothing")

  expect_equal(result, "No differences.")
})

test_that(paste(
  "Return 'No differences.'",
  "with 'omit' catching the one row cells in changed rows in content"
), {
  file1 <- testthat::test_path(base, "base.rtf")
  file2 <- testthat::test_path(base, "changes_one_row_content.rtf")

  comparator <- create_comparator(file1, file2)
  result     <- comparator$vrf_summary(config = config, omit = "setosa")

  expect_equal(result, "No differences.")
})

test_that(paste(
  "Returns 'No differences.'",
  "with 'omit' catching the row name cell in additional row in content"
), {
  file1 <- testthat::test_path(base, "base.rtf")
  file2 <- testthat::test_path(base, "addition_one_row_content.rtf")

  comparator <- create_comparator(file1, file2)
  result     <- comparator$vrf_summary(config = config, omit = "setosa")

  expect_equal(result, "No differences.")
})

test_that(paste(
  "Returns 'No differences.'",
  "with 'omit' catching the two row cells in additional row in content"
), {
  file1 <- testthat::test_path(base, "base.rtf")
  file2 <- testthat::test_path(base, "addition_two_rows_content.rtf")

  comparator <- create_comparator(file1, file2)
  result     <- comparator$vrf_summary(config = config, omit = "unknown")

  expect_equal(result, "No differences.")
})

test_that(paste(
  "Returns 'Different number of lines in compared content.'",
  "with 'omit' catching one of the row cells in additional row in content"
), {
  file1 <- testthat::test_path(base, "base.rtf")
  file2 <- testthat::test_path(base, "addition_two_rows_content.rtf")

  comparator <- create_comparator(file1, file2)
  result     <- comparator$vrf_summary(config = config, omit = "unknown1")

  expect_equal(result, "Different number of lines in compared content.")
})

test_that(paste(
  "Returns 'File content has changes in 1 place(s).'",
  "with 'omit' catching one of the row cells in changed rows in content"
), {
  file1 <- testthat::test_path(base, "base.rtf")
  file2 <- testthat::test_path(base, "changes_two_rows_content.rtf")

  comparator <- create_comparator(file1, file2)
  result     <- comparator$vrf_summary(config = config, omit = "setosa")

  expect_equal(result, "File content has changes in 1 place(s).")
})

################################################################################
# CONTENT comparison with images
################################################################################

test_that(paste(
  "Returns 'No differences. No differences in embedded images.'",
  "for identical files"
), {
  file1 <- testthat::test_path(base, "base_with_image.rtf")
  file2 <- testthat::test_path(base, "base_with_image.rtf")

  comparator <- create_comparator(file1, file2)
  result     <- comparator$vrf_summary(config = config)

  if (requireNamespace("magick", quietly = TRUE)) {
    expect_equal(result, "No differences. No differences in embedded images.")
  } else {
    expect_equal(result, "No differences.")
  }
})

test_that(paste(
  "Returns 'File content has changes in 1 place(s). 1/1 embedded images",
  "have differences.' for files with a single different row in content",
  "and differences in the single embedded image"
), {
  file1 <- testthat::test_path(base, "changes_one_row_content_one_image.rtf")
  file2 <- testthat::test_path(base, "base_with_image.rtf")

  comparator <- create_comparator(file1, file2)
  result     <- comparator$vrf_summary(config = config)

  if (requireNamespace("magick", quietly = TRUE)) {
    expect_equal(result, paste(
      "File content has changes in 1 place(s).",
      "1/1 embedded images have differences."
    ))
  } else {
    expect_equal(result, "File content has changes in 1 place(s).")
  }
})

test_that(paste(
  "Returns 'File content has changes in 1 place(s)' for files with",
  "with a single different row in content and differences in the single",
  "embedded image (images disabled)"
), {
  file1 <- testthat::test_path(base, "changes_one_row_content_one_image.rtf")
  file2 <- testthat::test_path(base, "base_with_image.rtf")

  config_local <- Config$new(FALSE)
  config_local$set("rtf.images", "no")

  comparator <- create_comparator(file1, file2)
  result     <- comparator$vrf_summary(config = config_local)

  expect_equal(result, "File content has changes in 1 place(s).")
})

################################################################################
# CONTENT comparison with images - magick library not available
################################################################################

test_that(paste(
  "Returns 'File content has changes in 1 place(s)' for files with",
  "with a single different row in content and differences in the single",
  "embedded image (images disabled)"
), {
  file1 <- testthat::test_path(base, "changes_one_row_content_one_image.rtf")
  file2 <- testthat::test_path(base, "base_with_image.rtf")

  config_local <- Config$new(FALSE)
  config_local$set("rtf.images", "no")

  comparator <- create_comparator(file1, file2)
  result     <- comparator$vrf_summary(config = config_local)

  expect_equal(result, "File content has changes in 1 place(s).")
})

test_that(paste(
  "Returns 'File content has changes in 1 place(s).' for files with a single",
  "different row in content and embedded image differences with magick",
  "library not available"
), {
  file1 <- testthat::test_path(base, "changes_one_row_content_one_image.rtf")
  file2 <- testthat::test_path(base, "base_with_image.rtf")

  # mock the magick available method to return false to replicate situation
  # that magick library is not installed.
  local_mocked_bindings(
    check_magick_available = function() FALSE
  )

  config_local <- Config$new(FALSE)

  comparator <- create_comparator(file1, file2)
  result     <- comparator$vrf_summary(config = config_local)

  expect_equal(result, paste(
    "File content has changes in 1 place(s)."
  ))
})

################################################################################
# Corrupted RTF files
################################################################################

test_that(paste(
  "Returns 'File content has changes in 5 place(s).' for corrupted",
  "RTF files"
), {
  file1 <- testthat::test_path(base, "changes_five_rows_content_corrupted.rtf")
  file2 <- testthat::test_path(base, "base_corrupted.rtf")

  comparator <- create_comparator(file1, file2)
  result     <- comparator$vrf_summary(config = config)

  expect_equal(result, paste(
    "File content has changes in 5 place(s)."
  ))
})

base   <- "test_outputs/rtf"
config <- Config$new(FALSE)

################################################################################
# Generic file existence checks
################################################################################

test_that(paste(
  "Returns 'File(s) not available; unable to compare.'",
  "if one file does not exist"
), {
  file1 <- testthat::test_path(base, "base.rtf")
  file2 <- testthat::test_path(base, "nonexisting.rtf")

  comparator <- create_comparator(file1, file2)
  result     <- comparator$vrf_details(config = config)[[1]]

  expect_equal(result$type, "text")
  expect_equal(result$contents, "File(s) not available; unable to compare.")
})

################################################################################
# Simple tests that a S4 object is received for details comparison
################################################################################

test_that(paste(
  "Returns S4 comparison object for two files with same content"
), {
  file1 <- testthat::test_path(base, "base.rtf")
  file2 <- testthat::test_path(base, "copy.rtf")

  comparator <- create_comparator(file1, file2)
  result     <- comparator$vrf_details(config = config)[[1]]

  expect_equal(result$type, "text")
  expect_equal(typeof(result$contents), "S4")
})

test_that(paste(
  "Returns S4 comparison object for two files with differences in content"
), {
  file1 <- testthat::test_path(base, "base.rtf")
  file2 <- testthat::test_path(base, "changes_one_row_content.rtf")

  comparator <- create_comparator(file1, file2)
  result     <- comparator$vrf_details(config = config)[[1]]

  expect_equal(result$type, "text")
  expect_equal(typeof(result$contents), "S4")
})

test_that(paste(
  "Returns S4 comparison object for two files with differences in content",
  "and in embedded images"
), {
  file1 <- testthat::test_path(base, "base_with_image.rtf")
  file2 <- testthat::test_path(base, "changes_one_row_content_one_image.rtf")

  comparator <- create_comparator(file1, file2)
  result     <- comparator$vrf_details(config = config)[[1]]

  expect_equal(result$type, "text")
  expect_equal(typeof(result$contents), "S4")
})

test_that(paste(
  "Returns S4 comparison object for two files with differences in content",
  "(images disabled)"
), {
  file1 <- testthat::test_path(base, "base_with_image.rtf")
  file2 <- testthat::test_path(base, "changes_one_row_content_one_image.rtf")

  config_local <- Config$new(FALSE)
  config_local$set("rtf.images", "no")

  comparator <- create_comparator(file1, file2)
  result     <- comparator$vrf_details(config = config_local)[[1]]

  expect_equal(result$type, "text")
  expect_equal(typeof(result$contents), "S4")
})

################################################################################
# Test logging enabled
################################################################################

test_that(paste(
  "Prints expected logs to console when debugging is enabled"
), {
  file1 <- testthat::test_path(base, "base_with_image.rtf")
  file2 <- testthat::test_path(base, "changes_one_row_content_one_image.rtf")

  config_local <- Config$new(FALSE)
  config_local$set("generic.debug", "yes")

  output <- capture.output({
    comparator <- create_comparator(file1, file2)
    result     <- comparator$vrf_details(config = config_local)[[1]]
  })

  expected <- "'File::vrf_details, mode: summary' (execution time"
  expect_content(expected, output)

  expected <- "'File 1: test_outputs/rtf/base_with_image.rtf'"
  expect_content(expected, output)

  expected <- "'File 2: test_outputs/rtf/changes_one_row_content_one_image.rtf'"
  expect_content(expected, output)

  expected <- "'TxtWithImages::vrf_details_inner' (execution time"
  expect_content(expected, output)
})

base   <- "test_outputs/txt"
config <- Config$new(FALSE)

################################################################################
# Generic file existence checks
################################################################################

test_that(paste(
  "Returns 'File(s) not available; unable to compare.' ",
  "if both of the files do not exist"
), {
  file1 <- testthat::test_path(base, "nonexisting1.txt")
  file2 <- testthat::test_path(base, "nonexisting2.txt")

  comparator <- create_comparator(file1, file2)
  result     <- comparator$vrf_details(config = config)[[1]]

  expect_equal(result$type, "text")
  expect_equal(result$contents, "File(s) not available; unable to compare.")
})

test_that(paste(
  "Returns 'File(s) not available; unable to compare.' ",
  "if one file does not exist"
), {
  file1 <- testthat::test_path(base, "base.txt")
  file2 <- testthat::test_path(base, "nonexisting.txt")

  comparator <- create_comparator(file1, file2)
  result     <- comparator$vrf_details(config = config)[[1]]

  expect_equal(result$type, "text")
  expect_equal(result$contents, "File(s) not available; unable to compare.")
})

################################################################################
# Simple tests that a S4 object is received for details comparison
################################################################################

test_that(paste(
  "Returns S4 comparison object for two files with same content"
), {
  file1 <- testthat::test_path(base, "base.txt")
  file2 <- testthat::test_path(base, "copy.txt")

  config_local <- Config$new(FALSE)
  config_local$set("details.mode", "summary")

  comparator <- create_comparator(file1, file2)
  result     <- comparator$vrf_details(config = config_local)[[1]]

  expect_equal(result$type, "text")
  expect_equal(typeof(result$contents), "S4")

  html_output <- as.character(result$contents)
  html_output <- paste(html_output, collapse = "\n")
  expect_false(grepl("='insert'><span class='diffobj-trim'>\\[", html_output))
})

test_that(paste(
  "Returns S4 comparison object for two files with differences in content"
), {
  file1 <- testthat::test_path(base, "base.txt")
  file2 <- testthat::test_path(base, "changes_one_row.txt")

  config_local <- Config$new(FALSE)
  config_local$set("details.mode", "full")

  comparator <- create_comparator(file1, file2)
  result     <- comparator$vrf_details(config = config_local)[[1]]

  expect_equal(result$type, "text")
  expect_equal(typeof(result$contents), "S4")

  html_output <- as.character(result$contents)
  html_output <- paste(html_output, collapse = "\n")
  expect_true(grepl("='insert'><span class='diffobj-trim'>\\[", html_output))
})

test_that(paste(
  "Returns S4 comparison object for two files with differences in content",
  "spacing (generic.spaces option = 'no')"
), {
  file1 <- testthat::test_path(base, "base.txt")
  file2 <- testthat::test_path(base, "additional_spaces_and_tabs.txt")

  config_local <- Config$new(FALSE)
  config_local$set("generic.spaces", "no")

  comparator <- create_comparator(file1, file2)
  result     <- comparator$vrf_details(config = config_local)[[1]]

  expect_equal(result$type, "text")
  expect_equal(typeof(result$contents), "S4")

  html_output <- as.character(result$contents)
  html_output <- paste(html_output, collapse = "\n")
  expect_true(grepl("='insert'><span class='diffobj-trim'>\\[", html_output))
})

test_that(paste(
  "Returns S4 comparison object for two files with differences in content",
  "spacing (generic.spaces option = 'yes')"
), {
  file1 <- testthat::test_path(base, "base.txt")
  file2 <- testthat::test_path(base, "additional_spaces_and_tabs.txt")

  config_local <- Config$new(FALSE)
  config_local$set("generic.spaces", "yes")

  comparator <- create_comparator(file1, file2)
  result     <- comparator$vrf_details(config = config_local)[[1]]

  expect_equal(result$type, "text")
  expect_equal(typeof(result$contents), "S4")

  html_output <- as.character(result$contents)
  html_output <- paste(html_output, collapse = "\n")
  expect_false(grepl("='insert'><span class='diffobj-trim'>\\[", html_output))
})

test_that(paste(
  "Invokes the custom diffObj finalizer with full mode and omit"
), {
  file1 <- testthat::test_path(base, "base.txt")
  file2 <- testthat::test_path(base, "addition_one_row.txt")

  config <- Config$new(FALSE)
  config$set("generic.level", "row")
  config$set("details.mode", "full")

  comparator <- create_comparator(file1, file2)
  result     <- comparator$vrf_details(config = config, omit = "Line 4")[[1]]

  expect_equal(result$type, "text")
  expect_equal(typeof(result$contents), "S4")

  # get the result contents to invoke the finalizer callback used for printing
  html_output <- as.character(result$contents)
  html_output <- paste(html_output, collapse = "\n")
  expect_true(grepl("class='ignore'", html_output))
})

#' FileComparatorFactory.R
#'
#' Factory method for creating comparator instance based on the given two files.
#'
#' @param file1 first file to compare
#' @param file2 second file to compare
#'
#' @examples
#'
#' # instantiating the compared files
#' file1 <- "file1.rtf"
#' file2 <- "file2.rtf"
#'
#' # instantiating the configuration
#' config <- Config$new()
#'
#' # instantiating a new comparator instance for every comparison:
#' comparator <- verifyr2::create_comparator(file1, file2)
#'
#' # calling the summary and details comparison methods
#' comparator$vrf_summary(config = config)
#' comparator$vrf_details(config = config)
#'
#' @include BinaryFileComparator.R
#' @include TxtFileComparator.R
#'
#' @export
#'
create_comparator <- function(file1, file2) {
  if (!file.exists(file1) || !file.exists(file2)) {
    return(BinaryFileComparator$new(file1 = file1, file2 = file2))
  }

  file_extension  <- tools::toTitleCase(tools::file_ext(file1))

  if (file_extension %in% list("Jpg", "Jpeg", "Png")) {
    file_extension <- "Img"
  }

  # construct the comparator name
  comparator_name <- paste0(file_extension, "FileComparator")

  if (exists(comparator_name, envir = .GlobalEnv)) {
    class_def <- get(comparator_name, envir = .GlobalEnv)

    # return the specific class instance if the class exists
    if (inherits(class_def, "R6ClassGenerator")) {
      return(do.call(class_def$new, list(file1 = file1, file2 = file2)))
    }
  }

  # generic comparator class used based on the file contents (text/binary).
  mime_type <- mime::guess_type(file1)

  text_like <- c(
    "application/json",
    "application/xml",
    "application/javascript",
    "application/x-yaml",
    "application/sql",
    "application/x-httpd-php",
    "application/x-sh",
    "application/csv",
    "application/x-tex",
    "application/x-markdown"
  )

  # additional types that the mime::guess_type mapping doesn't handle correctly
  additional_text_types <- c(
    "Lst",
    "Sas"
  )

  txt_type <- startsWith(mime_type, "text/")
  fix_type <- mime_type %in% text_like
  add_type <- file_extension %in% additional_text_types

  if (txt_type || fix_type || add_type) {
    TxtFileComparator$new(file1 = file1, file2 = file2)
  } else {
    BinaryFileComparator$new(file1 = file1, file2 = file2)
  }
}

is_r6_class <- function(class_name) {
  obj <- try(get(class_name, envir = .GlobalEnv), silent = TRUE)
  inherits(obj, "R6ClassGenerator")
}

base   <- "test_outputs/img"
config <- Config$new(FALSE)

################################################################################
# Generic file existence checks
################################################################################

test_that(paste(
  "Returns 'File(s) not available; unable to compare.'",
  "if both files do not exist"
), {
  file1 <- testthat::test_path(base, "nonexisting1.jpg")
  file2 <- testthat::test_path(base, "nonexisting2.jpg")

  comparator <- create_comparator(file1, file2)
  result     <- comparator$vrf_summary(config = config)

  expect_equal(result, "File(s) not available; unable to compare.")
})

test_that(paste(
  "Returns 'File(s) not available; unable to compare.'",
  "if one file does not exist"
), {
  file1 <- testthat::test_path(base, "base.jpeg")
  file2 <- testthat::test_path(base, "nonexisting.jpeg")

  comparator <- create_comparator(file1, file2)
  result     <- comparator$vrf_summary(config = config)

  expect_equal(result, "File(s) not available; unable to compare.")
})

################################################################################
# comparison
################################################################################

test_that(paste(
  "Returns 'No differences.' for identical files"
), {
  file1 <- testthat::test_path(base, "base.png")
  file2 <- testthat::test_path(base, "copy.png")

  comparator <- create_comparator(file1, file2)
  result     <- comparator$vrf_summary(config = config)

  if (requireNamespace("magick", quietly = TRUE)) {
    expect_equal(result, "No differences.")
  } else {
    expect_equal(result, paste(
      "No differences.",
      "Image details comparison disabled."
    ))
  }
})

test_that(paste(
  "Returns 'Different file sizes for compared files.'"
), {
  file1 <- testthat::test_path(base, "base.jpg")
  file2 <- testthat::test_path(base, "modified1.jpg")

  comparator <- create_comparator(file1, file2)
  result     <- comparator$vrf_summary(config = config)

  if (requireNamespace("magick", quietly = TRUE)) {
    expect_equal(result, "Different file sizes for compared files.")
  } else {
    expect_equal(result, paste(
      "Different file sizes for compared files.",
      "Image details comparison disabled."
    ))
  }
})

################################################################################
# comparison - with magick library not available
################################################################################

test_that(paste(
  "Returns 'Different file sizes for compared files.",
  "Image details comparison disabled.'"
), {
  file1 <- testthat::test_path(base, "base.jpg")
  file2 <- testthat::test_path(base, "modified1.jpg")

  # mock the magick available method to return false to replicate situation
  # that magick library is not installed.
  local_mocked_bindings(
    check_magick_available = function() FALSE
  )

  config_local <- Config$new(FALSE)
  comparator   <- create_comparator(file1, file2)
  result       <- comparator$vrf_summary(config = config_local)

  expect_equal(result, paste(
    "Different file sizes for compared files.",
    "Image details comparison disabled."
  ))
})

base   <- "test_outputs/json"
config <- Config$new(FALSE)

################################################################################
# Text file comparison - WITHOUT omit
################################################################################

test_that(paste(
  "Returns 'No differences' for identical files."
), {
  file1 <- testthat::test_path(base, "base.json")
  file2 <- testthat::test_path(base, "copy.json")

  comparator <- create_comparator(file1, file2)
  result     <- comparator$vrf_summary(config = config)

  expect_equal(result, "No differences.")
})

test_that(paste(
  "Returns 'Different number of lines in compared content.'",
  "for files with additional line in the content"
), {
  file1 <- testthat::test_path(base, "base.json")
  file2 <- testthat::test_path(base, "modified.json")

  comparator <- create_comparator(file1, file2)
  result     <- comparator$vrf_summary(config = config)

  expect_equal(result, "Different number of lines in compared content.")
})

################################################################################
# Text file comparison - WITH omit
################################################################################

test_that(paste(
  "Returns 'No differences.'",
  "for comparison with 'omit' parameter that is not present in either file"
), {
  file1 <- testthat::test_path(base, "base.json")
  file2 <- testthat::test_path(base, "copy.json")

  comparator <- create_comparator(file1, file2)
  result     <- comparator$vrf_summary(config = config, omit = "Nothing")

  expect_equal(result, "No differences.")
})

test_that(paste(
  "Returns 'Different number of lines in compared content.'",
  "when called with 'omit' catching the changed rows in content"
), {
  file1 <- testthat::test_path(base, "base.json")
  file2 <- testthat::test_path(base, "modified.json")

  comparator <- create_comparator(file1, file2)
  result     <- comparator$vrf_summary(config = config, omit = "height")

  expect_equal(result, "Different number of lines in compared content.")
})

test_that(paste(
  "Returns 'File content has changes in 5 place(s).'",
  "when called with 'omit' catching the additional rows in content"
), {
  file1 <- testthat::test_path(base, "base.json")
  file2 <- testthat::test_path(base, "modified.json")

  comparator <- create_comparator(file1, file2)
  result     <- comparator$vrf_summary(config = config, omit = "sex")

  expect_equal(result, "File content has changes in 5 place(s).")
})

base <- "test_outputs/img"
config <- Config$new(FALSE)

################################################################################
# Generic file existence checks
################################################################################

test_that(paste(
  "Returns 'File(s) not available; unable to compare.'",
  "if both of the files do not exist"
), {
  file1 <- testthat::test_path(base, "nonexisting1.jpg")
  file2 <- testthat::test_path(base, "nonexisting2.jpg")

  comparator <- create_comparator(file1, file2)
  result     <- comparator$vrf_details(config = config)[[1]]

  expect_equal(result$type, "text")
  expect_equal(result$contents, "File(s) not available; unable to compare.")
})

test_that(paste(
  "Returns 'File(s) not available; unable to compare.'",
  "if one file does not exist"
), {
  file1 <- testthat::test_path(base, "base.png")
  file2 <- testthat::test_path(base, "nonexisting.png")

  comparator <- create_comparator(file1, file2)
  result     <- comparator$vrf_details(config = config)[[1]]

  expect_equal(result$type, "text")
  expect_equal(result$contents, "File(s) not available; unable to compare.")
})

################################################################################
# Simple tests that a S4 object is received for details comparison
################################################################################

test_that(paste(
  "Returns S4 comparison object for two files with same file size"
), {
  file1 <- testthat::test_path(base, "base.jpeg")
  file2 <- testthat::test_path(base, "base.jpeg")

  comparator <- create_comparator(file1, file2)
  result     <- comparator$vrf_details(config = config)[[1]]
  contents   <- result$contents

  if (requireNamespace("magick", quietly = TRUE)) {
    expect_equal(result$type, "image")
    expect_equal(typeof(contents), "list")
    expect_equal(names(contents), c("image1", "image2", "image3"))
    expect_equal(contents$image1, contents$image2)
    expect_false(isTRUE(all.equal(contents$image1, contents$image3)))
    expect_false(isTRUE(all.equal(contents$image2, contents$image3)))
  } else {
    expect_equal(result$type, "text")
    expect_equal(result$contents, "Image details comparison disabled.")
  }
})

test_that(paste(
  "Returns S4 comparison object for two files with differences in file size"
), {
  file1 <- testthat::test_path(base, "base.png")
  file2 <- testthat::test_path(base, "modified1.png")

  comparator <- create_comparator(file1, file2)
  result     <- comparator$vrf_details(config = config)[[1]]
  contents   <- result$contents

  if (requireNamespace("magick", quietly = TRUE)) {
    expect_equal(result$type, "image")
    expect_equal(typeof(contents), "list")
    expect_equal(names(contents), c("image1", "image2", "image3"))
    expect_false(isTRUE(all.equal(contents$image1, contents$image2)))
    expect_false(isTRUE(all.equal(contents$image1, contents$image3)))
    expect_false(isTRUE(all.equal(contents$image2, contents$image3)))
  } else {
    expect_equal(result$type, "text")
    expect_equal(result$contents, "Image details comparison disabled.")
  }
})

base   <- "test_outputs/bin"
config <- Config$new(FALSE)

################################################################################
# Generic file existence checks
################################################################################

test_that(paste(
  "'File(s) not available; unable to compare.' ",
  "returned if both of the files do not exist"
), {
  file1 <- testthat::test_path(base, "nonexisting1.bin")
  file2 <- testthat::test_path(base, "nonexisting2.bin")

  comparator <- create_comparator(file1, file2)
  result     <- comparator$vrf_details(config = config)[[1]]

  expect_equal(result$type, "text")
  expect_equal(result$contents, "File(s) not available; unable to compare.")
})

test_that(paste(
  "'File(s) not available; unable to compare.' ",
  "returned if one file does not exist"
), {
  file1 <- testthat::test_path(base, "base.bin")
  file2 <- testthat::test_path(base, "nonexisting.bin")

  comparator <- create_comparator(file1, file2)
  result     <- comparator$vrf_details(config = config)[[1]]

  expect_equal(result$type, "text")
  expect_equal(result$contents, "File(s) not available; unable to compare.")
})

################################################################################
# Binary files don't support details comparison
################################################################################

test_that(paste(
  "'Binary file without applicable comparator; unable to compare details.' ",
  "returned for binary files details comparison"
), {
  file1 <- testthat::test_path(base, "base.bin")
  file2 <- testthat::test_path(base, "modified.bin")

  comparator <- create_comparator(file1, file2)
  result     <- comparator$vrf_details(config = config)[[1]]

  expect_equal(result$type, "text")
  expect_equal(result$contents, paste(
    "No details comparison available."
  ))
})

base   <- "test_outputs/pdf"
config <- Config$new(FALSE)

################################################################################
# Generic file existence checks
################################################################################

test_that(paste(
  "Returns 'File(s) not available; unable to compare.' ",
  "if both files do not exist"
), {
  file1 <- testthat::test_path(base, "nonexisting1.pdf")
  file2 <- testthat::test_path(base, "nonexisting2.pdf")

  comparator <- create_comparator(file1, file2)
  result     <- comparator$vrf_summary(config = config)

  expect_equal(result, "File(s) not available; unable to compare.")
})

test_that(paste(
  "Returns 'File(s) not available; unable to compare.' ",
  "if one file does not exist"
), {
  file1 <- testthat::test_path(base, "base.pdf")
  file2 <- testthat::test_path(base, "nonexisting.pdf")

  comparator <- create_comparator(file1, file2)
  result     <- comparator$vrf_summary(config = config)

  expect_equal(result, "File(s) not available; unable to compare.")
})

################################################################################
# Text file comparison - WITHOUT omit
################################################################################

test_that(paste(
  "Returns 'No differences.' for identical files",
  "(one page)"
), {
  file1 <- testthat::test_path(base, "base.pdf")
  file2 <- testthat::test_path(base, "copy.pdf")

  comparator <- create_comparator(file1, file2)
  result     <- comparator$vrf_summary(config = config)

  if (requireNamespace("pdftools", quietly = TRUE)) {
    expect_equal(result, "No differences.")
  } else {
    expect_equal(result, paste(
      "No differences.",
      "Pdf details comparison disabled."
    ))
  }
})

test_that(paste(
  "Returns 'No differences.' for identical files",
  "(two pages)"
), {
  file1 <- testthat::test_path(base, "two_pages.pdf")
  file2 <- testthat::test_path(base, "two_pages_copy.pdf")

  comparator <- create_comparator(file1, file2)
  result     <- comparator$vrf_summary(config = config)

  if (requireNamespace("pdftools", quietly = TRUE)) {
    expect_equal(result, "No differences.")
  } else {
    expect_equal(result, paste(
      "No differences.",
      "Pdf details comparison disabled."
    ))
  }
})

test_that(paste(
  "Returns 'Different number of lines in compared content.' ",
  "for files with additional line in the content",
  "(one page)"
), {
  file1 <- testthat::test_path(base, "base.pdf")
  file2 <- testthat::test_path(base, "addition_one_row.pdf")

  comparator <- create_comparator(file1, file2)
  result     <- comparator$vrf_summary(config = config)

  if (requireNamespace("pdftools", quietly = TRUE)) {
    expect_equal(result, "Different number of lines in compared content.")
  } else {
    expect_equal(result, paste(
      "Different number of lines in compared content.",
      "Pdf details comparison disabled."
    ))
  }
})

test_that(paste(
  "Returns 'Different number of lines in compared content.' ",
  "for files with additional line in the content",
  "(two pages)"
), {
  file1 <- testthat::test_path(base, "two_pages.pdf")
  file2 <- testthat::test_path(base, "two_pages_addition_one_row.pdf")

  comparator <- create_comparator(file1, file2)
  result     <- comparator$vrf_summary(config = config)

  if (requireNamespace("pdftools", quietly = TRUE)) {
    expect_equal(result, "Different number of lines in compared content.")
  } else {
    expect_equal(result, paste(
      "Different number of lines in compared content.",
      "Pdf details comparison disabled."
    ))
  }
})

test_that(paste(
  "Returns 'File content has changes in 1 place(s).' ",
  "for files with a single different row in content",
  "(one page)"
), {
  file1 <- testthat::test_path(base, "changes_one_row.pdf")
  file2 <- testthat::test_path(base, "base.pdf")

  comparator <- create_comparator(file1, file2)
  result     <- comparator$vrf_summary(config = config)

  if (requireNamespace("pdftools", quietly = TRUE)) {
    expect_equal(result, "File content has changes in 1 place(s).")
  } else {
    expect_equal(result, paste(
      "Different number of lines in compared content.",
      "Pdf details comparison disabled."
    ))
  }
})

test_that(paste(
  "Returns 'File content has changes in 1 place(s).' ",
  "for files with a single different row in content",
  "(two pages)"
), {
  file1 <- testthat::test_path(base, "two_pages_changes_one_row.pdf")
  file2 <- testthat::test_path(base, "two_pages.pdf")

  comparator <- create_comparator(file1, file2)
  result     <- comparator$vrf_summary(config = config)

  if (requireNamespace("pdftools", quietly = TRUE)) {
    expect_equal(result, "File content has changes in 1 place(s).")
  } else {
    expect_equal(result, paste(
      "Different number of lines in compared content.",
      "Pdf details comparison disabled."
    ))
  }
})

test_that(paste(
  "Returns 'File content has changes in 2 place(s).' ",
  "for files with two different rows in content",
  "(two pages)"
), {
  file1 <- testthat::test_path(base, "two_pages_changes_two_rows.pdf")
  file2 <- testthat::test_path(base, "two_pages.pdf")

  comparator <- create_comparator(file1, file2)
  result     <- comparator$vrf_summary(config = config)

  if (requireNamespace("pdftools", quietly = TRUE)) {
    expect_equal(result, "File content has changes in 2 place(s).")
  } else {
    expect_equal(result, paste(
      "Different number of lines in compared content.",
      "Pdf details comparison disabled."
    ))
  }
})

################################################################################
# Text file comparison - WITH omit
# Note that with pdftools disabled, omit doesn't take effect
################################################################################

test_that(paste(
  "Returns 'No differences.' ",
  "for comparison with 'omit' parameter that is not present in either file",
  "(one page)"
), {
  file1 <- testthat::test_path(base, "base.pdf")
  file2 <- testthat::test_path(base, "copy.pdf")

  omit       <- "Nothing."
  comparator <- create_comparator(file1, file2)
  result     <- comparator$vrf_summary(config = config, omit = omit)

  if (requireNamespace("pdftools", quietly = TRUE)) {
    expect_equal(result, "No differences.")
  } else {
    expect_equal(result, paste(
      "No differences.",
      "Pdf details comparison disabled."
    ))
  }
})

test_that(paste(
  "Returns 'No differences.' ",
  "for comparison with 'omit' parameter that is not present in either file",
  "(two pages)"
), {
  file1 <- testthat::test_path(base, "two_pages.pdf")
  file2 <- testthat::test_path(base, "two_pages_copy.pdf")

  omit       <- "Nothing."
  comparator <- create_comparator(file1, file2)
  result     <- comparator$vrf_summary(config = config, omit = omit)

  if (requireNamespace("pdftools", quietly = TRUE)) {
    expect_equal(result, "No differences.")
  } else {
    expect_equal(result, paste(
      "No differences.",
      "Pdf details comparison disabled."
    ))
  }
})

test_that(paste(
  "Returns 'No differences.' ",
  "when called with 'omit' catching the one row changed row in content",
  "(one page)"
), {
  file1 <- testthat::test_path(base, "base.pdf")
  file2 <- testthat::test_path(base, "changes_one_row.pdf")

  omit       <- "simple example pdf"
  comparator <- create_comparator(file1, file2)
  result     <- comparator$vrf_summary(config = config, omit = omit)

  if (requireNamespace("pdftools", quietly = TRUE)) {
    expect_equal(result, "No differences.")
  } else {
    expect_equal(result, paste(
      "Different number of lines in compared content.",
      "Pdf details comparison disabled."
    ))
  }
})

test_that(paste(
  "Return 'No differences.' ",
  "when called with 'omit' catching the one row changed row in content",
  "(two pages)"
), {
  file1 <- testthat::test_path(base, "two_pages.pdf")
  file2 <- testthat::test_path(base, "two_pages_changes_one_row.pdf")

  omit       <- "two paged example pdf"
  comparator <- create_comparator(file1, file2)
  result     <- comparator$vrf_summary(config = config, omit = omit)

  if (requireNamespace("pdftools", quietly = TRUE)) {
    expect_equal(result, "No differences.")
  } else {
    expect_equal(result, paste(
      "Different number of lines in compared content.",
      "Pdf details comparison disabled."
    ))
  }
})

test_that(paste(
  "Returns 'No differences.' ",
  "when called with 'omit' catching the one additional row in content",
  "(one page)"
), {
  file1 <- testthat::test_path(base, "base.pdf")
  file2 <- testthat::test_path(base, "addition_one_row.pdf")

  omit       <- "one additional row"
  comparator <- create_comparator(file1, file2)
  result     <- comparator$vrf_summary(config = config, omit = omit)

  if (requireNamespace("pdftools", quietly = TRUE)) {
    expect_equal(result, "No differences.")
  } else {
    expect_equal(result, paste(
      "Different number of lines in compared content.",
      "Pdf details comparison disabled."
    ))
  }
})

test_that(paste(
  "Returns 'No differences.' ",
  "when called with 'omit' catching the one additional row in content",
  "(two pages)"
), {
  file1 <- testthat::test_path(base, "two_pages.pdf")
  file2 <- testthat::test_path(base, "two_pages_addition_one_row.pdf")

  omit       <- "additional row"
  comparator <- create_comparator(file1, file2)
  result     <- comparator$vrf_summary(config = config, omit = omit)

  if (requireNamespace("pdftools", quietly = TRUE)) {
    expect_equal(result, "No differences.")
  } else {
    expect_equal(result, paste(
      "Different number of lines in compared content.",
      "Pdf details comparison disabled."
    ))
  }
})

test_that(paste(
  "Returns 'No differences.' ",
  "when called with 'omit' catching the two additional rows in content",
  "(one page)"
), {
  file1 <- testthat::test_path(base, "base.pdf")
  file2 <- testthat::test_path(base, "addition_two_rows.pdf")

  omit       <- "additional row"
  comparator <- create_comparator(file1, file2)
  result     <- comparator$vrf_summary(config = config, omit = omit)

  if (requireNamespace("pdftools", quietly = TRUE)) {
    expect_equal(result, "No differences.")
  } else {
    expect_equal(result, paste(
      "Different number of lines in compared content.",
      "Pdf details comparison disabled."
    ))
  }
})

test_that(paste(
  "Returns 'No differences.' ",
  "when called with 'omit' catching the two additional rows in content",
  "(two pages)"
), {
  file1 <- testthat::test_path(base, "two_pages.pdf")
  file2 <- testthat::test_path(base, "two_pages_addition_two_rows.pdf")

  omit       <- "additional row"
  comparator <- create_comparator(file1, file2)
  result     <- comparator$vrf_summary(config = config, omit = omit)

  if (requireNamespace("pdftools", quietly = TRUE)) {
    expect_equal(result, "No differences.")
  } else {
    expect_equal(result, paste(
      "Different number of lines in compared content.",
      "Pdf details comparison disabled."
    ))
  }
})

test_that(paste(
  "Returns 'Different number of lines in compared content.' ",
  "when called with 'omit' catching one of the two additional rows",
  "(one page)"
), {
  file1 <- testthat::test_path(base, "base.pdf")
  file2 <- testthat::test_path(base, "addition_two_rows.pdf")

  omit       <- "one additional row"
  comparator <- create_comparator(file1, file2)
  result     <- comparator$vrf_summary(config = config, omit = omit)

  if (requireNamespace("pdftools", quietly = TRUE)) {
    expect_equal(result, "Different number of lines in compared content.")
  } else {
    expect_equal(result, paste(
      "Different number of lines in compared content.",
      "Pdf details comparison disabled."
    ))
  }
})

test_that(paste(
  "Returns 'Different number of lines in compared content.' ",
  "when called with 'omit' catching one of the two additional rows",
  "(two pages)"
), {
  file1 <- testthat::test_path(base, "two_pages.pdf")
  file2 <- testthat::test_path(base, "two_pages_addition_two_rows.pdf")

  omit       <- "one additional row"
  comparator <- create_comparator(file1, file2)
  result     <- comparator$vrf_summary(config = config, omit = omit)

  if (requireNamespace("pdftools", quietly = TRUE)) {
    expect_equal(result, "Different number of lines in compared content.")
  } else {
    expect_equal(result, paste(
      "Different number of lines in compared content.",
      "Pdf details comparison disabled."
    ))
  }
})

################################################################################
# Text file comparison - with PDF package missing
################################################################################

test_that(paste(
  "Returns 'Different number of lines in compared content.",
  "Pdf details comparison disabled.' when pdftools library ",
  "is not available"
), {
  file1 <- testthat::test_path(base, "two_pages.pdf")
  file2 <- testthat::test_path(base, "two_pages_addition_two_rows.pdf")

  # mock the pdftools available method to return false to replicate situation
  # that pdftools library is not installed.
  local_mocked_bindings(
    check_pdftools_available = function() FALSE
  )

  config_local <- Config$new(FALSE)

  omit       <- "Nothing"
  comparator <- create_comparator(file1, file2)
  result     <- comparator$vrf_summary(config = config_local, omit = omit)

  expect_equal(result, paste(
    "Different number of lines in compared content.",
    "Pdf details comparison disabled."
  ))
})

base   <- "test_outputs/pdf"
config <- Config$new(FALSE)

################################################################################
# Generic file existence checks
################################################################################

test_that(paste(
  "Returns 'File(s) not available; unable to compare.' ",
  "if both of the files do not exist"
), {
  file1 <- testthat::test_path(base, "nonexisting1.pdf")
  file2 <- testthat::test_path(base, "nonexisting2.pdf")

  comparator <- create_comparator(file1, file2)
  result     <- comparator$vrf_details(config = config)[[1]]

  expect_equal(result$type, "text")
  expect_equal(result$contents, "File(s) not available; unable to compare.")
})

test_that(paste(
  "Returns 'File(s) not available; unable to compare.' ",
  "if one file does not exist"
), {
  file1 <- testthat::test_path(base, "base.pdf")
  file2 <- testthat::test_path(base, "nonexisting.pdf")

  comparator <- create_comparator(file1, file2)
  result     <- comparator$vrf_details(config = config)[[1]]

  expect_equal(result$type, "text")
  expect_equal(result$contents, "File(s) not available; unable to compare.")
})

################################################################################
# Simple tests that a S4 object is received for details comparison
################################################################################

test_that(paste(
  "Returns S4 comparison object for two files with same content."
), {
  file1 <- testthat::test_path(base, "base.pdf")
  file2 <- testthat::test_path(base, "copy.pdf")

  comparator <- create_comparator(file1, file2)
  result     <- comparator$vrf_details(config = config)[[1]]

  expect_equal(result$type, "text")

  if (requireNamespace("pdftools", quietly = TRUE)) {
    expect_equal(typeof(result$contents), "S4")
  } else {
    expect_equal(result$contents, "Pdf details comparison disabled.")
  }
})

test_that(paste(
  "Returns S4 comparison object for two files with differences in content"
), {
  file1 <- testthat::test_path(base, "two_pages.pdf")
  file2 <- testthat::test_path(base, "two_pages_changes_one_row.pdf")

  comparator <- create_comparator(file1, file2)
  result     <- comparator$vrf_details(config = config)[[1]]

  expect_equal(result$type, "text")

  if (requireNamespace("pdftools", quietly = TRUE)) {
    expect_equal(typeof(result$contents), "S4")
  } else {
    expect_equal(result$contents, "Pdf details comparison disabled.")
  }
})

config <- Config$new(FALSE)

################################################################################
# Generic file existence checks
################################################################################

test_that(paste(
  "Returns 'File(s) not available; unable to compare.' ",
  "if both files do not exist"
), {
  file1 <- testthat::test_path("test_outputs/bin/nonexisting1.bin")
  file2 <- testthat::test_path("test_outputs/bin/nonexisting2.bin")

  comparator <- create_comparator(file1, file2)
  result     <- comparator$vrf_summary(config = config)

  expect_equal(result, "File(s) not available; unable to compare.")
})

test_that(paste(
  "Returns 'File(s) not available; unable to compare.' ",
  "if one file does not exist"
), {
  file1 <- testthat::test_path("test_outputs/bin/base.bin")
  file2 <- testthat::test_path("test_outputs/bin/nonexisting.bin")

  comparator <- create_comparator(file1, file2)
  result     <- comparator$vrf_summary(config = config)

  expect_equal(result, "File(s) not available; unable to compare.")
})

################################################################################
# Binary file comparison
################################################################################

test_that(paste(
  "Returns 'No differences' for identical files."
), {
  file1 <- testthat::test_path("test_outputs/bin/base.bin")
  file2 <- testthat::test_path("test_outputs/bin/copy.bin")

  comparator <- create_comparator(file1, file2)
  result     <- comparator$vrf_summary(config = config)

  expect_equal(result, "No differences. No details comparison available.")
})

test_that(paste(
  "Returns 'Different file sizes for compared files.' ",
  "for files different sizes"
), {
  file1 <- testthat::test_path("test_outputs/bin/base.bin")
  file2 <- testthat::test_path("test_outputs/bin/modified.bin")

  comparator <- create_comparator(file1, file2)
  result     <- comparator$vrf_summary(config = config)

  expect_equal(result, paste(
    "Different file sizes for compared files.",
    "No details comparison available."
  ))
})

test_that(paste(
  "Returns 'Different content in compared files' ",
  "for files with same sizes but different contents"
), {
  file1 <- testthat::test_path("test_outputs/bin/base.bin")
  file2 <- testthat::test_path("test_outputs/bin/modified_same_size.bin")

  comparator <- create_comparator(file1, file2)
  result     <- comparator$vrf_summary(config = config)

  expect_equal(result, paste(
    "Different content in compared files.",
    "No details comparison available."
  ))
})

#' RtfFileComparator.R
#'
#' Specialiced comparator for RTF file comparison.
#' This comparator contains the custom handling for handling only RTF content
#' part for the comparison.
#'
#' @import striprtf
#'
#' @include TxtWithImagesComparator.R
#'
#' @examples
#'
#' # The normal way for creating a comparator would be to call the generic
#' # factory method verifyr2::create_comparator that will automatically create
#' # the correct comparator instance based on the file types.
#'
#' file1 <- 'my_file1.rtf'
#' file2 <- 'my_file2.rtf'
#' comparator <- verifyr2::create_comparator(file1, file2)
#'
#' # If needed, an explicit comparator can be created as well.
#'
#' file1 <- 'my_file1.rtf'
#' file2 <- 'my_file2.rft'
#' comparator <- RtfFileComparator$new(file1, file2)
#'
#' @export
#'
RtfFileComparator <- R6::R6Class(
  "RtfFileComparator",
  inherit = TxtWithImagesFileComparator,
  public = list(

    #' @description
    #' Method for getting the single file contents for the comparison. The
    #' method returns the file contents in two separate vectors inside a list.
    #' The first vector is the file contents and the second one is the file
    #' contents with the rows matching the omit string excluded. This method
    #' can be overwritten by more specialized comparator classes. This method
    #' is intended to be called only by the comparator classes in the processing
    #' and shouldn't be called directly by the user.
    #'
    #' For RtfComparator, only the RTF file content part is returned for
    #' comparison.
    #'
    #' @param file   file for which to get the contents
    #' @param config configuration values
    #' @param omit   string pattern to omit from the comparison
    #'
    vrf_contents = function(file, config, omit) {
      self$vrf_open_debug("Rtf::vrf_contents", config)

      # Get the RTF text content
      contents <- self$safe_read_rtf(file = file)
      result   <- self$vrf_contents_inner(contents, config, omit)

      self$vrf_close_debug()
      result
    },

    #' @description
    #' "Abstract" method for getting the raw image hex vector array from the
    #' given source file.
    #'
    #' @param file   file for which to get the embedded image details
    #' @param config configuration values
    #'
    vrf_images = function(file, config) {
      self$vrf_open_debug("Rtf::vrf_images", config)

      # return empty list if embedded image processing is disabled
      if ("no" == super$vrf_option_value(config, "rtf.images")) {
        self$vrf_close_debug()
        return(list())
      }

      # return empty list if image processing is disabled
      if ("no" == super$vrf_option_value(config, "generic.images")) {
        self$vrf_close_debug()
        return(list())
      }

      result <- list()

      #  Read the RTF file embedded images
      rtf_content <- readLines(file, warn = FALSE)
      rtf_content <- paste(rtf_content, collapse = "\n")

      # regexp for finding the png pictures from the raw RTF content.
      pattern <- "\\\\pict\\\\pngblip[^{]+([A-Za-z0-9+/=]+)"
      matches <- stringr::str_extract_all(rtf_content, pattern)[[1]]

      if (length(matches) > 0) {
        for (index in seq_along(matches)) {
          split_parts <- strsplit(matches[index], " ")[[1]]
          base64_data_with_braces <- split_parts[2]
          base64_data <- strsplit(base64_data_with_braces, "}")[[1]][1]

          if (!is.na(base64_data) && nchar(base64_data) > 0) {
            raw_data <- self$hex2raw(base64_data)
            result <- c(result, list(raw_data))
          }
        }
      }

      self$vrf_close_debug()
      result
    },

    #' @description
    #' RTF content reading method with some fallback options to fix simple
    #' file content issues.
    #'
    #' @param file file for which to get the embedded image details
    #'
    safe_read_rtf = function(file) {
      tryCatch({
        striprtf::read_rtf(file)
      }, error = function(e) {
        txt <- readLines(file, warn = FALSE)

        if (!any(grepl("\\\\ansicpg0", txt, fixed = FALSE))) {
          stop(e)
        }

        txt_fixed  <- sub("\\\\ansicpg0", "\\\\ansicpg1252", txt)
        txt_single <- paste(txt_fixed, collapse = "\n")

        striprtf::strip_rtf(txt_single)
      })
    }
  )
)

#' List files that exist in two folders
#'
#' \code{list_folder_files} List files that exist in two folders with a specific
#' name pattern
#'
#' @param folder1 character, giving the the full file path and name of the
#'                folder where first files are stored (required)
#' @param folder2 character, giving the the full file path and name of the
#'                folder where second new files are stored (required)
#' @param pattern character, limit the files to be listed to contain a
#'                specific pattern (optional)
#'
#' @return Returns a tibble, \code{selected_files} with 2 columns \code{file1},
#'                 \code{file2}
#'
#' @examples
#'
#' folder1 <- paste0(fs::path_package("/extdata/base_files/",
#'                                    package = "verifyr2"))
#'
#' folder2 <- paste0(fs::path_package("/extdata/compare_files/",
#'                                    package = "verifyr2"))
#'
#' verifyr2::list_folder_files(folder1, folder2, "base")
#'
#' @export

list_folder_files <- function(folder1, folder2, pattern = NULL) {

  ## do the comparison only if both of the folders exist
  if (file.exists(folder1) && file.exists(folder2)) {

    folder1_info <- folder_info(folder1, "file1", pattern)
    folder2_info <- folder_info(folder2, "file2", pattern)

    selected_files <- dplyr::full_join(folder1_info,
                                       folder2_info,
                                       by = "file") |>
      dplyr::arrange(file) |>
      dplyr::select("file1", "file2")

    return(selected_files)
  }

  NULL
}

folder_info <- function(folder, column_name, pattern) {
  files  <- list.files(path = folder, pattern = pattern)
  paths  <- list.files(path = folder, pattern = pattern, full.names = TRUE)
  data   <- tibble::tibble(file = files)
  data[[column_name]] <- paths

  data
}

base1 <- "test_outputs/bin"
base2 <- "test_outputs/rtf"

################################################################################
# Generic file existence checks
################################################################################

test_that("returns null if both of the folders do not exist", {
  folder1 <- testthat::test_path("test_outputs/nonexisting_folder1")
  folder2 <- testthat::test_path("test_outputs/nonexisting_folder2")

  result <- list_folder_files(folder1, folder2)
  expect_null(result)
})

test_that("returns null if one of the folders does not exist", {
  folder1 <- testthat::test_path(base1)
  folder2 <- testthat::test_path("test_outputs/nonexisting_folder2")

  result <- list_folder_files(folder1, folder2)
  expect_null(result)
})

################################################################################
# Successfull cases
################################################################################

test_that("Returns tibble when both folders exist", {
  folder1 <- testthat::test_path(base1)
  folder2 <- testthat::test_path(base2)

  result <- list_folder_files(folder1, folder2)

  expect_s3_class(result, "tbl_df")
  expect_named(result, c("file1", "file2"))
})

#' FileComparator.R
#'
#' Comparator 'abstract' class containing the generic comparison methods and
#' handling for high level checks (like file existence). This class should never
#' be instantiated - doing that and calling the comparison methods will lead to
#' error.
#'
#' @importFrom R6 R6Class
#'
#' @field file1               file1
#' @field file2               file2
#' @field file1_contents_list file1 contents
#' @field file2_contents_list file2 contents
#' @field summary_comparison  summary comparison result
#' @field details_comparison  details comparison result
#' @field debugger            debugger instance
#'
#' @export
#'

# Disable cyclomatic complexity lint for the R6 class definition as lintr
# considers the whole class definition as a single function.
#
# nolint start: cyclocomp_linter
FileComparator <- R6::R6Class(
  "FileComparator",
  public <- list(
    file1 = NULL,
    file2 = NULL,
    file1_contents_list = NULL,
    file2_contents_list = NULL,
    summary_comparison  = NULL,
    details_comparison  = NULL,
    debugger = NULL,

    #' @description
    #' Initialize a FileComparator instance
    #'
    #' @param file1 First file to compare.
    #' @param file2 Second file to compare.
    #'
    initialize = function(file1 = NULL, file2 = NULL) {
      self$file1 <- file1
      self$file2 <- file2
      self$details_comparison <- list("summary" = NULL, "full" = NULL)
    },

    #' @description
    #' Method for comparing the file summary information. This method is
    #' intended to be implemented only this class level. For comparator
    #' specific rules, the internal method vrf_summary_inner should be
    #' customized on lower levels instead.
    #'
    #' @param config configuration values
    #' @param omit   string pattern to omit from the comparison
    #'
    vrf_summary = function(config, omit = NULL) {
      self$vrf_open_debug("File::vrf_summary", config)
      self$vrf_add_debug_files()

      if (!is.null(self$summary_comparison)) {
        self$vrf_add_debug("Returning previously calculated comparison results")
        return(self$summary_comparison)
      }

      if (!file.exists(self$file1) || !file.exists(self$file2)) {
        self$vrf_add_debug("One of both of the files not available, unable perform comparison")
        result <- "File(s) not available; unable to compare."
      } else {
        tryCatch({
          result   <- self$vrf_summary_inner(config, omit)
          addition <- self$vrf_details_supported(config)

          if ('' != addition) {
            result <- paste(result, addition)
          }
        }, error = function(e) {
          self$vrf_add_debug(paste("Processing failed with exception: ", conditionMessage(e)))
          result <- paste0("Error reading file contents: ", conditionMessage(e))
        })

      }

      self$summary_comparison <- result
      self$vrf_close_debug()

      return(result)
    },

    #' @description
    #' Method for comparing the file details information. This method is
    #' intended to be implemented only this class level. For comparator
    #' specific rules, the internal method vrf_summary_inner should be
    #' customized on lower levels instead.
    #'
    #' @param config configuration values
    #' @param omit   string pattern to omit from the comparison
    #'
    vrf_details = function(config, omit = NULL) {
      mode <- self$vrf_option_value(config, "details.mode")
      if ("NA" == mode) {
        mode <- "summary"
      }

      self$vrf_open_debug(paste("File::vrf_details, mode:", mode), config)
      self$vrf_add_debug_files()

      if (!is.null(self$details_comparison[[mode]])) {
        self$vrf_add_debug("Returning previously calculated comparison results")
        return(self$details_comparison[[mode]])
      }

      if (!file.exists(self$file1) || !file.exists(self$file2)) {
        self$vrf_add_debug("One of both of the files not available, unable perform comparison")
        result <- list(
          list(
            type = "text",
            contents = "File(s) not available; unable to compare."
          )
        )
      } else if ('' != self$vrf_details_supported(config)) {
        self$vrf_add_debug("Details comparison not supported/enabled, unable perform comparison")
        result <- list(
          list(
            type = "text",
            contents = self$vrf_details_supported(config)
          )
        )
      } else {
        tryCatch({
          result <- self$vrf_details_inner(config, omit)
        }, error = function(e) {
          self$vrf_add_debug(paste("Processing failed with exception: ", conditionMessage(e)))
          result <- list(
            list(
              type = "text",
              contents = paste0("Error reading file contents: ", conditionMessage(e))
            )
          )
        })
      }

      self$details_comparison[[mode]] <- result
      self$vrf_close_debug()

      return(result)
    },

    #' @description
    #' "Abstract" method for comparing the inner part for the summary. This
    #' method has to be overwritten by more specialized comparator classes. This
    #' method is intended to be called only by the comparator classes in the
    #' processing and shouldn't be called directly by the user.
    #'
    #' @param config configuration values
    #' @param omit   string pattern to omit from the comparison
    #'
    vrf_summary_inner = function(config, omit) {
      stop("vrf_summary_inner must be implemented in a subclass.")
    },

    #' @description
    #' "Abstract" method for comparing the inner part for the detailsThis method
    #' has to be overwritten by more specialized comparator classes. This method
    #' is intended to be called only by the comparator classes in the processing
    #' and shouldn't be called directly by the user.
    #'
    #' @param config configuration values
    #' @param omit   string pattern to omit from the comparison
    #'
    vrf_details_inner = function(config, omit) {
      stop("vrf_details_inner must be implemented in a subclass.")
    },

    #' @description
    #' Inherited method for indicating whether detailed comparison is available
    #' with the current comparator. Returns an empty string if the comparator is
    #' is supported, otherwise a string that will be concatenated with the
    #' summary string.
    #'
    #' @param config configuration values
    #'
    vrf_details_supported = function(config) {
      return("No details comparison available.")
    },

    #' @description
    #' Method for getting specific value from the config In the initial
    #' version, returns 'NA' if null con is passed.
    #'
    #' @param config configuration values
    #' @param key    key to search from the parameters
    #'
    vrf_option_value = function(config, key) {
      if (is.null(config)) {
        return("NA")
      }
      value <- config$get(key)
      return(value)
    },

    #' @description
    #' Wrapper method for the opening a new debugging instance with Debugger
    #' class if debugging is enabled in config class. Creates the used
    #' debugger instance if needed.
    #'
    #' @param message message to debug to console
    #' @param config  configuration values
    #'
    vrf_open_debug = function(message, config) {
      if ("yes" != self$vrf_option_value(config, "generic.debug")) {
        return()
      }

      if (is.null(self$debugger)) {
        self$debugger <- Debugger$new()
      }

      self$debugger$open_debug(message)
    },

    #' @description
    #' Wrapper method for the adding a new debugging message with Debugger
    #' class.
    #'
    #' @param message message to debug to console
    #'
    vrf_add_debug = function(message) {
      if (is.null(self$debugger)) {
        return()
      }

      self$debugger$add_debug(message)
    },

    #' @description
    #' Special method for adding the compared files into debugger stack.
    #'
    vrf_add_debug_files = function() {
      if (is.null(self$debugger)) {
        return()
      }

      self$debugger$add_debug(paste("File 1:", self$file1))
      self$debugger$add_debug(paste("File 2:", self$file2))
    },

    #' @description
    #' Wrapper method for the stopping (closing) current debugging instance with
    #' Debugger class.
    #'
    vrf_close_debug = function() {
      if (is.null(self$debugger)) {
        return()
      }
      self$debugger$close_debug()
    }
  )
)
# nolint end: cyclocomp_linter

#' Call for shiny example where the user can test verifyr2 package functions
#'
#' \code{verifyr2::run_example} returns simple Shiny App where user can see how
#' the verifyr2 functions work
#'
#' @export
#'
#' @param debug option to override debug configuration (TRUE only)
#'
run_example <- function(debug = FALSE) {
  appDir <- system.file("shiny_examples", "app", package = "verifyr2")

  if (appDir == "") {
    stop("Could not find example directory. Try re-installing `verifyr2`.",
         call. = FALSE)
  }

  options(verifyr2.debug = debug)
  shiny::runApp(appDir, display.mode = "normal")
}

#' TxtWithImageFileComparator.R
#'
#' "Abstract"  comparator for txt based comparator classes that can additionally
#' contain embedded images. This abstraction level contains generic logic for
#' handling embedded images and storing the related data.
#'
#' @import stringr
#'
#' @include TxtFileComparator.R
#'
#' @field file1_images_raw local property for storing image1 raw data
#' @field file2_images_raw local property for storing image2 raw data
#'

# Disable cyclomatic complexity lint for the R6 class definition as lintr
# considers the whole class definition as a single function.
#
# nolint start: cyclocomp_linter
TxtWithImagesFileComparator <- R6::R6Class(
  "TxtWithImagesFileComparator",
  inherit = TxtFileComparator,
  public = list(
    file1_images_raw = NULL,
    file2_images_raw = NULL,

    #' @description
    #' Initialize a TxtWithImagesFileComparator instance
    #'
    #' @param file1 First file to compare.
    #' @param file2 Second file to compare.
    #'
    initialize = function(file1 = NULL, file2 = NULL) {
      super$initialize(file1, file2)
    },

    #' @description
    #' Method for comparing the inner part for the details query. This method
    #' can be overwritten by more specialized comparator classes. This method is
    #' intended to be called only by the comparator classes in the processing
    #' and shouldn't be called directly by the user.
    #'
    #' @param config configuration values
    #' @param omit   string pattern to omit from the comparison
    #'
    vrf_summary_inner = function(config, omit) {
      self$vrf_open_debug("TxtWithImages::vrf_summary_inner" , config)

      result <- super$vrf_summary_inner(config, omit)

      if ("no" != super$vrf_option_value(config, "generic.images")) {
        file1_contents_list <- self$file1_contents_list
        file2_contents_list <- self$file2_contents_list

        if (is.null(self$file1_images_raw)) {
          self$file1_images_raw <- self$vrf_images(self$file1, config)
        }

        if (is.null(self$file2_images_raw)) {
          self$file2_images_raw <- self$vrf_images(self$file2, config)
        }

        # Generate additional summary string based on embedded image differences
        # if applicable.
        if (0 != length(self$file1_images_raw) && 0 != length(self$file2_images_raw)) {
          result_images <- "No differences in embedded images."
  
          if (length(self$file1_images_raw) != length(self$file2_images_raw)) {
            # Number of found embedded images differs between the files.
            result_images <- "Different amount of embedded images."
          } else {
            # Number of found embedded images is the same; calculate how many of
            # the embedded images has changed (based on raw file data) compared to
            # total count.
            matches <- 0
            total <- length(self$file1_images_raw)

            for (index in seq_along(self$file1_images_raw)) {
              if (identical(self$file1_images_raw[[index]], self$file2_images_raw[[index]])) {
                matches <- matches + 1
              }
            }

            if (matches != length(self$file1_images_raw)) {
              result_images <- paste0(total - matches, "/", total, " embedded images have differences.")
            }
          }
          result <- paste0(result, " ", result_images)
        }
      }

      self$vrf_close_debug()
      return(result)
    },

    #' @description
    #' Method for comparing the inner part for the details query. This method
    #' can be overwritten by more specialized comparator classes. This method is
    #' intended to be called only by the comparator classes in the processing
    #' and shouldn't be called directly by the user.
    #'
    #' @param config configuration values
    #' @param omit   string pattern to omit from the comparison
    #'
    vrf_details_inner = function(config, omit) {
      self$vrf_open_debug("TxtWithImages::vrf_details_inner" , config)

      result <- super$vrf_details_inner(config, omit)

      if ("no" != super$vrf_option_value(config, "generic.images")) {
        file1_contents_list <- self$file1_contents_list
        file2_contents_list <- self$file2_contents_list

        if (is.null(self$file1_images_raw)) {
          self$file1_images_raw <- self$vrf_images(self$file1, config)
        }

        if (is.null(self$file2_images_raw)) {
          self$file2_images_raw <- self$vrf_images(self$file2, config)
        }

        # Append the possible extended images into the result list if applicable.
        # List of images is included in the fileX_contents_lists if found from
        # content getter.
        if (0 != length(self$file1_images_raw) && 0 != length(self$file2_images_raw)) {

          # Only display the differences if there is the same amount of images
          # found from the compared files. Otherwise it would require additional
          # logic to decide which files should be compared with each others (which
          # is something that could be developed further with size comparisons).
          if (length(self$file1_images_raw) == length(self$file2_images_raw)) {
            for (index in seq_along(self$file1_images_raw)) {
              # Manually create a ImgFileComparator instance for every embedded
              # image and call the details comparison based on existing bin data.
              comparator <- ImgFileComparator$new(
                NULL,
                NULL,
                self$file1_images_raw[[index]],
                self$file2_images_raw[[index]]
              )
              result <- append(result, comparator$vrf_details_inner(config, omit))
            }
          }
        }
      }

      self$vrf_close_debug()
      return(result)
    },

    #' @description
    #' "Abstract" method for getting the raw image hex vector array from the
    #' given source file.
    #'
    #' @param file file for which to get the embedded image details
    #'
    vrf_images = function(file) {
      stop("vrf_images must be implemented in a subclass.")
    },

    #' @description
    #' Internal helper method for converting a hex string to raw vector.
    #'
    #' @param hex_string hexadecimal string to be converted to raw vector
    #'
    hex2raw = function(hex_string) {
      # Remove non-hex characters
      hex_string <- gsub("[^0-9a-fA-F]", "", hex_string)

      # check that the input string is a hex string
      if (nchar(hex_string) %% 2 != 0 || nchar(hex_string) < 2) {
        stop("input string isn't a hex string")
      }

      # Generate start and end indices
      start_indices <- seq(1, nchar(hex_string), 2)
      end_indices <- seq(2, nchar(hex_string), 2)

      # Extract byte-sized chunks
      bytes <- substring(hex_string, start_indices, end_indices)

      as.raw(as.hexmode(bytes))
    }
  )
)
# nolint end: cyclocomp_linter

#' PdfFileComparator.R
#'
#' Specialiced comparator for PDF file comparison.
#' This comparator contains the custom handling for handling only PDF content
#' part for the comparison.
#'
#' @include BinaryFileComparator.R
#' @include TxtFileComparator.R
#'
#' @examples
#'
#' # The normal way for creating a comparator would be to call the generic
#' # factory method verifyr2::create_comparator that will automatically create
#' # the correct comparator instance based on the file types.
#'
#' file1 <- 'my_file1.pdf'
#' file2 <- 'my_file2.pdf'
#' comparator <- verifyr2::create_comparator(file1, file2)
#'
#' # If needed, an explicit comparator can be created as well.
#'
#' file1 <- 'my_file1.pdf'
#' file2 <- 'my_file2.pdf'
#' comparator <- PdfFileComparator$new(file1, file2)
#'
#' @export
#'
PdfFileComparator <- R6::R6Class(
  "PdfFileComparator",
  inherit = TxtFileComparator,
  public = list(

    #' @description
    #' Method for getting the single file contents for the comparison. The
    #' method returns the file contents in two separate vectors inside a list.
    #' The first vector is the file contents and the second one is the file
    #' contents with the rows matching the omit string excluded. This method
    #' can be overwritten by more specialized comparator classes. This method
    #' is intended to be called only by the comparator classes in the processing
    #' and shouldn't be called directly by the user.
    #'
    #' @param file   file for which to get the contents
    #' @param config configuration values
    #' @param omit   string pattern to omit from the comparison
    #'
    vrf_contents = function(file, config, omit) {
      self$vrf_open_debug("Pdf::vrf_contents", config)

      if ("no" == super$vrf_option_value(config, "pdf.details")) {
        result <- super$vrf_contents(file, config, omit)

        self$vrf_close_debug()
        return(result)
      }

      content <- pdftools::pdf_text(file)
      content <- paste(content, collapse = "")
      content <- strsplit(content, "\n")[[1]]

      result <- self$vrf_contents_inner(content, config, omit)

      self$vrf_close_debug()
      result
    },

    #' @description
    #' Inherited method for indicating whether detailed comparison is available
    #' with the current comparator. Returns an empty string if the comparator is
    #' is supported, otherwise a string that will be concatenated with the
    #' summary string.
    #'
    #' @param config configuration values
    #'
    vrf_details_supported = function(config) {
      if ("no" == super$vrf_option_value(config, "pdf.details")) {
        return("Pdf details comparison disabled.")
      }
      super$vrf_details_supported(config)
    }
  )
)

library(shinytest2)

test_that("{shinytest2} recording: app_startup generic", {
  app <- AppDriver$new(
    variant = platform_variant(),
    name = "app_startup",
    height = 777,
    width = 1598
  )
  app$expect_screenshot()
})

test_that("{shinytest2} recording: app_startup input fields", {
  app <- AppDriver$new(
    name = "app_startup2",
    height = 777,
    width = 1598
  )
  # manually test the file & folder input values
  path_val <- app$get_value(input = "file1")
  expect_true(grepl("file2_additional_rows.rtf$", path_val), info = path_val)

  path_val <- app$get_value(input = "file2")
  expect_true(grepl("file3_changed_rows.rtf$", path_val), info = path_val)

  path_val <- app$get_value(input = "folder1")
  expect_true(grepl("base_files$", path_val), info = path_val)

  path_val <- app$get_value(input = "folder2")
  expect_true(grepl("compare_files$", path_val), info = path_val)
})

library(shinytest2)

#' helper method for creating the app for testing
create_app <- function(name) {
  app_path <- system.file("shiny_examples/app", package = "verifyr2")
  app <- AppDriver$new(app_path, name = name, height = 544, width = 941)
  app
}

compare_button <- function(index) {
  button <- paste0(
    "<button class=\"btn btn-default action-button process_button\" ",
    "id=\"process_",
    index,
    "\" onclick=\"Shiny.setInputValue(&#39;process_row&#39;, ",
    index,
    ")\" type=\"button\">\n  ",
    "<span class=\"action-label\">Compare</span>\n</button>"
  )
  button
}

#' test folders and files separately as the app & snapshot contain full
#' resolved paths that differ depending on user machine.
custom_expect_values <- function(app) {
  expect_true(grepl(
    "base_files",
    app$get_value(input = "folder1")
  ))

  expect_true(grepl(
    "compare_files",
    app$get_value(input = "folder2")
  ))

  expect_true(grepl(
    "file2_additional_rows.rtf",
    app$get_value(input = "file1")
  ))

  expect_true(grepl(
    "file3_changed_rows.rtf",
    app$get_value(input = "file2")
  ))

  vals    <- app$get_values()
  exclude <- c("folder1", "folder2", "file1", "file2")

  app$expect_values(
    input = setdiff(names(vals$input), exclude)
  )
}

test_that("{shinytest2} recording: startup", {
  skip_on_cran()
  app <- create_app("startup")

  custom_expect_values(app)
  app$stop()
})

test_that("{shinytest2} recording: folder_compare", {
  skip_on_cran()
  app <- create_app("folder_compare")

  app$click("go")

  app$wait_for_idle(500)
  custom_expect_values(app)
  app$stop()
})

test_that("{shinytest2} recording: file_compare", {
  skip_on_cran()
  app <- create_app("file_compare")

  app$set_inputs(compare_tabs = "tabs_file")
  app$click("go")

  app$wait_for_idle(500)
  custom_expect_values(app)
  app$stop()
})

test_that("{shinytest2} recording: options_reset", {
  skip_on_cran()
  app <- create_app("options_reset")

  app$click("configure")
  app$wait_for_idle()
  app$click("reset_config_modal")

  custom_expect_values(app)
  app$stop()
})

test_that("{shinytest2} recording: comment_persistence", {
  skip_on_cran()
  app <- create_app("comment_persistence")

  cur <- c(1, 2, 3, 4, 5, 6, 7, 8, 9, 10)
  all <- c(1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12)

  app$click("go")
  app$set_inputs(
    summary_out_rows_current = cur,
    allow_no_input_binding_ = TRUE,
    wait_ = FALSE
  )

  app$set_inputs(
    summary_out_rows_all = all,
    allow_no_input_binding_ = TRUE,
    wait_ = FALSE
  )

  app$set_inputs(
    summary_out_state = c(
      1772479993305,
      0,
      10,
      "",
      TRUE,
      FALSE,
      TRUE,
      c(TRUE, "", TRUE, FALSE, TRUE),
      c(TRUE, "", TRUE, FALSE, TRUE),
      c(TRUE, "", TRUE, FALSE, TRUE),
      c(TRUE, "", TRUE, FALSE, TRUE),
      c(TRUE, "", TRUE, FALSE, TRUE),
      c(TRUE, "", TRUE, FALSE, TRUE),
      c(FALSE, "", TRUE, FALSE, TRUE),
      c(TRUE, "", TRUE, FALSE, TRUE)
    ),
    allow_no_input_binding_ = TRUE
  )

  # click compare on row 2
  app$set_inputs(process_row = 2, allow_no_input_binding_ = TRUE)
  app$set_inputs(
    summary_out_cell_clicked = c("2", "7", compare_button("2")),
    allow_no_input_binding_ = TRUE,
    priority_ = "event"
  )
  app$wait_for_idle(500)

  # check that comment field is empty
  expect_equal(app$get_value(input = "details_out_comments"), "")

  # set and store comment
  app$set_inputs(details_out_comments = "comment2")
  app$click("save_comments")
  expect_equal(app$get_value(input = "details_out_comments"), "comment2")

  app$set_inputs(summary_out_state = c(
    1772480019284,
    0,
    10,
    "",
    TRUE,
    FALSE,
    TRUE,
    c(TRUE, "", TRUE, FALSE, TRUE),
    c(TRUE, "", TRUE, FALSE, TRUE),
    c(TRUE, "", TRUE, FALSE, TRUE),
    c(TRUE, "", TRUE, FALSE, TRUE),
    c(TRUE, "", TRUE, FALSE, TRUE),
    c(TRUE, "", TRUE, FALSE, TRUE),
    c(FALSE, "", TRUE, FALSE, TRUE),
    c(TRUE, "", TRUE, FALSE, TRUE)
  ), allow_no_input_binding_ = TRUE)

  # click compare on row 1
  app$set_inputs(process_row = 1, allow_no_input_binding_ = TRUE)
  app$set_inputs(
    summary_out_cell_clicked = c("1", "7", compare_button("1")),
    allow_no_input_binding_ = TRUE,
    priority_ = "event"
  )
  app$wait_for_idle(500)

  # check that comment field is empty - set comment but do not save
  expect_equal(app$get_value(input = "details_out_comments"), "")
  app$set_inputs(details_out_comments = "comment1")

  # click compare on row 2
  app$set_inputs(process_row = 2, allow_no_input_binding_ = TRUE)
  app$set_inputs(
    summary_out_cell_clicked = c("2", "7", compare_button("2")),
    allow_no_input_binding_ = TRUE,
    priority_ = "event"
  )
  app$wait_for_idle(500)

  # check that comment field contains value "comment2"
  expect_equal(app$get_value(input = "details_out_comments"), "comment2")

  # click compare on row 1
  app$set_inputs(process_row = 1, allow_no_input_binding_ = TRUE)
  app$set_inputs(
    summary_out_cell_clicked = c("1", "7", compare_button("1")),
    allow_no_input_binding_ = TRUE,
    priority_ = "event"
  )
  app$wait_for_idle(500)

  # check that comment field is empty (previous value was not saved)
  expect_equal(app$get_value(input = "details_out_comments"), "")
})

---
title: "File Comparison Report"
output:
  html_document:
    theme: flatly
    highlight: tango
    self_contained: true
params:
  # Comparison data frame containing file paths, results and comments
  dt:       NULL
  # Optional string pattern used to omit rows from comparisons
  omit_val: NULL
  # Used verifyr2 library version
  version:  NULL
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(echo = FALSE, warning = FALSE, message = FALSE)

dt       <- params$dt
omit_val <- params$omit_val
version  <- params$version
n_total  <- nrow(dt)
n_pass   <- sum(grepl("No differences", dt$comparison))
n_fail   <- n_total - n_pass
gh_link  <- "https://github.com/ejuhjav/verifyr2"

# Pre-render all FAIL diffs
diff_results <- vector("list", n_total)

# Initialise config once outside the loop
opts <- verifyr2::Config$new()
opts$set("details.mode", "summary")

for (i in seq_len(n_total)) {
  if (grepl("No differences", dt$comparison[i])) next

  comparator <- verifyr2::create_comparator(dt$file1[i], dt$file2[i])

  details <- tryCatch(
    comparator$vrf_details(config = opts, omit = omit_val),
    error = function(e) {
      list(list(
        type    = "text",
        content = paste0(
          "<p style='color:red;'>Error: ", conditionMessage(e), "</p>"
        )
      ))
    }
  )

  # Collect all detail items — a file can return multiple items
  item_html <- character(0)
  for (d in details) {
    if (d$type == "text") {
      item_html <- c(item_html, as.character(d$content))
    } else if (d$type == "image") {
      img1      <- d$content$image1
      img2      <- d$content$image2
      img3      <- d$content$image3
      img3_html <- if (!is.null(img3)) {
        paste0(
          "<div style='flex:1;min-width:200px;'>",
          "<div class='path-label'>Difference</div>",
          "<img src='", img3, "' style='width:100%;border:1px solid #eee;'/>",
          "</div>"
        )
      } else {
        ""
      }
      item_html <- c(item_html, paste0(
        "<div style='display:flex;gap:12px;flex-wrap:wrap;margin-top:12px;'>",
        "<div style='flex:1;min-width:200px;'>",
        "<div class='path-label'>Base</div>",
        "<img src='", img1, "' style='width:100%;border:1px solid #eee;'/>",
        "</div>",
        "<div style='flex:1;min-width:200px;'>",
        "<div class='path-label'>Compare</div>",
        "<img src='", img2, "' style='width:100%;border:1px solid #eee;'/>",
        "</div>",
        img3_html,
        "</div>"
      ))
    }
  }
  diff_results[[i]] <- paste(item_html, collapse = "")
}
```

```{r load_styles, results='asis', echo=FALSE}
css_file <- "report_styles.css"
if (file.exists(css_file)) {
  cat(
    "<style>\n",
    paste(readLines(css_file, warn = FALSE), collapse = "\n"),
    "\n</style>\n"
  )
}
```

```{r header, results='asis'}
logo_path <- "verifyr2.ico"
logo_html <- ""
if (file.exists(logo_path)) {
  logo_b64  <- base64enc::base64encode(logo_path)
  logo_html <- paste0(
    "<a href='", gh_link, "' title='verifyr2 in github' target='_blank'>",
    "<img src='data:image/x-icon;base64,",
    base64enc::base64encode(logo_path),
    "'/>",
    "</a>"
  )
}
omit_html <- if (!is.null(omit_val)) {
  paste0(
    " &nbsp;|&nbsp; <strong>Omit pattern:</strong> <code>",
    omit_val,
    "</code>"
  )
} else {
  ""
}
cat(paste0(
  "<div class='report-header'>",
  logo_html,
  "<strong>Generated:</strong> ",
  format(Sys.time(), "%Y-%m-%d %H:%M:%S"),
  omit_html,
  " &nbsp;|&nbsp; <strong>Generated with</strong> ",
  "<a href='", gh_link, "' title='verifyr2 in github' target='_blank'>",
  "verifyr2 (v", version, ")",
  "</a>",
  "</div>"
))
```

## Overview

```{r overview}
knitr::kable(
  data.frame(
    "Total files" = n_total,
    Match = paste0(
      "<span style='color:#5a8a20;font-weight:700;'>", n_pass, "</span>"
    ),
    Mismatch = paste0(
      "<span style='color:#c04820;font-weight:700;'>", n_fail, "</span>"
    ),
    check.names = FALSE
  ),
  format = "html",
  align  = c("c", "c", "c"),
  escape = FALSE
)
```

## Summary

```{r summary_table}
is_match    <- grepl("No differences", dt$comparison)
badge_match <- paste0(
  "<span class='badge-match' style='padding:2px 8px;border-radius:20px;",
  "font-size:11px;font-weight:700;background:#e8f5d0;color:#5a8a20;'>",
  "Match</span>"
)
badge_mismatch <- paste0(
  "<span class='badge-mismatch' style='padding:2px 8px;border-radius:20px;",
  "font-size:11px;font-weight:700;background:#fde9e0;color:#c04820;'>",
  "Mismatch</span>"
)
status_html <- ifelse(is_match, badge_match, badge_mismatch)
file_links <- paste0(
  "<a href='#detail-", seq_len(nrow(dt)), "'>",
  basename(dt$file1),
  "</a>"
)
knitr::kable(
  data.frame(
    File     = file_links,
    Result   = dt$comparison,
    Status   = status_html,
    Comments = ifelse(
      !is.null(dt$comments_details) & dt$comments_details != "",
      dt$comments_details,
      "\u2014"
    ),
    stringsAsFactors = FALSE
  ),
  format = "html",
  align  = c("l", "l", "c", "l"),
  escape = FALSE
)
```

## Details

```{r details, results='asis'}
for (i in seq_len(nrow(dt))) {
  file1   <- dt$file1[i]
  file2   <- dt$file2[i]
  result  <- dt$comparison[i]
  comment <- dt$comments_details[i]
  is_pass <- grepl("No differences", result)
  status  <- if (is_pass) "Match" else "Mismatch"
  section_class <- if (is_pass) "file-match" else "file-mismatch"
  badge_class   <- if (is_pass) "badge-match" else "badge-mismatch"

  comment_html <- if (!is.null(comment) && comment != "") {
    paste0(
      "<div class='comment-box'><strong>Comment:</strong> ", comment, "</div>"
    )
  } else {
    ""
  }

  cat(paste0(
    "\n<div id='detail-", i, "' class='file-section ", section_class, "'>",
    "<div class='file-header'>",
    "<span class='file-name'>",
    "<strong>", basename(file1), "</strong>",
    " [<a href='#header'>back to top</a>]",
    "</span>",
    "<span class='status-badge ", badge_class, "'>", status, "</span>",
    "</div>",
    "<div class='file-paths'>",
    "<span class='path-label'>Base</span><br/>", file1, "<br/>",
    "<span class='path-label'>Compare</span><br/>", file2,
    "</div>",
    "<div class='file-result'>",
    "<span>", result, "</span>",
    comment_html,
    "</div>"
  ))

  if (is_pass) {
    cat(paste0(
      "<div class='diff-output match-message'>",
      "&#10003;&nbsp; Files are identical.",
      "</div>"
    ))
  } else {
    diff_html <- diff_results[[i]]
    cat("<div class='diff-output'>")
    if (!is.null(diff_html) && nchar(diff_html) > 0) {
      cat(diff_html)
    } else {
      cat(paste0("<p class='no-diff'>", result, "</p>"))
    }
    cat("</div>")
  }

  cat("</div>\n")
}
```

---
title: "File Comparison Report"
output:
  html_document:
    theme: flatly
    highlight: tango
    self_contained: true
params:
  # Comparison data frame containing file paths, results and comments
  dt:       NULL
  # Optional string pattern used to omit rows from comparisons
  omit_val: NULL
  # Used verifyr2 library version
  version:  NULL
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(echo = FALSE, warning = FALSE, message = FALSE)

dt         <- params$dt
omit_val   <- params$omit_val
version    <- params$version
n_total    <- nrow(dt)
n_match    <- sum(grepl("No differences", dt$comparison))
n_mismatch <- n_total - n_match
gh_link    <- "https://github.com/ejuhjav/verifyr2"

# Pre-render all FAIL diffs
diff_results <- vector("list", n_total)

# Initialise config once outside the loop
opts <- verifyr2::Config$new()
opts$set("details.mode", "summary")
opts$set("generic.level", "word")

for (i in seq_len(n_total)) {
  if (grepl("No differences", dt$comparison[i])) next

  comparator <- verifyr2::create_comparator(dt$file1[i], dt$file2[i])

  details <- tryCatch(
    comparator$vrf_details(config = opts, omit = omit_val),
    error = function(e) {
      list(list(
        type    = "text",
        content = paste0(
          "<p style='color:red;'>Error: ", conditionMessage(e), "</p>"
        )
      ))
    }
  )

  # Collect all detail items — a file can return multiple items
  item_html <- htmltools::tagList()

  for (d in details) {
    content <- d$content

    # S4 object returned by diffobj is not directly compatible with htmltools
    if ("text" == d$type) {
      content <- htmltools::HTML(as.character(content))
    }

    item_html <- htmltools::tagAppendChildren(
      item_html,
      content
    )
  }

  diff_results[[i]] <- item_html
}
```

```{r load_styles, results='asis', echo=FALSE}
css_file1 <- "report_styles.css"
css_file2 <- "image_diff_styles.css"

if (file.exists(css_file1) && file.exists(css_file2)) {
  htmltools::tags$style(
    includeText(css_file1),
    includeText(css_file2)
  )
}

```

```{r header, results='asis'}
logo_path <- "verifyr2.ico"
logo_tag  <- NULL
omit_tag  <- NULL

if (file.exists(logo_path)) {
  logo_tag <- htmltools::tags$a(
    href = gh_link,
    title = "verifyr2 in github",
    target = "_blank",
    htmltools::tags$img(
      src = paste0(
        "data:image/x-icon;base64,",
        base64enc::base64encode(logo_path)
      )
    )
  )
}

if (!is.null(omit_val)) {
  omit_tag <- tagList(
    " \u00a0|\u00a0 ", # Safe unicode for &nbsp;
    htmltools::tags$strong("Omit pattern:"), " ",
    htmltools::tags$code(omit_val)
  )
}

report_header <- htmltools::tags$div(
  class = "report-header",
  logo_tag,
  htmltools::tags$strong("Generated:"),
  " ",
  format(Sys.time(), "%Y-%m-%d %H:%M:%S"),
  omit_tag,
  " \u00a0|\u00a0 ",
  htmltools::tags$strong("Generated with"), " ",
  htmltools::tags$a(
    href = gh_link,
    title = "verifyr2 in github",
    target = "_blank",
    paste0("verifyr2 (v", version, ")")
  )
)

report_header
```

## Overview

```{r overview}
htmltools::tags$table(
  class = "summary-table",

  htmltools::tags$thead(
    htmltools::tags$tr(
      htmltools::tags$th("Total files"),
      htmltools::tags$th("Match"),
      htmltools::tags$th("Mismatch")
    )
  ),

  htmltools::tags$tr(
    htmltools::tags$td(n_total),

    htmltools::tags$td(
      htmltools::tags$span(
        class = "summary-match",
        n_match
      )
    ),

    htmltools::tags$td(
      htmltools::tags$span(
        class = "summary-mismatch",
        n_mismatch
      )
    )
  )
)
```

## Summary

```{r summary_table}
status_badge <- function(comparison_result) {
  if (grepl("No differences", comparison_result)) {
    htmltools::tags$span(
      class = "status-badge badge-match",
      "Match"
    )
  } else {
    htmltools::tags$span(
      class = "status-badge badge-mismatch",
      "Mismatch"
    )
  }
}

rows <- lapply(seq_len(nrow(dt)), function(i) {
  htmltools::tags$tr(
    # File name as link
    htmltools::tags$td(
      htmltools::tags$a(
        href = paste0("#detail-", i),
        basename(dt$file1[i])
      )
    ),
    # Result
    htmltools::tags$td(
      dt$comparison[i]
    ),
    # Status
    htmltools::tags$td(
      class = "status-column",
      status_badge(dt$comparison[i])
    ),
    # Comments
    htmltools::tags$td(
      if (!is.null(dt$comments_details[i]) && nzchar(dt$comments_details[i])) {
        dt$comments_details[i]
      } else {
        "\u2014"
      }
    )
  )
})

table_html <- htmltools::tags$table(
  class = "diff-table",

  htmltools::tags$thead(
    htmltools::tags$tr(
      htmltools::tags$th("File"),
      htmltools::tags$th("Result"),
      htmltools::tags$th("Status"),
      htmltools::tags$th("Comments")
    )
  ),

  htmltools::tags$tbody(
    tagList(rows)
  )
)

table_html
```

## Details

```{r details, results='asis'}
detail_sections <- lapply(seq_len(nrow(dt)), function(i) {

  file1   <- dt$file1[i]
  file2   <- dt$file2[i]
  result  <- dt$comparison[i]
  comment <- dt$comments_details[i]

  is_pass <- grepl("No differences", result)

  section_class <- if (is_pass) "file-match" else "file-mismatch"
  badge_class   <- if (is_pass) "badge-match" else "badge-mismatch"

  # ---- Comment block (SAFE HTML via tags) ----
  comment_block <- if (!is.null(comment) && nzchar(comment)) {
    htmltools::tags$div(
      class = "comment-box",
      htmltools::tags$strong("Comment:"),
      " ",
      comment
    )
  } else {
    NULL
  }

  # ---- Header ----
  header <- htmltools::tags$div(
    class = "file-header",

    htmltools::tags$span(
      class = "file-name",
      htmltools::tags$strong(basename(file1)),
      " [",
      htmltools::tags$a(href = "#header", "back to top"),
      "]"
    ),

    htmltools::tags$span(
      class = badge_class,
      if (is_pass) "Match" else "Mismatch"
    )
  )

  # ---- Paths ----
  paths <- htmltools::tags$div(
    class = "file-paths",

    htmltools::tags$span(class = "path-label", "Base"),
    htmltools::tags$br(),
    file1,
    htmltools::tags$br(),

    htmltools::tags$span(class = "path-label", "Compare"),
    htmltools::tags$br(),
    file2
  )

  # ---- Result block ----
  result_block <- htmltools::tags$div(
    class = "file-result",
    htmltools::tags$span(result),
    comment_block
  )

  # ---- Diff output ----
  diff_html  <- diff_results[[i]]
  diff_block <- if (is_pass) {
    htmltools::tags$div(
      class = "diff-output match-message",
      htmltools::HTML("&#10003;&nbsp; Files are identical.")
    )
  } else {
    htmltools::tags$div(
      class = "diff-output",
      if (!is.null(diff_html) && length(diff_html) > 0) {
        diff_html
      } else {
        htmltools::tags$p(class = "no-diff", result)
      }
    )
  }

  # ---- Full section ----
  htmltools::tags$div(
    id = paste0("detail-", i),
    class = paste("file-section", section_class),
    header,
    paths,
    result_block,
    diff_block
  )
})

htmltools::tagList(detail_sections)
```

% Generated by roxygen2: do not edit by hand % Please edit documentation in R/TxtWithImagesComparator.R \name{TxtWithImagesFileComparator} \alias{TxtWithImagesFileComparator} \title{TxtWithImageFileComparator.R} \description{ TxtWithImageFileComparator.R TxtWithImageFileComparator.R } \details{ "Abstract" comparator for txt based comparator classes that can additionally contain embedded images. This abstraction level contains generic logic for handling embedded images and storing the related data. } \section{Super classes}{ \code{\link[verifyr2:FileComparator]{verifyr2::FileComparator}} -> \code{\link[verifyr2:BinaryFileComparator]{verifyr2::BinaryFileComparator}} -> \code{\link[verifyr2:TxtFileComparator]{verifyr2::TxtFileComparator}} -> \code{TxtWithImagesFileComparator} } \section{Public fields}{ \if{html}{\out{
}} \describe{ \item{\code{file1_images_raw}}{local property for storing image1 raw data} \item{\code{file2_images_raw}}{local property for storing image2 raw data} } \if{html}{\out{
}} } \section{Methods}{ \subsection{Public methods}{ \itemize{ \item \href{#method-TxtWithImagesFileComparator-new}{\code{TxtWithImagesFileComparator$new()}} \item \href{#method-TxtWithImagesFileComparator-vrf_summary_inner}{\code{TxtWithImagesFileComparator$vrf_summary_inner()}} \item \href{#method-TxtWithImagesFileComparator-vrf_details_inner}{\code{TxtWithImagesFileComparator$vrf_details_inner()}} \item \href{#method-TxtWithImagesFileComparator-vrf_images}{\code{TxtWithImagesFileComparator$vrf_images()}} \item \href{#method-TxtWithImagesFileComparator-hex2raw}{\code{TxtWithImagesFileComparator$hex2raw()}} \item \href{#method-TxtWithImagesFileComparator-clone}{\code{TxtWithImagesFileComparator$clone()}} } } \if{html}{\out{
Inherited methods

- `verifyr2::FileComparator$vrf_add_debug()`
- `verifyr2::FileComparator$vrf_add_debug_files()`
- `verifyr2::FileComparator$vrf_close_debug()`
- `verifyr2::FileComparator$vrf_details()`
- `verifyr2::FileComparator$vrf_open_debug()`
- `verifyr2::FileComparator$vrf_option_value()`
- `verifyr2::FileComparator$vrf_summary()`
- `verifyr2::BinaryFileComparator$vrf_contents()`
- `verifyr2::TxtFileComparator$vrf_contents_inner()`
- `verifyr2::TxtFileComparator$vrf_details_supported()`

}} \if{html}{\out{

}} \if{html}{\out{}} \if{latex}{\out{\hypertarget{method-TxtWithImagesFileComparator-new}{}}} \subsection{Method \code{new()}}{ Initialize a TxtWithImagesFileComparator instance \subsection{Usage}{ \if{html}{\out{
}}\preformatted{TxtWithImagesFileComparator$new(file1 = NULL, file2 = NULL)}\if{html}{\out{
}} } \subsection{Arguments}{ \if{html}{\out{
}} \describe{ \item{\code{file1}}{First file to compare.} \item{\code{file2}}{Second file to compare.} } \if{html}{\out{
}} } } \if{html}{\out{

}} \if{html}{\out{}} \if{latex}{\out{\hypertarget{method-TxtWithImagesFileComparator-vrf_summary_inner}{}}} \subsection{Method \code{vrf_summary_inner()}}{ Method for comparing the inner part for the details query. This method can be overwritten by more specialized comparator classes. This method is intended to be called only by the comparator classes in the processing and shouldn't be called directly by the user. \subsection{Usage}{ \if{html}{\out{
}}\preformatted{TxtWithImagesFileComparator$vrf_summary_inner(config, omit)}\if{html}{\out{
}} } \subsection{Arguments}{ \if{html}{\out{
}} \describe{ \item{\code{config}}{configuration values} \item{\code{omit}}{string pattern to omit from the comparison} } \if{html}{\out{
}} } } \if{html}{\out{

}} \if{html}{\out{}} \if{latex}{\out{\hypertarget{method-TxtWithImagesFileComparator-vrf_details_inner}{}}} \subsection{Method \code{vrf_details_inner()}}{ Method for comparing the inner part for the details query. This method can be overwritten by more specialized comparator classes. This method is intended to be called only by the comparator classes in the processing and shouldn't be called directly by the user. \subsection{Usage}{ \if{html}{\out{
}}\preformatted{TxtWithImagesFileComparator$vrf_details_inner(config, omit)}\if{html}{\out{
}} } \subsection{Arguments}{ \if{html}{\out{
}} \describe{ \item{\code{config}}{configuration values} \item{\code{omit}}{string pattern to omit from the comparison} } \if{html}{\out{
}} } } \if{html}{\out{

}} \if{html}{\out{}} \if{latex}{\out{\hypertarget{method-TxtWithImagesFileComparator-vrf_images}{}}} \subsection{Method \code{vrf_images()}}{ "Abstract" method for getting the raw image hex vector array from the given source file. \subsection{Usage}{ \if{html}{\out{
}}\preformatted{TxtWithImagesFileComparator$vrf_images(file)}\if{html}{\out{
}} } \subsection{Arguments}{ \if{html}{\out{
}} \describe{ \item{\code{file}}{file for which to get the embedded image details} } \if{html}{\out{
}} } } \if{html}{\out{

}} \if{html}{\out{}} \if{latex}{\out{\hypertarget{method-TxtWithImagesFileComparator-hex2raw}{}}} \subsection{Method \code{hex2raw()}}{ Internal helper method for converting a hex string to raw vector. \subsection{Usage}{ \if{html}{\out{
}}\preformatted{TxtWithImagesFileComparator$hex2raw(hex_string)}\if{html}{\out{
}} } \subsection{Arguments}{ \if{html}{\out{
}} \describe{ \item{\code{hex_string}}{hexadecimal string to be converted to raw vector} } \if{html}{\out{
}} } } \if{html}{\out{

}} \if{html}{\out{}} \if{latex}{\out{\hypertarget{method-TxtWithImagesFileComparator-clone}{}}} \subsection{Method \code{clone()}}{ The objects of this class are cloneable with this method. \subsection{Usage}{ \if{html}{\out{
}}\preformatted{TxtWithImagesFileComparator$clone(deep = FALSE)}\if{html}{\out{
}} } \subsection{Arguments}{ \if{html}{\out{
}} \describe{ \item{\code{deep}}{Whether to make a deep clone.} } \if{html}{\out{
}} } } }

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Debugger.R
\name{Debugger}
\alias{Debugger}
\title{Debugger.R}
\description{
Debugger.R

Debugger.R
}
\details{
Class for managing the library debugging. Tracks the debugged method
execution times and prints out the full debugging information only when
the main debugging instance is stopped.
}
\examples{

# Creates a debugger instance.

debugger <- Debugger$new()

# Opening and closing debugs in multileveled function calls.

function1 <- function() {
  debugger$open_debug("function1")
  function2()
  debuger$close_debug()
}

function2 <- function() {
  debugger$open_debug("function2")
  Sys.sleep(1)
  debugger$close_debug()
}

# This will produce the following printout to the console after the
# function1 finishes
#
# - 'function1' (execution time 7 ms)
#   - 'function2' (execution time 5 ms)

}
\section{Public fields}{
\if{html}{\out{<div class="r6-fields">}}
\describe{
\item{\code{stack}}{local property for storing debugging labels and start times}
}
\if{html}{\out{</div>}}
}
\section{Methods}{
\subsection{Public methods}{
\itemize{
\item \href{#method-Debugger-new}{\code{Debugger$new()}}
\item \href{#method-Debugger-open_debug}{\code{Debugger$open_debug()}}
\item \href{#method-Debugger-add_debug}{\code{Debugger$add_debug()}}
\item \href{#method-Debugger-close_debug}{\code{Debugger$close_debug()}}
\item \href{#method-Debugger-print_debug_tree}{\code{Debugger$print_debug_tree()}}
\item \href{#method-Debugger-clone}{\code{Debugger$clone()}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Debugger-new"></a>}}
\if{latex}{\out{\hypertarget{method-Debugger-new}{}}}
\subsection{Method \code{new()}}{
Constructor for initializing the Debugger instance.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Debugger$new()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Debugger-open_debug"></a>}}
\if{latex}{\out{\hypertarget{method-Debugger-open_debug}{}}}
\subsection{Method \code{open_debug()}}{
Method for opening new debug instance to the current debugging stack.
Stores also the start time for execution time calculation.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Debugger$open_debug(label)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{label}}{debugging message}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Debugger-add_debug"></a>}}
\if{latex}{\out{\hypertarget{method-Debugger-add_debug}{}}}
\subsection{Method \code{add_debug()}}{
Method for adding a new debug string under the currently open
debug instance.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Debugger$add_debug(label)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{label}}{debugging message}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Debugger-close_debug"></a>}}
\if{latex}{\out{\hypertarget{method-Debugger-close_debug}{}}}
\subsection{Method \code{close_debug()}}{
Method for closing a debug instance from the current debugging
stack. If the stopped debug instance is the main level one, the
whole debug data is printed out to console. If the stopped debug
instance is not the main level one, calculates the execution time
of current debug instance and updates the stack data.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Debugger$close_debug()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Debugger-print_debug_tree"></a>}}
\if{latex}{\out{\hypertarget{method-Debugger-print_debug_tree}{}}}
\subsection{Method \code{print_debug_tree()}}{
Recursive method for printing out the current debug stack items and
recursively all the item children. This method is called for the whole
stack once the topmost debug instance is stopped.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Debugger$print_debug_tree(entry, depth = 0)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{entry}}{current debug level being processed for printing}

\item{\code{depth}}{current processing depth for printing indentation}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Debugger-clone"></a>}}
\if{latex}{\out{\hypertarget{method-Debugger-clone}{}}}
\subsection{Method \code{clone()}}{
The objects of this class are cloneable with this method.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Debugger$clone(deep = FALSE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{deep}}{Whether to make a deep clone.}
}
\if{html}{\out{</div>}}
}
}
}

% Generated by roxygen2: do not edit by hand % Please edit documentation in R/PdfFileComparator.R \name{PdfFileComparator} \alias{PdfFileComparator} \title{PdfFileComparator.R} \description{ PdfFileComparator.R PdfFileComparator.R } \details{ Specialiced comparator for PDF file comparison. This comparator contains the custom handling for handling only PDF content part for the comparison. } \examples{ # The normal way for creating a comparator would be to call the generic # factory method verifyr2::create_comparator that will automatically create # the correct comparator instance based on the file types. file1 <- 'my_file1.pdf' file2 <- 'my_file2.pdf' comparator <- verifyr2::create_comparator(file1, file2) # If needed, an explicit comparator can be created as well. file1 <- 'my_file1.pdf' file2 <- 'my_file2.pdf' comparator <- PdfFileComparator$new(file1, file2) } \section{Super classes}{ \code{\link[verifyr2:FileComparator]{verifyr2::FileComparator}} -> \code{\link[verifyr2:BinaryFileComparator]{verifyr2::BinaryFileComparator}} -> \code{\link[verifyr2:TxtFileComparator]{verifyr2::TxtFileComparator}} -> \code{PdfFileComparator} } \section{Methods}{ \subsection{Public methods}{ \itemize{ \item \href{#method-PdfFileComparator-vrf_contents}{\code{PdfFileComparator$vrf_contents()}} \item \href{#method-PdfFileComparator-vrf_details_supported}{\code{PdfFileComparator$vrf_details_supported()}} \item \href{#method-PdfFileComparator-clone}{\code{PdfFileComparator$clone()}} } } \if{html}{\out{
Inherited methods

- `verifyr2::FileComparator$initialize()`
- `verifyr2::FileComparator$vrf_add_debug()`
- `verifyr2::FileComparator$vrf_add_debug_files()`
- `verifyr2::FileComparator$vrf_close_debug()`
- `verifyr2::FileComparator$vrf_details()`
- `verifyr2::FileComparator$vrf_open_debug()`
- `verifyr2::FileComparator$vrf_option_value()`
- `verifyr2::FileComparator$vrf_summary()`
- `verifyr2::TxtFileComparator$vrf_contents_inner()`
- `verifyr2::TxtFileComparator$vrf_details_inner()`
- `verifyr2::TxtFileComparator$vrf_summary_inner()`

}} \if{html}{\out{

}} \if{html}{\out{}} \if{latex}{\out{\hypertarget{method-PdfFileComparator-vrf_contents}{}}} \subsection{Method \code{vrf_contents()}}{ Method for getting the single file contents for the comparison. The method returns the file contents in two separate vectors inside a list. The first vector is the file contents and the second one is the file contents with the rows matching the omit string excluded. This method can be overwritten by more specialized comparator classes. This method is intended to be called only by the comparator classes in the processing and shouldn't be called directly by the user. \subsection{Usage}{ \if{html}{\out{
}}\preformatted{PdfFileComparator$vrf_contents(file, config, omit)}\if{html}{\out{
}} } \subsection{Arguments}{ \if{html}{\out{
}} \describe{ \item{\code{file}}{file for which to get the contents} \item{\code{config}}{configuration values} \item{\code{omit}}{string pattern to omit from the comparison} } \if{html}{\out{
}} } } \if{html}{\out{

}} \if{html}{\out{}} \if{latex}{\out{\hypertarget{method-PdfFileComparator-vrf_details_supported}{}}} \subsection{Method \code{vrf_details_supported()}}{ Inherited method for indicating whether detailed comparison is available with the current comparator. Returns an empty string if the comparator is is supported, otherwise a string that will be concatenated with the summary string. \subsection{Usage}{ \if{html}{\out{
}}\preformatted{PdfFileComparator$vrf_details_supported(config)}\if{html}{\out{
}} } \subsection{Arguments}{ \if{html}{\out{
}} \describe{ \item{\code{config}}{configuration values} } \if{html}{\out{
}} } } \if{html}{\out{

}} \if{html}{\out{}} \if{latex}{\out{\hypertarget{method-PdfFileComparator-clone}{}}} \subsection{Method \code{clone()}}{ The objects of this class are cloneable with this method. \subsection{Usage}{ \if{html}{\out{
}}\preformatted{PdfFileComparator$clone(deep = FALSE)}\if{html}{\out{
}} } \subsection{Arguments}{ \if{html}{\out{
}} \describe{ \item{\code{deep}}{Whether to make a deep clone.} } \if{html}{\out{
}} } } }

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/list_files.R
\name{list_files}
\alias{list_files}
\title{One row list for two distinct files}
\usage{
list_files(file1, file2)
}
\arguments{
\item{file1}{character, giving the the full file path of the first file}

\item{file2}{character, giving the the full file path of the second file}
}
\value{
Returns a tibble, \code{selected_files} with 2 columns \code{file1},
                \code{file2}
}
\description{
\code{list_files} List single file row based on the explicit parameter
files. This is a conveniency function for building same structure list for
direct two file comparison case.
}
\examples{

path1 <- "/extdata/base_files/file2_additional_rows.rtf"
file1 <- paste0(fs::path_package(path1, package = "verifyr2"))

path2 <- "/extdata/compare_files/file3_changed_rows.rtf"
file2 <- paste0(fs::path_package(path2, package = "verifyr2"))

verifyr2::list_files(file1, file2)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Config.R
\name{Config}
\alias{Config}
\title{Config.R}
\description{
Config.R

Config.R
}
\details{
Class for manging the library configuration options. Creates the default
configuration without any source file, populates partial or missing config
elements, stores the config file to local machine, and provides easy access
methods for setting and getting config values.
}
\examples{

# Creates the configuration instance. Checks automatically if there is
# a previously stored configuration json file available for usage. Note
# that you don't need to explicitly define the config file path for the
# Config instance - by default the config file will be searched from and
# written in user-specific configuration directory for the package.

path   <- tempfile(fileext = ".json")
config <- Config$new(config_path = path)

# Getting and setting configuration values

value <- config$get("defailts.mode")
config$set("details.mode", "full")

# Saving the current configuration to local machine (to tmp folder with
# the given explicit file path in initialization).

config$save()

}
\section{Public fields}{
\if{html}{\out{<div class="r6-fields">}}
\describe{
\item{\code{schema}}{configuration schema}

\item{\code{config}}{current configuration data}

\item{\code{path}}{configuration json file path}
}
\if{html}{\out{</div>}}
}
\section{Methods}{
\subsection{Public methods}{
\itemize{
\item \href{#method-Config-new}{\code{Config$new()}}
\item \href{#method-Config-get_schema_item}{\code{Config$get_schema_item()}}
\item \href{#method-Config-get}{\code{Config$get()}}
\item \href{#method-Config-set}{\code{Config$set()}}
\item \href{#method-Config-save}{\code{Config$save()}}
\item \href{#method-Config-get_default_config}{\code{Config$get_default_config()}}
\item \href{#method-Config-get_default_schema}{\code{Config$get_default_schema()}}
\item \href{#method-Config-clone}{\code{Config$clone()}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Config-new"></a>}}
\if{latex}{\out{\hypertarget{method-Config-new}{}}}
\subsection{Method \code{new()}}{
Constructor for initializing the configuration. Checks the local machine
for existing configuration file is load_config = TRUE. Ensures that all
the project configuration values are included.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Config$new(load_config = TRUE, config_path = NULL)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{load_config}}{load configuration from local machine if available}

\item{\code{config_path}}{location of the used/stored configuration json file}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Config-get_schema_item"></a>}}
\if{latex}{\out{\hypertarget{method-Config-get_schema_item}{}}}
\subsection{Method \code{get_schema_item()}}{
Method for getting schema item based on configuration key.
Configuration item children are separated with a dot in the key notation.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Config$get_schema_item(key)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{key}}{configuration property key for which to get the schema item}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Config-get"></a>}}
\if{latex}{\out{\hypertarget{method-Config-get}{}}}
\subsection{Method \code{get()}}{
Method for getting configuration value based on configuration key.
Configuration item children are separated with a dot in the key notation.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Config$get(key)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{key}}{configuration property key for which to get the value}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Config-set"></a>}}
\if{latex}{\out{\hypertarget{method-Config-set}{}}}
\subsection{Method \code{set()}}{
Method for setting configuration value based on configuration key.
Configuration item children are separated with a dot in the key notation.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Config$set(key, value)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{key}}{configuration property key for which to get the value}

\item{\code{value}}{value to set for the specified configuration key}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Config-save"></a>}}
\if{latex}{\out{\hypertarget{method-Config-save}{}}}
\subsection{Method \code{save()}}{
Method for saving the current configuration data into local machine.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Config$save()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Config-get_default_config"></a>}}
\if{latex}{\out{\hypertarget{method-Config-get_default_config}{}}}
\subsection{Method \code{get_default_config()}}{
Helper method for getting configuration default values. These default
values will be used in the configuration in case the configuration
properties are not present previously.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Config$get_default_config()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Config-get_default_schema"></a>}}
\if{latex}{\out{\hypertarget{method-Config-get_default_schema}{}}}
\subsection{Method \code{get_default_schema()}}{
Method for getting the full configuration schema. Apart from the
configuration data, the schema contains property descriptions as well as
all possible values for the configuration properties.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Config$get_default_schema()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Config-clone"></a>}}
\if{latex}{\out{\hypertarget{method-Config-clone}{}}}
\subsection{Method \code{clone()}}{
The objects of this class are cloneable with this method.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Config$clone(deep = FALSE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{deep}}{Whether to make a deep clone.}
}
\if{html}{\out{</div>}}
}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/FileComparator.R
\name{FileComparator}
\alias{FileComparator}
\title{FileComparator.R}
\description{
FileComparator.R

FileComparator.R
}
\details{
Comparator 'abstract' class containing the generic comparison methods and
handling for high level checks (like file existence). This class should never
be instantiated - doing that and calling the comparison methods will lead to
error.
}
\section{Public fields}{
\if{html}{\out{<div class="r6-fields">}}
\describe{
\item{\code{file1}}{file1}

\item{\code{file2}}{file2}

\item{\code{file1_contents_list}}{file1 contents}

\item{\code{file2_contents_list}}{file2 contents}

\item{\code{summary_comparison}}{summary comparison result}

\item{\code{details_comparison}}{details comparison result}

\item{\code{debugger}}{debugger instance}
}
\if{html}{\out{</div>}}
}
\section{Methods}{
\subsection{Public methods}{
\itemize{
\item \href{#method-FileComparator-new}{\code{FileComparator$new()}}
\item \href{#method-FileComparator-vrf_summary}{\code{FileComparator$vrf_summary()}}
\item \href{#method-FileComparator-vrf_details}{\code{FileComparator$vrf_details()}}
\item \href{#method-FileComparator-vrf_summary_inner}{\code{FileComparator$vrf_summary_inner()}}
\item \href{#method-FileComparator-vrf_details_inner}{\code{FileComparator$vrf_details_inner()}}
\item \href{#method-FileComparator-vrf_details_supported}{\code{FileComparator$vrf_details_supported()}}
\item \href{#method-FileComparator-vrf_option_value}{\code{FileComparator$vrf_option_value()}}
\item \href{#method-FileComparator-vrf_open_debug}{\code{FileComparator$vrf_open_debug()}}
\item \href{#method-FileComparator-vrf_add_debug}{\code{FileComparator$vrf_add_debug()}}
\item \href{#method-FileComparator-vrf_add_debug_files}{\code{FileComparator$vrf_add_debug_files()}}
\item \href{#method-FileComparator-vrf_close_debug}{\code{FileComparator$vrf_close_debug()}}
\item \href{#method-FileComparator-clone}{\code{FileComparator$clone()}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-FileComparator-new"></a>}}
\if{latex}{\out{\hypertarget{method-FileComparator-new}{}}}
\subsection{Method \code{new()}}{
Initialize a FileComparator instance
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{FileComparator$new(file1 = NULL, file2 = NULL)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{file1}}{First file to compare.}

\item{\code{file2}}{Second file to compare.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-FileComparator-vrf_summary"></a>}}
\if{latex}{\out{\hypertarget{method-FileComparator-vrf_summary}{}}}
\subsection{Method \code{vrf_summary()}}{
Method for comparing the file summary information. This method is
intended to be implemented only this class level. For comparator
specific rules, the internal method vrf_summary_inner should be
customized on lower levels instead.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{FileComparator$vrf_summary(config, omit = NULL)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{config}}{configuration values}

\item{\code{omit}}{string pattern to omit from the comparison}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-FileComparator-vrf_details"></a>}}
\if{latex}{\out{\hypertarget{method-FileComparator-vrf_details}{}}}
\subsection{Method \code{vrf_details()}}{
Method for comparing the file details information. This method is
intended to be implemented only this class level. For comparator
specific rules, the internal method vrf_summary_inner should be
customized on lower levels instead.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{FileComparator$vrf_details(config, omit = NULL)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{config}}{configuration values}

\item{\code{omit}}{string pattern to omit from the comparison}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-FileComparator-vrf_summary_inner"></a>}}
\if{latex}{\out{\hypertarget{method-FileComparator-vrf_summary_inner}{}}}
\subsection{Method \code{vrf_summary_inner()}}{
"Abstract" method for comparing the inner part for the summary. This
method has to be overwritten by more specialized comparator classes. This
method is intended to be called only by the comparator classes in the
processing and shouldn't be called directly by the user.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{FileComparator$vrf_summary_inner(config, omit)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{config}}{configuration values}

\item{\code{omit}}{string pattern to omit from the comparison}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-FileComparator-vrf_details_inner"></a>}}
\if{latex}{\out{\hypertarget{method-FileComparator-vrf_details_inner}{}}}
\subsection{Method \code{vrf_details_inner()}}{
"Abstract" method for comparing the inner part for the detailsThis method
has to be overwritten by more specialized comparator classes. This method
is intended to be called only by the comparator classes in the processing
and shouldn't be called directly by the user.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{FileComparator$vrf_details_inner(config, omit)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{config}}{configuration values}

\item{\code{omit}}{string pattern to omit from the comparison}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-FileComparator-vrf_details_supported"></a>}}
\if{latex}{\out{\hypertarget{method-FileComparator-vrf_details_supported}{}}}
\subsection{Method \code{vrf_details_supported()}}{
Inherited method for indicating whether detailed comparison is available
with the current comparator. Returns an empty string if the comparator is
is supported, otherwise a string that will be concatenated with the
summary string.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{FileComparator$vrf_details_supported(config)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{config}}{configuration values}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-FileComparator-vrf_option_value"></a>}}
\if{latex}{\out{\hypertarget{method-FileComparator-vrf_option_value}{}}}
\subsection{Method \code{vrf_option_value()}}{
Method for getting specific value from the config In the initial
version, returns 'NA' if null con is passed.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{FileComparator$vrf_option_value(config, key)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{config}}{configuration values}

\item{\code{key}}{key to search from the parameters}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-FileComparator-vrf_open_debug"></a>}}
\if{latex}{\out{\hypertarget{method-FileComparator-vrf_open_debug}{}}}
\subsection{Method \code{vrf_open_debug()}}{
Wrapper method for the opening a new debugging instance with Debugger
class if debugging is enabled in config class. Creates the used
debugger instance if needed.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{FileComparator$vrf_open_debug(message, config)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{message}}{message to debug to console}

\item{\code{config}}{configuration values}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-FileComparator-vrf_add_debug"></a>}}
\if{latex}{\out{\hypertarget{method-FileComparator-vrf_add_debug}{}}}
\subsection{Method \code{vrf_add_debug()}}{
Wrapper method for the adding a new debugging message with Debugger
class.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{FileComparator$vrf_add_debug(message)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{message}}{message to debug to console}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-FileComparator-vrf_add_debug_files"></a>}}
\if{latex}{\out{\hypertarget{method-FileComparator-vrf_add_debug_files}{}}}
\subsection{Method \code{vrf_add_debug_files()}}{
Special method for adding the compared files into debugger stack.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{FileComparator$vrf_add_debug_files()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-FileComparator-vrf_close_debug"></a>}}
\if{latex}{\out{\hypertarget{method-FileComparator-vrf_close_debug}{}}}
\subsection{Method \code{vrf_close_debug()}}{
Wrapper method for the stopping (closing) current debugging instance with
Debugger class.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{FileComparator$vrf_close_debug()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-FileComparator-clone"></a>}}
\if{latex}{\out{\hypertarget{method-FileComparator-clone}{}}}
\subsection{Method \code{clone()}}{
The objects of this class are cloneable with this method.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{FileComparator$clone(deep = FALSE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{deep}}{Whether to make a deep clone.}
}
\if{html}{\out{</div>}}
}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/FileComparatorFactory.R
\name{create_comparator}
\alias{create_comparator}
\title{FileComparatorFactory.R}
\usage{
create_comparator(file1, file2)
}
\arguments{
\item{file1}{first file to compare}

\item{file2}{second file to compare}
}
\description{
Factory method for creating comparator instance based on the given two files.
}
\examples{

# instantiating the compared files
file1 <- "file1.rtf"
file2 <- "file2.rtf"

# instantiating the configuration
config <- Config$new()

# instantiating a new comparator instance for every comparison:
comparator <- verifyr2::create_comparator(file1, file2)

# calling the summary and details comparison methods
comparator$vrf_summary(config = config)
comparator$vrf_details(config = config)

}

% Generated by roxygen2: do not edit by hand % Please edit documentation in R/ImgFileComparator.R \name{ImgFileComparator} \alias{ImgFileComparator} \title{ImgFileComparator.R} \description{ ImgFileComparator.R ImgFileComparator.R } \details{ Specialiced comparator for image file (jpg, jpef, png) comparison. This comparator contains the custom handling for handling only img content part for the comparison. } \examples{ # The normal way for creating a comparator would be to call the generic # factory method verifyr2::create_comparator that will automatically create # the correct comparator instance based on the file types. file1 <- 'my_file1.jpg' file2 <- 'my_file2.jpg' comparator <- verifyr2::create_comparator(file1, file2) # If needed, an explicit comparator can be created as well. file1 <- 'my_file1.png' file2 <- 'my_file2.png' comparator <- ImgFileComparator$new(file1, file2) # This comparator has also second explicit creation method that is used # by the library for processing embedded image contents. image1_raw <- 'raw hex vector data' image2_raw <- 'raw hex vector data' comparator <- ImgFileComparator$new(NULL, NULL, image1_raw, image2_raw) } \section{Super classes}{ \code{\link[verifyr2:FileComparator]{verifyr2::FileComparator}} -> \code{\link[verifyr2:BinaryFileComparator]{verifyr2::BinaryFileComparator}} -> \code{ImgFileComparator} } \section{Public fields}{ \if{html}{\out{
}} \describe{ \item{\code{image1_raw}}{extracted raw data for image1.} \item{\code{image2_raw}}{extracted raw data for image2.} } \if{html}{\out{
}} } \section{Methods}{ \subsection{Public methods}{ \itemize{ \item \href{#method-ImgFileComparator-new}{\code{ImgFileComparator$new()}} \item \href{#method-ImgFileComparator-vrf_details_inner}{\code{ImgFileComparator$vrf_details_inner()}} \item \href{#method-ImgFileComparator-vrf_details_inner_from_raw}{\code{ImgFileComparator$vrf_details_inner_from_raw()}} \item \href{#method-ImgFileComparator-vrf_details_inner_from_files}{\code{ImgFileComparator$vrf_details_inner_from_files()}} \item \href{#method-ImgFileComparator-vrf_details_supported}{\code{ImgFileComparator$vrf_details_supported()}} \item \href{#method-ImgFileComparator-clone}{\code{ImgFileComparator$clone()}} } } \if{html}{\out{
Inherited methods

- `verifyr2::FileComparator$vrf_add_debug()`
- `verifyr2::FileComparator$vrf_add_debug_files()`
- `verifyr2::FileComparator$vrf_close_debug()`
- `verifyr2::FileComparator$vrf_details()`
- `verifyr2::FileComparator$vrf_open_debug()`
- `verifyr2::FileComparator$vrf_option_value()`
- `verifyr2::FileComparator$vrf_summary()`
- `verifyr2::BinaryFileComparator$vrf_contents()`
- `verifyr2::BinaryFileComparator$vrf_contents_inner()`
- `verifyr2::BinaryFileComparator$vrf_summary_inner()`

}} \if{html}{\out{

}} \if{html}{\out{}} \if{latex}{\out{\hypertarget{method-ImgFileComparator-new}{}}} \subsection{Method \code{new()}}{ Initialize a ImgFileComparator instance \subsection{Usage}{ \if{html}{\out{
}}\preformatted{ImgFileComparator$new(file1 = NULL, file2 = NULL, raw1 = NULL, raw2 = NULL)}\if{html}{\out{
}} } \subsection{Arguments}{ \if{html}{\out{
}} \describe{ \item{\code{file1}}{First file to compare.} \item{\code{file2}}{Second file to compare.} \item{\code{raw1}}{First image in raw format to compare.} \item{\code{raw2}}{Second image in raw format to compare.} } \if{html}{\out{
}} } } \if{html}{\out{

}} \if{html}{\out{}} \if{latex}{\out{\hypertarget{method-ImgFileComparator-vrf_details_inner}{}}} \subsection{Method \code{vrf_details_inner()}}{ Method for comparing the inner part for the details query. This method can be overwritten by more specialized comparator classes. This method is intended to be called only by the comparator classes in the processing and shouldn't be called directly by the user. \subsection{Usage}{ \if{html}{\out{
}}\preformatted{ImgFileComparator$vrf_details_inner(config, omit)}\if{html}{\out{
}} } \subsection{Arguments}{ \if{html}{\out{
}} \describe{ \item{\code{config}}{configuration values} \item{\code{omit}}{string pattern to omit from the comparison} } \if{html}{\out{
}} } } \if{html}{\out{

}} \if{html}{\out{}} \if{latex}{\out{\hypertarget{method-ImgFileComparator-vrf_details_inner_from_raw}{}}} \subsection{Method \code{vrf_details_inner_from_raw()}}{ Internal method for comparing the earlier populated raw image contents in details and generating the difference highlight image in case differences are found. \subsection{Usage}{ \if{html}{\out{
}}\preformatted{ImgFileComparator$vrf_details_inner_from_raw(config)}\if{html}{\out{
}} } \subsection{Arguments}{ \if{html}{\out{
}} \describe{ \item{\code{config}}{configuration values} } \if{html}{\out{
}} } } \if{html}{\out{

}} \if{html}{\out{}} \if{latex}{\out{\hypertarget{method-ImgFileComparator-vrf_details_inner_from_files}{}}} \subsection{Method \code{vrf_details_inner_from_files()}}{ Method for comparing the inner part for the details query with the file names as the base arguments. This is a part of a group of image processing functions that work with different image abstractions (file, image, raw image). These methods are intended to improve the performance so that best suiting method version can be used depending on what data is available from the earlier method calls to the same comparator instance. \subsection{Usage}{ \if{html}{\out{
}}\preformatted{ImgFileComparator$vrf_details_inner_from_files(config)}\if{html}{\out{
}} } \subsection{Arguments}{ \if{html}{\out{
}} \describe{ \item{\code{config}}{configuration values} } \if{html}{\out{
}} } } \if{html}{\out{

}} \if{html}{\out{}} \if{latex}{\out{\hypertarget{method-ImgFileComparator-vrf_details_supported}{}}} \subsection{Method \code{vrf_details_supported()}}{ Inherited method for indicating whether detailed comparison is available with the current comparator. Returns an empty string if the comparator is is supported, otherwise a string that will be concatenated with the summary string. \subsection{Usage}{ \if{html}{\out{
}}\preformatted{ImgFileComparator$vrf_details_supported(config)}\if{html}{\out{
}} } \subsection{Arguments}{ \if{html}{\out{
}} \describe{ \item{\code{config}}{configuration values} } \if{html}{\out{
}} } } \if{html}{\out{

}} \if{html}{\out{}} \if{latex}{\out{\hypertarget{method-ImgFileComparator-clone}{}}} \subsection{Method \code{clone()}}{ The objects of this class are cloneable with this method. \subsection{Usage}{ \if{html}{\out{
}}\preformatted{ImgFileComparator$clone(deep = FALSE)}\if{html}{\out{
}} } \subsection{Arguments}{ \if{html}{\out{
}} \describe{ \item{\code{deep}}{Whether to make a deep clone.} } \if{html}{\out{
}} } } }

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/list_folder_files.R
\name{list_folder_files}
\alias{list_folder_files}
\title{List files that exist in two folders}
\usage{
list_folder_files(folder1, folder2, pattern = NULL)
}
\arguments{
\item{folder1}{character, giving the the full file path and name of the
folder where first files are stored (required)}

\item{folder2}{character, giving the the full file path and name of the
folder where second new files are stored (required)}

\item{pattern}{character, limit the files to be listed to contain a
specific pattern (optional)}
}
\value{
Returns a tibble, \code{selected_files} with 2 columns \code{file1},
                \code{file2}
}
\description{
\code{list_folder_files} List files that exist in two folders with a specific
name pattern
}
\examples{

folder1 <- paste0(fs::path_package("/extdata/base_files/",
                                   package = "verifyr2"))

folder2 <- paste0(fs::path_package("/extdata/compare_files/",
                                   package = "verifyr2"))

verifyr2::list_folder_files(folder1, folder2, "base")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/run_example.R
\name{run_example}
\alias{run_example}
\title{Call for shiny example where the user can test verifyr2 package functions}
\usage{
run_example(debug = FALSE)
}
\arguments{
\item{debug}{option to override debug configuration (TRUE only)}
}
\description{
\code{verifyr2::run_example} returns simple Shiny App where user can see how
the verifyr2 functions work
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/TxtFileComparator.R
\name{my_finalizer}
\alias{my_finalizer}
\title{Custom finalizer method for diffobj html content finalizing. This method is
used to modify the diff html output so that omitted rows have their own
special styling and gutters.}
\usage{
my_finalizer(x, x.chr, omit)
}
\arguments{
\item{x}{comparator instance used for the comparison that is meant to
be created with the factory method vrf_comparator.}

\item{x.chr}{character text representation of x, typically generated with
the as character}

\item{omit}{all lines containing the omit string will be excluded from the
comparison (detaulf = NULL)}
}
\description{
Custom finalizer method for diffobj html content finalizing. This method is
used to modify the diff html output so that omitted rows have their own
special styling and gutters.
}
\keyword{internal}
